@@ -4,21 +4,22 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  *************************************************************************/
 
 #include "config.h"
-
+#include "../tools/openFile.h"
 #include <algorithm>
 #include <cgraph/cgraph.h>
-#include <cgraph/exit.h>
-#include <ingraphs/ingraphs.h>
-#include <common/pointset.h>
+#include <cgraph/ingraphs.h>
 #include <getopt.h>
 #include <iomanip>
 #include <iostream>
 #include <sstream>
+#include <unordered_map>
+#include <utility>
+#include <util/exit.h>
 #include <vector>
 
 #include <sparse/DotIO.h>
@@ -51,18 +52,7 @@ typedef struct {
 
 static char *fname;
 static FILE *outfile;
-
-static FILE *openFile(const char *name, const char* cmd)
-{
-    FILE *fp;
-
-	fp = fopen(name, "w");
-	if (!fp) {
-		std::cerr << cmd << ": could not open file " << name << " for writing\n";
-		graphviz_exit(-1);
-	}
-	return (fp);
-}
+static char **Files;
 
 static const char use_msg[] =
 "Usage: mingle <options> <file>\n\
@@ -109,92 +99,91 @@ checkG (Agraph_t* g)
 	return 0;
 }
 
-static void init(int argc, char *argv[], opts_t* opts)
-{
+static void init(int argc, char *argv[], opts_t &opts) {
 	int c;
 	char* cmd = argv[0];
 	double s;
 	int i;
 
 	opterr = 0;
-	opts->outer_iter = 4;
-	opts->method = METHOD_INK_AGGLOMERATE;
-	opts->compatibility_method = COMPATIBILITY_FULL;
-	opts->K = -1;
-	opts->fmt = FMT_GV;
-	opts->nneighbors = 10;
-	opts->max_recursion = 100;
-	opts->angle_param = -1;
-	opts->angle = 40.0/180.0*M_PI;
+	opts.outer_iter = 4;
+	opts.method = METHOD_INK_AGGLOMERATE;
+	opts.compatibility_method = COMPATIBILITY_FULL;
+	opts.K = -1;
+	opts.fmt = FMT_GV;
+	opts.nneighbors = 10;
+	opts.max_recursion = 100;
+	opts.angle_param = -1;
+	opts.angle = 40.0/180.0*M_PI;
 
 	while ((c = getopt(argc, argv, ":a:c:i:k:K:m:o:p:r:T:v:?")) != -1) {
 		switch (c) {
 		case 'a':
-			if ((sscanf(optarg,"%lf",&s) > 0) && (s >= 0))
-				opts->angle =  M_PI*s/180;
+			if (sscanf(optarg, "%lf", &s) > 0 && s >= 0)
+				opts.angle =  M_PI * s / 180;
 			else
 				std::cerr << "-a arg " << optarg << " must be positive real - ignored\n";
 			break;
 		case 'c':
-			if ((sscanf(optarg,"%d",&i) > 0) && (0 <= i) && (i <= COMPATIBILITY_FULL))
-				opts->compatibility_method =  i;
+			if (sscanf(optarg, "%d", &i) > 0 && 0 <= i && i <= COMPATIBILITY_FULL)
+				opts.compatibility_method =  i;
 			else
 				std::cerr << "-c arg " << optarg << " must be an integer in [0,"
 					<< COMPATIBILITY_FULL << "] - ignored\n";
 			break;
 		case 'i':
-			if ((sscanf(optarg,"%d",&i) > 0) && (i >= 0))
-				opts->outer_iter =  i;
+			if (sscanf(optarg, "%d", &i) > 0 && i >= 0)
+				opts.outer_iter =  i;
 			else
 				std::cerr << "-i arg " << optarg << " must be a non-negative integer - "
 					"ignored\n";
 			break;
 		case 'k':
-			if ((sscanf(optarg,"%d",&i) > 0) && (i >= 2))
-				opts->nneighbors =  i;
+			if (sscanf(optarg, "%d", &i) > 0 && i >= 2)
+				opts.nneighbors =  i;
 			else
 				std::cerr << "-k arg " << optarg << " must be an integer >= 2 - ignored\n";
 			break;
 		case 'K':
-			if ((sscanf(optarg,"%lf",&s) > 0) && (s > 0))
-				opts->K =  s;
+			if (sscanf(optarg, "%lf", &s) > 0 && s > 0)
+				opts.K =  s;
 			else
 				std::cerr << "-K arg " << optarg << " must be positive real - ignored\n";
 			break;
 		case 'm':
-			if ((sscanf(optarg,"%d",&i) > 0) && (0 <= i) && (i <= METHOD_INK))
-				opts->method =  i;
+			if (sscanf(optarg, "%d", &i) > 0 && 0 <= i && i <= METHOD_INK)
+				opts.method =  i;
 			else
 				std::cerr << "-k arg " << optarg << " must be an integer >= 2 - ignored\n";
 			break;
 		case 'o':
-			outfile = openFile(optarg, cmd);
+			outfile = openFile(cmd, optarg, "w");
 			break;
 		case 'p':
-			if ((sscanf(optarg,"%lf",&s) > 0))
-				opts->angle_param =  s;
+			if (sscanf(optarg, "%lf", &s) > 0)
+				opts.angle_param =  s;
 			else
 				std::cerr << "-p arg " << optarg << " must be real - ignored\n";
 			break;
 		case 'r':
-			if ((sscanf(optarg,"%d",&i) > 0) && (i >= 0))
-				opts->max_recursion =  i;
+			if (sscanf(optarg, "%d", &i) > 0 && i >= 0)
+				opts.max_recursion =  i;
 			else
 				std::cerr << "-r arg " << optarg << " must be a non-negative integer - "
 					"ignored\n";
 			break;
 		case 'T':
 			if (!strcmp(optarg, "gv"))
-				opts->fmt = FMT_GV;
+				opts.fmt = FMT_GV;
 			else if (!strcmp(optarg,"simple"))
-				opts->fmt = FMT_SIMPLE;
+				opts.fmt = FMT_SIMPLE;
 			else
 				std::cerr << "-T arg " << optarg << " must be \"gv\" or \"simple\" - "
 					"ignored\n";
 			break;
 		case 'v':
 			Verbose = 1;
-			if ((sscanf(optarg,"%d",&i) > 0) && (i >= 0))
+			if (sscanf(optarg, "%d", &i) > 0 && i >= 0)
 				Verbose = static_cast<unsigned char>(i);
 			else
 				optind--;
@@ -228,15 +217,15 @@ static void init(int argc, char *argv[], opts_t* opts)
     if (Verbose) {
        std::cerr
          << "Mingle params:\n"
-         << "  outer_iter = " << opts->outer_iter << '\n'
-         << "  method = " << opts->method << '\n'
-         << "  compatibility_method = " << opts->compatibility_method << '\n'
-         << "  K = " << std::setprecision(2) << opts->K << '\n'
-         << "  fmt = " << (opts->fmt ? "simple" : "gv") << '\n'
-         << "  nneighbors = " << opts->nneighbors << '\n'
-         << "  max_recursion = " <<  opts->max_recursion << '\n'
-         << "  angle_param = " << std::setprecision(2) <<  opts->angle_param << '\n'
-         << "  angle = " << std::setprecision(2) << (180 * opts->angle / M_PI) << '\n';
+         << "  outer_iter = " << opts.outer_iter << '\n'
+         << "  method = " << opts.method << '\n'
+         << "  compatibility_method = " << opts.compatibility_method << '\n'
+         << "  K = " << std::setprecision(2) << opts.K << '\n'
+         << "  fmt = " << (opts.fmt ? "simple" : "gv") << '\n'
+         << "  nneighbors = " << opts.nneighbors << '\n'
+         << "  max_recursion = " <<  opts.max_recursion << '\n'
+         << "  angle_param = " << std::setprecision(2) <<  opts.angle_param << '\n'
+         << "  angle = " << std::setprecision(2) << (180 * opts.angle / M_PI) << '\n';
     }
 }
 
@@ -244,26 +233,18 @@ static void init(int argc, char *argv[], opts_t* opts)
  * We have ninterval+1 points. We drop the ninterval-1 internal points, and add 4 points to the first
  * and last intervals, and 3 to the rest, giving the needed 3*ninterval+4 points.
  */
-static void genBundleSpline(pedge edge, std::ostream &os) {
-	int k, j, mm, kk;
-	int dim = edge->dim;
-	double* x = edge->x;
-	double tt1[3]={0.15,0.5,0.85};
-	double tt2[4]={0.15,0.4,0.6,0.85};
-	double t, *tt;
+static void genBundleSpline(const pedge &edge, std::ostream &os) {
+	int k, j;
+	const int dim = edge.dim;
+	const std::vector<double> &x = edge.x;
 
-	for (j = 0; j < edge->npoints; j++){
+	for (j = 0; j < edge.npoints; j++){
 		if (j != 0) {
 			os << ' ';
-			if ((j == 1) || (j == edge->npoints - 1)) {
-				tt = tt2;
-				mm = 4;
-			} else {
-				tt = tt1;
-				mm = 3;
-			}
-			for (kk = 1; kk <= mm; kk++){
-				t = tt[kk-1];
+			const std::vector<double> tt1{0.15, 0.5, 0.85};
+			const std::vector<double> tt2{0.15, 0.4, 0.6, 0.85};
+			const std::vector<double> &tt = (j == 1 || j == edge.npoints - 1) ? tt2 : tt1;
+			for (const double t : tt){
 				for (k = 0; k < dim; k++) {
 					if (k != 0) os << ',';
 					os << std::setprecision(3) << (x[(j-1)*dim+k]*(1-t)+x[j*dim+k]*t);
@@ -271,7 +252,7 @@ static void genBundleSpline(pedge edge, std::ostream &os) {
 				os << ' ';
 			}
 		}
-		if ((j == 0) || (j == edge->npoints - 1)) {
+		if (j == 0 || j == edge.npoints - 1) {
 			for (k = 0; k < dim; k++) {
 				if (k != 0) os << ',';
 				os << std::setprecision(3) << x[j * dim + k];
@@ -280,32 +261,33 @@ static void genBundleSpline(pedge edge, std::ostream &os) {
     }
 }
 
-static void genBundleInfo(pedge edge, std::ostream &os) {
+static void genBundleInfo(const pedge &edge, std::ostream &os) {
 	int k, j;
-	int dim = edge->dim;
-	double* x = edge->x;
+	const int dim = edge.dim;
+	const std::vector<double> &x = edge.x;
 
-	for (j = 0; j < edge->npoints; j++){
+	for (j = 0; j < edge.npoints; j++){
 		if (j != 0) os << ':';
 		for (k = 0; k < dim; k++) {
 			if (k != 0) os << ',';
 			os << std::setprecision(3) << x[j * dim + k];
 		}
 
-		if (j < edge->npoints - 1 && !edge->wgts.empty())  {
-			os << ';' << std::setprecision(3) << edge->wgts[j];
+		if (j < edge.npoints - 1 && !edge.wgts.empty())  {
+			os << ';' << std::setprecision(3) << edge.wgts[j];
 		}
 	}
 }
 
-static void genBundleColors(pedge edge, std::ostream &os, double maxwgt) {
+static void genBundleColors(const pedge &edge, std::ostream &os,
+                            double maxwgt) {
 	int k, j, r, g, b;
 	double len, t, len_total0 = 0;
-	int dim = edge->dim;
-	double* x = edge->x;
-	std::vector<double> lens(edge->npoints);
+	const int dim = edge.dim;
+	const std::vector<double> &x = edge.x;
+	std::vector<double> lens(edge.npoints);
 
-	for (j = 0; j < edge->npoints - 1; j++){
+	for (j = 0; j < edge.npoints - 1; j++){
 		len = 0;
 		for (k = 0; k < dim; k++){
 			len += (x[dim*j+k] - x[dim*(j+1)+k])*(x[dim*j+k] - x[dim*(j+1)+k]);
@@ -313,38 +295,36 @@ static void genBundleColors(pedge edge, std::ostream &os, double maxwgt) {
 		lens[j] = len = sqrt(len/k);
 		len_total0 += len;
 	}
-	for (j = 0; j < edge->npoints - 1; j++){
-		t = edge->wgts[j]/maxwgt;
+	for (j = 0; j < edge.npoints - 1; j++){
+		t = edge.wgts[j] / maxwgt;
 		/* interpolate between red (t = 1) to blue (t = 0) */
 		r = 255*t; g = 0; b = 255*(1-t);
 		if (j != 0) os << ':';
 		os << std::hex << std::setw(2) << std::setfill('0') << '#' << r << g << b
 			<< 85;
-		if (j < edge->npoints-2) {
+		if (j < edge.npoints - 2) {
 			os << ';' << (lens[j] / len_total0);
 		}
 	}
 	os << std::dec << std::setw(0); // reset stream characteristics
 }
 
-static void
-export_dot (FILE* fp, int ne, pedge *edges, Agraph_t* g)
-{
-	Agsym_t* epos = agattr(g, AGEDGE, const_cast<char*>("pos"), "");
-	Agsym_t* esects = agattr(g, AGEDGE, const_cast<char*>("bundle"), "");
+static void export_dot(FILE *fp, int ne, const std::vector<pedge> &edges,
+                       Agraph_t *g) {
+	Agsym_t* epos = agattr_text(g, AGEDGE, const_cast<char*>("pos"), "");
+	Agsym_t* esects = agattr_text(g, AGEDGE, const_cast<char*>("bundle"), "");
 	Agsym_t* eclrs = nullptr;
 	Agnode_t* n;
 	Agedge_t* e;
 	int i, j;
 	double maxwgt = 0;
-	pedge edge;
 
 	  /* figure out max number of bundled original edges in a pedge */
 	for (i = 0; i < ne; i++){
-		edge = edges[i];
-		if (!edge->wgts.empty()) {
-			for (j = 0; j < edge->npoints - 1; j++){
-				maxwgt = MAX(maxwgt, edge->wgts[j]);
+		const pedge &edge = edges[i];
+		if (!edge.wgts.empty()) {
+			for (j = 0; j < edge.npoints - 1; j++){
+				maxwgt = std::max(maxwgt, edge.wgts[j]);
 			}
 		}
 	}
@@ -352,7 +332,7 @@ export_dot (FILE* fp, int ne, pedge *edges, Agraph_t* g)
 	std::ostringstream buf;
 	for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
 		for (e = agfstout (g, n); e; e = agnxtout (g, e)) {
-			edge = edges[ED_idx(e)];
+			const pedge &edge = edges[ED_idx(e)];
 
 			genBundleSpline(edge, buf);
 			agxset(e, epos, buf.str().c_str());
@@ -362,8 +342,8 @@ export_dot (FILE* fp, int ne, pedge *edges, Agraph_t* g)
 			agxset(e, esects, buf.str().c_str());
 			buf.str("");
 
-			if (!edge->wgts.empty()) {
-				if (!eclrs) eclrs = agattr(g, AGEDGE, const_cast<char*>("color"), "");
+			if (!edge.wgts.empty()) {
+				if (!eclrs) eclrs = agattr_text(g, AGEDGE, const_cast<char*>("color"), "");
 				genBundleColors(edge, buf, maxwgt);
 				agxset(e, eclrs, buf.str().c_str());
 				buf.str("");
@@ -373,23 +353,29 @@ export_dot (FILE* fp, int ne, pedge *edges, Agraph_t* g)
 	agwrite(g, fp);
 }
 
-static int
-bundle (Agraph_t* g, opts_t* opts)
-{
+/// a hash derivation function for int pairs
+struct PointHash {
+public:
+  size_t operator()(const std::pair<int, int> &x) const {
+    return std::hash<int>{}(x.first) ^ std::hash<int>{}(x.second);
+  }
+};
+
+using PointMap = std::unordered_map<std::pair<int, int>, int, PointHash>;
+
+static int bundle(Agraph_t *g, const opts_t &opts) {
 	double *x = nullptr;
 	int dim = 2;
     int i;
-	int rv = 0;
 
 	if (checkG(g)) {
-		agerr(AGERR, "Graph %s (%s) contains loops or multiedges\n", agnameof(g),
-		      fname);
+		agerrorf("Graph %s (%s) contains loops or multiedges\n", agnameof(g), fname);
 		return 1;
 	}
     initDotIO(g);
 	SparseMatrix A = SparseMatrix_import_dot(g, dim, &x, FORMAT_CSR);
 	if (!A){
-		agerr (AGERR, "Error: could not convert graph %s (%s) into matrix\n", agnameof(g), fname);
+		agerrorf("Error: could not convert graph %s (%s) into matrix\n", agnameof(g), fname);
 		return 1;
     }
     if (x == nullptr) {
@@ -398,8 +384,8 @@ bundle (Agraph_t* g, opts_t* opts)
     }
 
 	A = SparseMatrix_symmetrize(A, true);
-	if (opts->fmt == FMT_GV) {
-		PointMap* pm = newPM();    /* map from node id pairs to edge index */
+	if (opts.fmt == FMT_GV) {
+		PointMap pm; // map from node id pairs to edge index
 		Agnode_t* n;
 		Agedge_t* e;
 		int idx = 0;
@@ -409,27 +395,25 @@ bundle (Agraph_t* g, opts_t* opts)
 		for (i = 0; i < A->m; i++){
 			for (int j = ia[i]; j < ia[i+1]; j++){
 				if (ja[j] > i){
-					insertPM (pm, i, ja[j], idx++);
+					pm[std::pair(i, ja[j])] = idx++;
 				}
 			}
 		}
 		for (i = 0, n = agfstnode(g); n; n = agnxtnode(g,n)) {
 			setDotNodeID(n, i++);
 		}
-		for (i = 0, n = agfstnode(g); n; n = agnxtnode(g,n)) {
+		for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
 			for (e = agfstout (g, n); e; e = agnxtout (g, e)) {
 				i = getDotNodeID (agtail(e));
 				int j = getDotNodeID (aghead(e));
 				if (j < i) {
 					std::swap(i, j);
 				}
-				int k = insertPM (pm, i, j, -1);
-				assert (k >= 0);
+				const int k = pm[std::pair(i, j)];
 				agbindrec (e, "info", sizeof(etoi_t), true);
 				ED_idx(e) = k;
 			}
 		}
-		freePM (pm);
 	}
 		
 	const int *ia = A->ia;
@@ -452,25 +436,24 @@ bundle (Agraph_t* g, opts_t* opts)
 	if (Verbose)
 		std::cerr << "n = " << A->m << " nz = " << nz << '\n';
 
-	SparseMatrix B = nearest_neighbor_graph(nz, std::min(opts->nneighbors, nz), xx.data());
+	SparseMatrix B = nearest_neighbor_graph(nz, std::min(opts.nneighbors, nz), xx);
 
 	SparseMatrix_delete(A);
 	A = B;
 	free(x);
-	x = xx.data();
 
-	pedge *edges = edge_bundling(A, 2, x, opts->outer_iter, opts->K, opts->method,
-	                             opts->nneighbors, opts->compatibility_method,
-	                             opts->max_recursion, opts->angle_param,
-	                             opts->angle);
-	
-	if (opts->fmt == FMT_GV) {
-	    	export_dot (outfile, A->m, edges, g);
+	std::vector<pedge> edges =
+            edge_bundling(A, 2, xx, opts.outer_iter, opts.K, opts.method,
+                          opts.nneighbors, opts.compatibility_method,
+                          opts.max_recursion, opts.angle_param, opts.angle);
+
+	if (opts.fmt == FMT_GV) {
+	    	export_dot(outfile, A->m, edges, g);
 	}
 	else {
 		pedge_export_gv(outfile, A->m, edges);
 	}
-	return rv;
+	return 0;
 }
 
 int main(int argc, char *argv[])
@@ -481,7 +464,7 @@ int main(int argc, char *argv[])
 	int rv = 0;
 	opts_t opts;
 
-	init(argc, argv, &opts);
+	init(argc, argv, opts);
 	newIngraph(&ig, Files);
 
 	while ((g = nextGraph(&ig)) != 0) {
@@ -491,7 +474,7 @@ int main(int argc, char *argv[])
 		fname = fileName(&ig);
 		if (Verbose)
 		    std::cerr << "Process graph " << agnameof(g) << " in file " << fname << '\n';
-		rv |= bundle (g, &opts);
+		rv |= bundle(g, opts);
 	}
 
 	graphviz_exit(rv);
