@@ -3,13 +3,14 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #pragma once
 
+#include <stdbool.h>
 #include <topfish/hierarchy.h>
 
     typedef struct {
@@ -23,13 +24,12 @@
   // All 5 must be set
 	int width;		/* viewport width */
 	int height;		/* viewport height */
-	int margin;		/* viewport margin */
 	double distortion;	/* default of 1.0 */
     } reposition_t;
 
     void positionAllItems(Hierarchy * hp, focus_t * fs,
 			  reposition_t * parms);
-    Hierarchy *makeHier(int nnodes, int nedges, v_data *, double *,
-			double *, hierparms_t *);
+Hierarchy *makeHier(int nnodes, int nedges, v_data *, double *, double *,
+                    bool dist2_limit);
 
     focus_t *initFocus(int ncnt);
