@@ -3,14 +3,13 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include <assert.h>
 #include "topviewfuncs.h"
-#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
 #include "smyrna_utils.h"
 #include <common/colorprocs.h>
@@ -21,12 +20,13 @@
 #include "selectionfuncs.h"
 #include <common/types.h>
 #include <common/utils.h>
-#include <ctype.h>
 #include <limits.h>
 #include <float.h>
 #include <math.h>
 #include <stdbool.h>
-#include <stddef.h>
+#include <stdlib.h>
+#include <util/alloc.h>
+#include <util/gv_ctype.h>
 
 static xdot *parseXdotwithattrs(void *e)
 {
@@ -100,18 +100,17 @@ static glCompPoint getEdgeTail(Agedge_t *  edge)
 {
     return getPointFromStr(agget(agtail(edge),"pos"));
 }
-static GLfloat getEdgeLength(Agedge_t *  edge)
-{
-    GLfloat rv=0;	
+
+static float getEdgeLength(Agedge_t *edge) {
     glCompPoint A,B;
     A=getEdgeTail(edge);
     B=getEdgeHead(edge);
-    rv=(A.x-B.x)*(A.x-B.x) + (A.y-B.y)*(A.y-B.y) +(A.z-B.z)*(A.z-B.z);
+    float rv = (A.x - B.x) * (A.x - B.x) + (A.y - B.y) * (A.y - B.y) + (A.z - B.z) * (A.z - B.z);
     rv=sqrtf(rv);
     return rv;
 }
-static void glCompColorxlate(glCompColor* c,char* str)
-{
+
+static void glCompColorxlate(glCompColor *c, const char *str) {
         gvcolor_t cl;
 	colorxlate(str, &cl, RGBA_DOUBLE);
 	c->R=cl.u.RGBA[0];
@@ -125,14 +124,11 @@ static void glCompColorxlate(glCompColor* c,char* str)
  */
 static int visible(Agsym_t* attr, void* obj)
 {
-    char* s;
-
     if (attr) {
-	s = agxget (obj, attr);
+	const char *const s = agxget(obj, attr);
 	if (*s) return mapbool(s);
-	else return 1;
     }
-    else return 1;
+    return 1;
 }
 
 static int object_color(void* obj,glCompColor* c)
@@ -140,13 +136,10 @@ static int object_color(void* obj,glCompColor* c)
     gvcolor_t cl;
     Agraph_t* g=view->g[view->activeGraph];
     Agraph_t* objg=agraphof(obj);
-    int return_value = 1;
-    int objType;
     float Alpha = 1;
-    char* bf;
     Agsym_t* vis;
 
-    objType=AGTYPE(obj);
+    const int objType = AGTYPE(obj);
 
     if(objType==AGEDGE) {
 	Alpha=getAttrFloat(g,objg,"defaultedgealpha",1);
@@ -160,9 +153,9 @@ static int object_color(void* obj,glCompColor* c)
     if (!visible(vis,obj))
 	return 0;
 
-    setColorScheme (agget (obj, "colorscheme"));
+    char *previous_color_scheme = setColorScheme(agget (obj, "colorscheme"));
     /*get objects's color attribute */
-    bf = getAttrStr(g,obj,"color",NULL);
+    const char *const bf = getAttrStr(g,obj,"color",NULL);
     if(bf && (*bf)) {
 	colorxlate(bf, &cl, RGBA_DOUBLE);
 	c->R = cl.u.RGBA[0];
@@ -182,10 +175,15 @@ static int object_color(void* obj,glCompColor* c)
 	    c->B = cl.u.RGBA[2];
 	    c->A = cl.u.RGBA[3];
 	}
-	c->A = c->A*Alpha;
+	c->A *= Alpha;
 
     }
-    return return_value;
+
+    char *color_scheme = setColorScheme(previous_color_scheme);
+    free(color_scheme);
+    free(previous_color_scheme);
+
+    return 1;
 }
 
 
@@ -193,38 +191,9 @@ static int object_color(void* obj,glCompColor* c)
 	draws multi edges , single edges
 	this function assumes     glBegin(GL_LINES) has been called 
 */
-static void draw_edge(glCompPoint* posT,glCompPoint* posH, GLfloat length,int deg)
-{
-    double alpha, R, ITERANGLE;
-    double X1, Y1, X2, Y2;
-
-    if (deg) {
-	R = length / 20.0;
-	if ((deg / 2) * 2 != deg)	/*odd */
-	    ITERANGLE = (deg) * 15.00 * -1;
-	else
-	    ITERANGLE = (deg) * 15.00;
-	ITERANGLE = DEG2RAD * ITERANGLE;
-
-	alpha = atan((posH->y - posT->y) / (posH->x - posT->x));
-	if (posT->x > posH->x)
-	    ITERANGLE = 180 * DEG2RAD - ITERANGLE;
-	X1 = R * cos(alpha - ITERANGLE) + posT->x;
-	Y1 = R * sin(alpha - ITERANGLE) + posT->y;
-	X2 = R * cos(alpha - (180 * DEG2RAD - ITERANGLE)) + posH->x;
-	Y2 = R * sin(alpha - (180 * DEG2RAD - ITERANGLE)) + posH->y;
-	glVertex3f(posT->x, posT->y, posT->z);
-	glVertex3f(X1, Y1, posT->z);
-	glVertex3f(X1, Y1, posT->z);
-	glVertex3f(X2, Y2, posH->z);
-	glVertex3f(X2, Y2, posH->z);
-	glVertex3f(posH->x, posH->y, posH->z);
-    } else {
-	glVertex3f(posT->x, posT->y, posT->z);
-	glVertex3f(posH->x, posH->y, posH->z);
-
-    }
-
+static void draw_edge(glCompPoint posT, glCompPoint posH) {
+    glVertex3f(posT.x, posT.y, posT.z);
+    glVertex3f(posH.x, posH.y, posH.z);
 }
 
 static char* labelOf (Agraph_t* g, Agnode_t* v)
@@ -254,7 +223,7 @@ static void renderSelectedNodes(Agraph_t * g)
     Agsym_t* l_color_attr = GG_nodelabelcolor(g);
     glCompColor c;
     int defaultNodeShape;
-    GLfloat nodeSize;
+    float nodeSize;
 
     glCompColorxlate(&c,agxget(g,l_color_attr));
 
@@ -309,13 +278,12 @@ static void renderNodes(Agraph_t * g)
     Agsym_t* pos_attr = GN_pos(g);
     Agsym_t* size_attr = GN_size(g);
     Agsym_t* selected_attr = GN_selected(g);
-    int defaultNodeShape;
-    GLfloat nodeSize;
+    float nodeSize;
     glCompColor c;
     xdot * x;
     int ind;
 
-    defaultNodeShape=getAttrInt(g,g,"defaultnodeshape",0);
+    const int defaultNodeShape=getAttrInt(g, g, "defaultnodeshape", 0);
 
     x=parseXdotwithattrs(g);
     if (x) {
@@ -324,7 +292,7 @@ static void renderNodes(Agraph_t * g)
     }
     for (v = agfstnode(g); v; v = agnxtnode(g, v)) 
     {
-	    if(!object_color(v,&c))
+	    if (!object_color(v, &(glCompColor){0}))
 		continue;
 	    x=parseXdotwithattrs(v);
 	    draw_xdot(x, -0.1);
@@ -355,7 +323,7 @@ static void renderNodes(Agraph_t * g)
 	}
 	glColor4f(c.R,c.G,c.B,c.A);	    
 	pos=getPointFromStr(agxget(v, pos_attr));
-	nodeSize=(GLfloat)l_float(v, size_attr,0);
+	nodeSize = l_float(v, size_attr, 0);
 
 	ND_A(v) = pos;
 
@@ -385,7 +353,6 @@ static void renderSelectedEdges(Agraph_t * g)
     xdot * x;
     glCompPoint posT;	/*Tail position*/
     glCompPoint posH;	/*Head position*/
-    glCompColor c;
     /*xdots tend to be drawn as background shapes,that is why they are being rendered before edges*/
 
     for (v = agfstnode(g); v; v = agnxtnode(g, v)) 
@@ -394,7 +361,7 @@ static void renderSelectedEdges(Agraph_t * g)
 	{
 	    if(!ED_selected(e))
 		continue;
-	    if(!object_color(e,&c))
+	    if (!object_color(e, &(glCompColor){0}))
 		continue;
 
 	    x=parseXdotwithattrs(e);
@@ -412,14 +379,14 @@ static void renderSelectedEdges(Agraph_t * g)
 	    if(!ED_selected(e))
 		continue;
 
-	    if(!object_color(e,&c))
+	    if (!object_color(e, &(glCompColor){0}))
 		continue;
 	    glColor4f(1,0,0,1);	    
 	    posT = ED_posTail(e);
 	    posH = ED_posHead(e);
 	    posT.z +=0.01f;
 	    posH.z +=0.01f;
-	    draw_edge(&posT,&posH,getEdgeLength(e),0);
+	    draw_edge(posT, posH);
 	}
     }
     glEnd();
@@ -430,7 +397,7 @@ static void renderSelectedEdges(Agraph_t * g)
  */
 static char* skipWS (char* p)
 {
-    while (isspace(*p)) p++;
+    while (gv_isspace(*p)) p++;
     return p;
 }
 
@@ -439,7 +406,7 @@ static char* skipWS (char* p)
  */
 static char* skipNWS (char* p)
 {
-    while (*p && !isspace(*p)) p++;
+    while (*p && !gv_isspace(*p)) p++;
     return p;
 }
 
@@ -637,13 +604,13 @@ static void edge_xdot (Agraph_t* g, Agedge_t* e, glCompColor c)
 static void edge_seg (Agraph_t* g, Agedge_t* e, glCompColor c)
 {
     Agsym_t* pos_attr = GN_pos(g);
-    glCompPoint posT;	/*Tail position*/
-    glCompPoint posH;	/*Head position*/
 
     glColor4f(c.R,c.G,c.B,c.A);	   
-    posT=getPointFromStr(agxget(agtail(e), pos_attr));
-    posH=getPointFromStr(agxget(aghead(e), pos_attr));
-    draw_edge(&posT,&posH,getEdgeLength(e),0);
+    // tail position
+    const glCompPoint posT = getPointFromStr(agxget(agtail(e), pos_attr));
+    // head position
+    const glCompPoint posH = getPointFromStr(agxget(aghead(e), pos_attr));
+    draw_edge(posT, posH);
     ED_posTail(e) = posT;
     ED_posHead(e) = posH;
 }
@@ -651,10 +618,9 @@ static void edge_seg (Agraph_t* g, Agedge_t* e, glCompColor c)
 static void edge_spline (Agraph_t* g, Agedge_t* e, glCompColor c)
 {
     Agsym_t* pos_attr_e = GE_pos(g);
-    xdot * x;
 
     glColor4f(c.R,c.G,c.B,c.A);	   
-    x = makeXDotSpline (agxget(e,pos_attr_e));
+    xdot *const x = makeXDotSpline(agxget(e, pos_attr_e));
     if (x) {
 	draw_xdot(x,0);
 	freeXDot (x);
@@ -713,7 +679,6 @@ static void renderEdgeLabels(Agraph_t * g)
     Agsym_t* data_attr = GE_labelattribute(g);
     Agsym_t* l_color_attr = GG_edgelabelcolor(g);
     glCompColor c;
-    GLfloat x,y,z;
 
     glCompColorxlate(&c,agxget(g,l_color_attr));
 
@@ -731,9 +696,9 @@ static void renderEdgeLabels(Agraph_t * g)
 	    posT = ED_posTail(e);
 	    posH = ED_posHead(e);
 	    glColor4f(c.R,c.G,c.B,c.A);
-	    x=posH.x+(posT.x-posH.x)/2;
-	    y=posH.y+(posT.y-posH.y)/2;
-	    z=posH.z+(posT.z-posH.z)/2;
+	    float x = posH.x + (posT.x - posH.x) / 2;
+	    float y = posH.y + (posT.y - posH.y) / 2;
+	    float z = posH.z + (posT.z - posH.z) / 2;
 	    glprintfglut(view->glutfont,x,y,z,agxget(e,data_attr));
 
 	}
@@ -816,11 +781,9 @@ void updateSmGraph(Agraph_t * g,topview* t)
 
     t->Nodecount=0;
     t->maxedgelen=0;
-    t->minedgelen=-1;
 
     t->global_z=0;
-    t->sel.selPoly.cnt=0;
-    t->sel.selPoly.pts=NULL;
+    t->sel.selPoly = (glCompPoly_t){0};
 
     if(!t)
 	return ;
@@ -829,8 +792,6 @@ void updateSmGraph(Agraph_t * g,topview* t)
 	for (e = agfstout(g, v); e; e = agnxtout(g, e)) 
 	{
 	    eLength=getEdgeLength(e);
-	    if((t->minedgelen == -1) || (t->minedgelen > eLength))
-		t->minedgelen=eLength;
 	    if(eLength > t->maxedgelen)
 		t->maxedgelen=eLength;
 	    totalELength += eLength;
@@ -867,8 +828,8 @@ void initSmGraph(Agraph_t * g,topview* rv)
     rv->cache.selnode_id = UINT_MAX;
     rv->cache.edge_id = UINT_MAX;
     rv->cache.seledge_id = UINT_MAX;
-    rv->sel.selectEdges=0;
-    rv->sel.selectNodes=1;
+    rv->sel.selectEdges = false;
+    rv->sel.selectNodes = true;
 
     updateSmGraph(g,rv);
 }
