@@ -183,7 +183,6 @@ static void get_data_dir(void)
 }
 
 static void clear_color_theme(colorschemaset *cs) {
-  free(cs->s);
   *cs = (colorschemaset){0};
 }
 
@@ -279,11 +278,10 @@ void init_viewport(ViewInfo *vi) {
     //default view settings, camera is not active
     vi->panx = 0;
     vi->pany = 0;
-    vi->panz = 0;
 
     vi->zoom = -20;
 
-    vi->mouse.down = 0;
+    vi->mouse.down = false;
     vi->activeGraph = -1;
     vi->Topview = gv_alloc(sizeof(topview));
     vi->Topview->fisheyeParams.fs = 0;
@@ -292,7 +290,7 @@ void init_viewport(ViewInfo *vi) {
     /* init topfish parameters */
     vi->Topview->fisheyeParams.level.num_fine_nodes = 10;
     vi->Topview->fisheyeParams.level.coarsening_rate = 2.5;
-    vi->Topview->fisheyeParams.hier.dist2_limit = 1;
+    vi->Topview->fisheyeParams.dist2_limit = true;
     vi->Topview->fisheyeParams.repos.width = (int)(vi->bdxRight - vi->bdxLeft);
     vi->Topview->fisheyeParams.repos.height = (int)(vi->bdyTop - vi->bdyBottom);
     vi->Topview->fisheyeParams.repos.distortion = 1.0;
@@ -328,7 +326,7 @@ void init_viewport(ViewInfo *vi) {
  */
 static void update_graph_params(Agraph_t * graph)
 {
-    agattr(graph, AGRAPH, "GraphFileName",
+    agattr_text(graph, AGRAPH, "GraphFileName",
 	   view->Topview->Graphdata.GraphFileName);
 }
 
@@ -349,7 +347,7 @@ static Agraph_t *loadGraph(char *filename)
 
     /* If no position info, run layout with -Txdot
      */
-    if (!agattr(g, AGNODE, "pos", NULL)) {
+    if (!agattr_text(g, AGNODE, "pos", NULL)) {
 	g_print("There is no position info in graph %s in %s\n", agnameof(g), filename);
 	agclose (g);
 	return 0;
@@ -373,15 +371,15 @@ int add_graph_to_viewport_from_file(char *fileName)
  */
 void updateRecord (Agraph_t* g)
 {
-    GN_size(g) = agattr (g, AGNODE, "size", 0);
-    GN_visible(g) = agattr (g, AGNODE, "visible", 0);
-    GN_selected(g) = agattr (g, AGNODE, "selected", 0);
-    GN_labelattribute(g) = agattr (g, AGNODE, "nodelabelattribute", 0);
+    GN_size(g) = agattr_text (g, AGNODE, "size", 0);
+    GN_visible(g) = agattr_text (g, AGNODE, "visible", 0);
+    GN_selected(g) = agattr_text (g, AGNODE, "selected", 0);
+    GN_labelattribute(g) = agattr_text (g, AGNODE, "nodelabelattribute", 0);
 
-    GE_pos(g)=agattr(g,AGEDGE,"pos",0);
-    GE_visible(g) = agattr (g, AGEDGE, "visible", 0);
-    GE_selected(g) = agattr (g, AGEDGE, "selected", 0);
-    GE_labelattribute(g) = agattr (g, AGEDGE, "edgelabelattribute", 0);
+    GE_pos(g)=agattr_text(g,AGEDGE,"pos",0);
+    GE_visible(g) = agattr_text (g, AGEDGE, "visible", 0);
+    GE_selected(g) = agattr_text (g, AGEDGE, "selected", 0);
+    GE_labelattribute(g) = agattr_text (g, AGEDGE, "edgelabelattribute", 0);
 }
 
 /* graphRecord:
@@ -397,12 +395,12 @@ graphRecord (Agraph_t* g)
 {
     agbindrec(g, "graphRec", sizeof(graphRec), true);
 
-    GG_nodelabelcolor(g) = agattr (g, AGRAPH, "nodelabelcolor", 0);
-    GG_edgelabelcolor(g) = agattr (g, AGRAPH, "edgelabelcolor", 0);
-    GG_labelattribute(g) = agattr (g, AGRAPH, "nodelabelattribute", 0);
-    GG_elabelattribute(g) = agattr (g, AGRAPH, "edgelabelattribute", 0);
+    GG_nodelabelcolor(g) = agattr_text (g, AGRAPH, "nodelabelcolor", 0);
+    GG_edgelabelcolor(g) = agattr_text (g, AGRAPH, "edgelabelcolor", 0);
+    GG_labelattribute(g) = agattr_text (g, AGRAPH, "nodelabelattribute", 0);
+    GG_elabelattribute(g) = agattr_text (g, AGRAPH, "edgelabelattribute", 0);
 
-    GN_pos(g) = agattr (g, AGNODE, "pos", 0);
+    GN_pos(g) = agattr_text (g, AGNODE, "pos", 0);
 
 
     updateRecord (g);
@@ -553,7 +551,7 @@ void getcolorfromschema(const colorschemaset sc, float l, float maxl,
     float percl = l / maxl;
 
     // For smooth schemas, s[0].perc = 0, so we start with ind=1
-    for (ind = 1; ind + 1 < sc.schemacount; ind++) {
+    for (ind = 1; ind + 1 < sizeof(sc.s) / sizeof(sc.s[0]); ind++) {
 	if (percl < sc.s[ind].perc)
 	    break;
     }
@@ -570,13 +568,12 @@ void getcolorfromschema(const colorschemaset sc, float l, float maxl,
 /* set_color_theme_color:
  * Convert colors as strings to RGB
  */
-static void set_color_theme_color(colorschemaset * sc, char **colorstr)
-{
-    const size_t colorcnt = sc->schemacount;
+static void set_color_theme_color(colorschemaset *sc, const char **colorstr) {
+    const size_t colorcnt = sizeof(sc->s) / sizeof(sc->s[0]);
     gvcolor_t cl;
     float av_perc;
 
-    av_perc = 1.0 / (float) (colorcnt-1);
+    av_perc = 1.0f / (float)(colorcnt - 1);
     for (size_t ind = 0; ind < colorcnt; ind++) {
         colorxlate(colorstr[ind], &cl, RGBA_DOUBLE);
         sc->s[ind].c.R = cl.u.RGBA[0];
@@ -587,28 +584,26 @@ static void set_color_theme_color(colorschemaset * sc, char **colorstr)
     }
 }
 
-static char *deep_blue[] = {
+static const char *deep_blue[SCHEMACOUNT] = {
     "#C8CBED", "#9297D3", "#0000FF", "#2C2E41"
 };
-static char *pastel[] = {
+static const char *pastel[SCHEMACOUNT] = {
     "#EBBE29", "#D58C4A", "#74AE09", "#893C49"
 };
-static char *magma[] = {
+static const char *magma[SCHEMACOUNT] = {
     "#E0061E", "#F0F143", "#95192B", "#EB712F"
 };
-static char *rain_forest[] = {
+static const char *rain_forest[SCHEMACOUNT] = {
     "#1E6A10", "#2ABE0E", "#AEDD39", "#5EE88B"
 };
-#define CSZ(x) (sizeof(x)/sizeof(char*))
 typedef struct {
-    size_t cnt;
-    char **colors;
+    const char **colors;
 } colordata;
-static colordata palette[] = {
-    {CSZ(deep_blue), deep_blue},
-    {CSZ(pastel), pastel},
-    {CSZ(magma), magma},
-    {CSZ(rain_forest), rain_forest},
+static const colordata palette[] = {
+    {deep_blue},
+    {pastel},
+    {magma},
+    {rain_forest},
 };
 #define NUM_SCHEMES (sizeof(palette)/sizeof(colordata))
 
@@ -621,8 +616,6 @@ static colorschemaset create_color_theme(int themeid) {
     colorschemaset s = {0};
     clear_color_theme(&view->colschms);
 
-    s.schemacount = palette[themeid].cnt;
-    s.s = gv_calloc(s.schemacount, sizeof(colorschema));
     set_color_theme_color(&s, palette[themeid].colors);
 
     return s;
