@@ -15,10 +15,11 @@
  *************************************************************************/
 
 #include	<cgraph/cghdr.h>
-#include	<cgraph/streq.h>
-#include	<cgraph/unreachable.h>
-#include	<stddef.h>
 #include	<stdbool.h>
+#include	<stdlib.h>
+#include	<util/alloc.h>
+#include	<util/streq.h>
+#include	<util/unreachable.h>
 
 /*
  * dynamic attributes
@@ -26,9 +27,7 @@
 
 /* to create a graph's data dictionary */
 
-#define MINATTR	4		/* minimum allocation */
-
-static void freesym(void *obj, Dtdisc_t *disc);
+static void freesym(void *obj);
 
 Dtdisc_t AgDataDictDisc = {
     (int) offsetof(Agsym_t, name),	/* use symbol name as key */
@@ -81,14 +80,15 @@ static Dict_t *agdictof(Agraph_t * g, int kind)
     return dict;
 }
 
+/// @param is_html Is `value` an HTML-like string?
 static Agsym_t *agnewsym(Agraph_t * g, const char *name, const char *value,
-                         int id, int kind) {
-    Agsym_t *sym;
-    sym = agalloc(g, sizeof(Agsym_t));
+                         bool is_html, int id, int kind) {
+    Agsym_t *sym = gv_alloc(sizeof(Agsym_t));
     sym->kind = (unsigned char) kind;
     sym->name = agstrdup(g, name);
-    sym->defval = agstrdup(g, value);
+    sym->defval = is_html ? agstrdup_html(g, value) : agstrdup(g, value);
     sym->id = id;
+    sym->owner = g;
     return sym;
 }
 
@@ -98,7 +98,8 @@ static void agcopydict(Dict_t * src, Dict_t * dest, Agraph_t * g, int kind)
 
     assert(dtsize(dest) == 0);
     for (sym = dtfirst(src); sym; sym = dtnext(src, sym)) {
-	newsym = agnewsym(g, sym->name, sym->defval, sym->id, kind);
+	const bool is_html = aghtmlstr(sym->defval);
+	newsym = agnewsym(g, sym->name, sym->defval, is_html, sym->id, kind);
 	newsym->print = sym->print;
 	newsym->fixed = sym->fixed;
 	dtinsert(dest, newsym);
@@ -111,9 +112,9 @@ static Agdatadict_t *agmakedatadict(Agraph_t * g)
     Agdatadict_t *parent_dd, *dd;
 
     dd = agbindrec(g, DataDictName, sizeof(Agdatadict_t), false);
-    dd->dict.n = agdtopen(g, &AgDataDictDisc, Dttree);
-    dd->dict.e = agdtopen(g, &AgDataDictDisc, Dttree);
-    dd->dict.g = agdtopen(g, &AgDataDictDisc, Dttree);
+    dd->dict.n = agdtopen(&AgDataDictDisc, Dttree);
+    dd->dict.e = agdtopen(&AgDataDictDisc, Dttree);
+    dd->dict.g = agdtopen(&AgDataDictDisc, Dttree);
     if ((par = agparent(g))) {
 	parent_dd = agdatadict(par, false);
 	assert(dd != parent_dd);
@@ -136,39 +137,30 @@ static Agdatadict_t *agmakedatadict(Agraph_t * g)
 /* look up an attribute with possible viewpathing */
 static Agsym_t *agdictsym(Dict_t * dict, char *name)
 {
-    Agsym_t key;
-    key.name = name;
+    Agsym_t key = {.name = name};
     return dtsearch(dict, &key);
 }
 
 /* look up attribute in local dictionary with no view pathing */
 static Agsym_t *aglocaldictsym(Dict_t * dict, char *name)
 {
-    Agsym_t *rv;
-    Dict_t *view;
-
-    view = dtview(dict, NULL);
-    rv = agdictsym(dict, name);
+    Dict_t *const view = dtview(dict, NULL);
+    Agsym_t *const rv = agdictsym(dict, name);
     dtview(dict, view);
     return rv;
 }
 
 Agsym_t *agattrsym(void *obj, char *name)
 {
-    Agattr_t *data;
-    Agsym_t *rv;
-
-    data = agattrrec(obj);
+    Agattr_t *const data = agattrrec(obj);
     if (data)
-	rv = agdictsym(data->dict, name);
-    else
-	rv = NULL;
-    return rv;
+	return agdictsym(data->dict, name);
+    return NULL;
 }
 
 /* to create a graph's, node's edge's string attributes */
 
-char *AgDataRecName = "_AG_strdata";
+const char AgDataRecName[] = "_AG_strdata";
 
 static int topdictsize(Agobj_t * obj)
 {
@@ -181,7 +173,6 @@ static int topdictsize(Agobj_t * obj)
 /* g can be either the enclosing graph, or ProtoGraph */
 static Agrec_t *agmakeattrs(Agraph_t * context, void *obj)
 {
-    int sz;
     Agattr_t *rec;
     Agsym_t *sym;
     Dict_t *datadict;
@@ -191,18 +182,20 @@ static Agrec_t *agmakeattrs(Agraph_t * context, void *obj)
     assert(datadict);
     if (rec->dict == NULL) {
 	rec->dict = agdictof(agroot(context), AGTYPE(obj));
-	/* don't malloc(0) */
-	sz = topdictsize(obj);
-	if (sz < MINATTR)
-	    sz = MINATTR;
-	rec->str = agalloc(agraphof(obj), (size_t) sz * sizeof(char *));
+	const int sz = topdictsize(obj);
+	rec->str = gv_calloc((size_t)sz, sizeof(char *));
 	/* doesn't call agxset() so no obj-modified callbacks occur */
-	for (sym = dtfirst(datadict); sym; sym = dtnext(datadict, sym))
-	    rec->str[sym->id] = agstrdup(agraphof(obj), sym->defval);
+	for (sym = dtfirst(datadict); sym; sym = dtnext(datadict, sym)) {
+	    if (aghtmlstr(sym->defval)) {
+	        rec->str[sym->id] = agstrdup_html(agraphof(obj), sym->defval);
+	    } else {
+	        rec->str[sym->id] = agstrdup(agraphof(obj), sym->defval);
+	    }
+	}
     } else {
 	assert(rec->dict == datadict);
     }
-    return (Agrec_t *) rec;
+    return &rec->h;
 }
 
 static void freeattr(Agobj_t * obj, Agattr_t * attr)
@@ -213,18 +206,15 @@ static void freeattr(Agobj_t * obj, Agattr_t * attr)
     g = agraphof(obj);
     sz = topdictsize(obj);
     for (i = 0; i < sz; i++)
-	agstrfree(g, attr->str[i]);
-    agfree(g, attr->str);
+	agstrfree(g, attr->str[i], aghtmlstr(attr->str[i]));
+    free(attr->str);
 }
 
-static void freesym(void *obj, Dtdisc_t *disc) {
-    Agsym_t *sym;
-
-    sym = obj;
-    (void)disc;
-    agstrfree(Ag_G_global, sym->name);
-    agstrfree(Ag_G_global, sym->defval);
-    agfree(Ag_G_global, sym);
+static void freesym(void *obj) {
+    Agsym_t *const sym = obj;
+    agstrfree(sym->owner, sym->name, false);
+    agstrfree(sym->owner, sym->defval, aghtmlstr(sym->defval));
+    free(sym);
 }
 
 Agattr_t *agattrrec(void *obj)
@@ -232,24 +222,22 @@ Agattr_t *agattrrec(void *obj)
   return (Agattr_t *)aggetrec(obj, AgDataRecName, 0);
 }
 
-
-static void addattr(Agraph_t * g, Agobj_t * obj, Agsym_t * sym)
-{
+static void addattr(Agraph_t *g, Agobj_t *obj, void *symbol) {
+    Agsym_t *const sym = symbol;
     Agattr_t *attr = agattrrec(obj);
     assert(attr != NULL);
-    if (sym->id >= MINATTR)
-	attr->str = agrealloc(g, attr->str,
-						     (size_t) sym->id *
-						     sizeof(char *),
-						     ((size_t) sym->id +
-						      1) * sizeof(char *));
-    attr->str[sym->id] = agstrdup(g, sym->defval);
+    attr->str = gv_recalloc(attr->str, (size_t)sym->id, (size_t)sym->id + 1,
+                            sizeof(char *));
+    if (aghtmlstr(sym->defval)) {
+	attr->str[sym->id] = agstrdup_html(g, sym->defval);
+    } else {
+	attr->str[sym->id] = agstrdup(g, sym->defval);
+    }
 }
 
 static Agsym_t *getattr(Agraph_t *g, int kind, char *name) {
   Agsym_t *rv = 0;
-  Dict_t *dict;
-  dict = agdictof(g, kind);
+  Dict_t *dict = agdictof(g, kind);
   if (dict) {
     rv = agdictsym(dict, name); // viewpath up to root
   }
@@ -271,55 +259,57 @@ static void unviewsubgraphsattr(Agraph_t *parent, char *name) {
     if (lsym) {
       continue;
     }
-    lsym = agnewsym(agroot(subg), name, agxget(subg, psym), psym->id, AGRAPH);
+    char *const value = agxget(subg, psym);
+    const bool is_html = aghtmlstr(value);
+    lsym = agnewsym(agroot(subg), name, value, is_html, psym->id, AGRAPH);
     dtinsert(ldict, lsym);
   }
 }
 
-static Agsym_t *setattr(Agraph_t * g, int kind, char *name, const char *value) {
-    Dict_t *ldict, *rdict;
-    Agsym_t *lsym, *psym, *rsym, *rv;
-    Agraph_t *root;
-    Agnode_t *n;
-    Agedge_t *e;
+static int agxset_(void *obj, Agsym_t *sym, const char *value, bool is_html);
+
+/// @param is_html Is `value` an HTML-like string?
+static Agsym_t *setattr(Agraph_t * g, int kind, char *name, const char *value,
+                        bool is_html) {
+    Agsym_t *rv;
 
     assert(value);
-    root = agroot(g);
+    Agraph_t *root = agroot(g);
     agdatadict(g, true);	/* force initialization of string attributes */
-    ldict = agdictof(g, kind);
-    lsym = aglocaldictsym(ldict, name);
+    Dict_t *ldict = agdictof(g, kind);
+    Agsym_t *lsym = aglocaldictsym(ldict, name);
     if (lsym) {			/* update old local definition */
 	if (g != root && streq(name, "layout"))
 	    agwarningf("layout attribute is invalid except on the root graph\n");
         if (kind == AGRAPH) {
 	    unviewsubgraphsattr(g,name);
         }
-	agstrfree(g, lsym->defval);
-	lsym->defval = agstrdup(g, value);
+	agstrfree(g, lsym->defval, aghtmlstr(lsym->defval));
+	lsym->defval = is_html ? agstrdup_html(g, value) : agstrdup(g, value);
 	rv = lsym;
     } else {
-	psym = agdictsym(ldict, name);	/* search with viewpath up to root */
+	Agsym_t *psym = agdictsym(ldict, name); // search with viewpath up to root
 	if (psym) {		/* new local definition */
-	    lsym = agnewsym(g, name, value, psym->id, kind);
+	    lsym = agnewsym(g, name, value, is_html, psym->id, kind);
 	    dtinsert(ldict, lsym);
 	    rv = lsym;
 	} else {		/* new global definition */
-	    rdict = agdictof(root, kind);
-	    rsym = agnewsym(g, name, value, dtsize(rdict), kind);
+	    Dict_t *rdict = agdictof(root, kind);
+	    Agsym_t *rsym = agnewsym(root, name, value, is_html, dtsize(rdict), kind);
 	    dtinsert(rdict, rsym);
 	    switch (kind) {
 	    case AGRAPH:
-		agapply(root, (Agobj_t *)root, (agobjfn_t)addattr, rsym, true);
+		agapply(root, &root->base, addattr, rsym, true);
 		break;
 	    case AGNODE:
-		for (n = agfstnode(root); n; n = agnxtnode(root, n))
-		    addattr(g, (Agobj_t *) n, rsym);
+		for (Agnode_t *n = agfstnode(root); n; n = agnxtnode(root, n))
+		    addattr(g, &n->base, rsym);
 		break;
 	    case AGINEDGE:
 	    case AGOUTEDGE:
-		for (n = agfstnode(root); n; n = agnxtnode(root, n))
-		    for (e = agfstout(root, n); e; e = agnxtout(root, e))
-			addattr(g, (Agobj_t *) e, rsym);
+		for (Agnode_t *n = agfstnode(root); n; n = agnxtnode(root, n))
+		    for (Agedge_t *e = agfstout(root, n); e; e = agnxtout(root, e))
+			addattr(g, &e->base, rsym);
 		break;
 	    default:
 		UNREACHABLE();
@@ -328,7 +318,7 @@ static Agsym_t *setattr(Agraph_t * g, int kind, char *name, const char *value) {
 	}
     }
     if (rv && kind == AGRAPH)
-	agxset(g, rv, value);
+	agxset_(g, rv, value, is_html);
     agmethod_upd(g, g, rv);
     return rv;
 }
@@ -339,21 +329,51 @@ static Agsym_t *setattr(Agraph_t * g, int kind, char *name, const char *value) {
  * when a new attribute is created, existing graphs/nodes/edges
  * receive its default value.
  */
-Agsym_t *agattr(Agraph_t * g, int kind, char *name, const char *value) {
+static Agsym_t *agattr_(Agraph_t *g, int kind, char *name, const char *value,
+                 bool is_html) {
     Agsym_t *rv;
 
-    if (g == 0) {
-	if (ProtoGraph == 0)
-	    ProtoGraph = agopen(0, ProtoDesc, 0);
+    if (g == NULL) {
+	if (ProtoGraph == NULL)
+	    ProtoGraph = agopen(NULL, ProtoDesc, NULL);
 	g = ProtoGraph;
     }
     if (value)
-	rv = setattr(g, kind, name, value);
+	rv = setattr(g, kind, name, value, is_html);
     else
 	rv = getattr(g, kind, name);
     return rv;
 }
 
+Agsym_t *agattr_text(Agraph_t *g, int kind, char *name, const char *value) {
+  return agattr_(g, kind, name, value, false);
+}
+
+Agsym_t *agattr_html(Agraph_t *g, int kind, char *name, const char *value) {
+  return agattr_(g, kind, name, value, true);
+}
+
+Agsym_t *agattr(Agraph_t *g, int kind, char *name, const char *value) {
+  if (g == NULL) {
+    if (ProtoGraph == NULL) {
+      ProtoGraph = agopen(NULL, ProtoDesc, NULL);
+    }
+    g = ProtoGraph;
+  }
+
+  // Is the value we were passed a previously created HTML-like string? We
+  // essentially want to ask `aghtmlstr(value)` but cannot safely because
+  // `value` may not have originated from `agstrdup`/`agstrdup_html`.
+  if (value != NULL) {
+    const char *const alias = agstrbind_html(g, value);
+    if (alias == value && aghtmlstr(alias)) {
+      return agattr_html(g, kind, name, value);
+    }
+  }
+
+  return agattr_text(g, kind, name, value);
+}
+
 Agsym_t *agnxtattr(Agraph_t * g, int kind, Agsym_t * attr)
 {
     Dict_t *d;
@@ -387,9 +407,8 @@ int agraphattr_delete(Agraph_t * g)
     Agdatadict_t *dd;
     Agattr_t *attr;
 
-    Ag_G_global = g;
     if ((attr = agattrrec(g))) {
-	freeattr((Agobj_t *) g, attr);
+	freeattr(&g->base, attr);
 	agdelrec(g, attr->h.name);
     }
 
@@ -416,7 +435,7 @@ void agnodeattr_delete(Agnode_t * n)
     Agattr_t *rec;
 
     if ((rec = agattrrec(n))) {
-	freeattr((Agobj_t *) n, rec);
+	freeattr(&n->base, rec);
 	agdelrec(n, AgDataRecName);
     }
 }
@@ -435,72 +454,78 @@ void agedgeattr_delete(Agedge_t * e)
     Agattr_t *rec;
 
     if ((rec = agattrrec(e))) {
-	freeattr((Agobj_t *) e, rec);
+	freeattr(&e->base, rec);
 	agdelrec(e, AgDataRecName);
     }
 }
 
 char *agget(void *obj, char *name)
 {
-    Agsym_t *sym;
-    Agattr_t *data;
-    char *rv;
-
-    sym = agattrsym(obj, name);
-    if (sym == NULL)
-	rv = 0;			/* note was "", but this provides more info */
-    else {
-	data = agattrrec(obj);
-	rv = data->str[sym->id];
+    Agsym_t *const sym = agattrsym(obj, name);
+    if (sym == NULL) {
+	return NULL; // note was "", but this provides more info
     }
-    return rv;
+    Agattr_t *const data = agattrrec(obj);
+    return data->str[sym->id];
 }
 
 char *agxget(void *obj, Agsym_t * sym)
 {
-    Agattr_t *data;
-    char *rv;
-
-    data = agattrrec(obj);
+    Agattr_t *const data = agattrrec(obj);
     assert(sym->id >= 0 && sym->id < topdictsize(obj));
-    rv = data->str[sym->id];
-    return rv;
+    return data->str[sym->id];
+}
+
+static int agset_(void *obj, char *name, const char *value, bool is_html) {
+    Agsym_t *const sym = agattrsym(obj, name);
+    if (sym == NULL)
+	return FAILURE;
+    if (is_html) {
+	return agxset_html(obj, sym, value);
+    }
+    return agxset_text(obj, sym, value);
 }
 
 int agset(void *obj, char *name, const char *value) {
-    Agsym_t *sym;
-    int rv;
 
-    sym = agattrsym(obj, name);
-    if (sym == NULL)
-	rv = FAILURE;
-    else
-	rv = agxset(obj, sym, value);
-    return rv;
+  // Is the value we were passed a previously created HTML-like string? We
+  // essentially want to ask `aghtmlstr(value)` but cannot safely because
+  // `value` may not have originated from `agstrdup`/`agstrdup_html`.
+  if (value != NULL) {
+    const char *const alias = agstrbind_html(agraphof(obj), value);
+    if (alias == value && aghtmlstr(alias)) {
+      return agset_html(obj, name, value);
+    }
+  }
+
+  return agset_text(obj, name, value);
 }
 
-int agxset(void *obj, Agsym_t * sym, const char *value)
-{
-    Agraph_t *g;
-    Agobj_t *hdr;
-    Agattr_t *data;
+int agset_text(void *obj, char *name, const char *value) {
+  return agset_(obj, name, value, false);
+}
+
+int agset_html(void *obj, char *name, const char *value) {
+  return agset_(obj, name, value, true);
+}
+
+static int agxset_(void *obj, Agsym_t *sym, const char *value, bool is_html) {
     Agsym_t *lsym;
 
-    g = agraphof(obj);
-    hdr = obj;
-    data = agattrrec(hdr);
+    Agraph_t *g = agraphof(obj);
+    Agobj_t *hdr = obj;
+    Agattr_t *data = agattrrec(hdr);
     assert(sym->id >= 0 && sym->id < topdictsize(obj));
-    agstrfree(g, data->str[sym->id]);
-    data->str[sym->id] = agstrdup(g, value);
+    agstrfree(g, data->str[sym->id], aghtmlstr(data->str[sym->id]));
+    data->str[sym->id] = is_html ? agstrdup_html(g, value) : agstrdup(g, value);
     if (hdr->tag.objtype == AGRAPH) {
 	/* also update dict default */
-	Dict_t *dict;
-	dict = agdatadict(g, false)->dict.g;
+	Dict_t *dict = agdatadict(g, false)->dict.g;
 	if ((lsym = aglocaldictsym(dict, sym->name))) {
-	    agstrfree(g, lsym->defval);
-	    lsym->defval = agstrdup(g, value);
+	    agstrfree(g, lsym->defval, aghtmlstr(lsym->defval));
+	    lsym->defval = is_html ? agstrdup_html(g, value) : agstrdup(g, value);
 	} else {
-	    lsym = agnewsym(g, sym->name, value, sym->id, AGTYPE(hdr));
+	    lsym = agnewsym(g, sym->name, value, is_html, sym->id, AGTYPE(hdr));
 	    dtinsert(dict, lsym);
 	}
     }
@@ -508,15 +533,84 @@ int agxset(void *obj, Agsym_t * sym, const char *value)
     return SUCCESS;
 }
 
-int agsafeset(void *obj, char *name, const char *value, const char *def) {
-    Agsym_t *a;
+int agxset(void *obj, Agsym_t *sym, const char *value) {
 
-    a = agattr(agraphof(obj), AGTYPE(obj), name, 0);
+  // Is the value we were passed a previously created HTML-like string? We
+  // essentially want to ask `aghtmlstr(value)` but cannot safely because
+  // `value` may not have originated from `agstrdup`/`agstrdup_html`.
+  if (value != NULL) {
+    const char *const alias = agstrbind_html(agraphof(obj), value);
+    if (alias == value && aghtmlstr(alias)) {
+      return agxset_html(obj, sym, value);
+    }
+  }
+
+  return agxset_text(obj, sym, value);
+}
+
+int agxset_text(void *obj, Agsym_t *sym, const char *value) {
+  return agxset_(obj, sym, value, false);
+}
+
+int agxset_html(void *obj, Agsym_t *sym, const char *value) {
+  return agxset_(obj, sym, value, true);
+}
+
+int agsafeset_text(void *obj, char *name, const char *value, const char *def) {
+    Agsym_t *a = agattr_text(agraphof(obj), AGTYPE(obj), name, NULL);
     if (!a)
-	a = agattr(agraphof(obj), AGTYPE(obj), name, def);
+	a = agattr_text(agraphof(obj), AGTYPE(obj), name, def);
     return agxset(obj, a, value);
 }
 
+int agsafeset_html(void *obj, char *name, const char *value, const char *def) {
+  Agsym_t *a = agattr_html(agraphof(obj), AGTYPE(obj), name, NULL);
+  if (a == NULL) {
+    a = agattr_html(agraphof(obj), AGTYPE(obj), name, def);
+  }
+  return agxset_html(obj, a, value);
+}
+
+int agsafeset(void *obj, char *name, const char *value, const char *def) {
+
+  // find the graph containing the target object
+  Agraph_t *const graph = agraphof(obj);
+
+  // get the existing attribute, if there is one
+  Agsym_t *a = agattr_text(graph, AGTYPE(obj), name, NULL);
+
+  if (a == NULL) {
+    // Is the default we were passed a previously created HTML-like string? We
+    // essentially want to ask `aghtmlstr(def)` but cannot safely because `def`
+    // may not have originated from `agstrdup`/`agstrdup_html`.
+    bool is_def_html = false;
+    if (def != NULL) {
+      const char *const alias = agstrbind_html(graph, def);
+      if (alias == def && aghtmlstr(alias)) {
+        is_def_html = true;
+      }
+    }
+
+    if (is_def_html) {
+      a = agattr_html(graph, AGTYPE(obj), name, def);
+    } else {
+      a = agattr_text(graph, AGTYPE(obj), name, def);
+    }
+  }
+
+  // Is the value we were passed a previously created HTML-like string? We
+  // essentially want to ask `aghtmlstr(value)` but cannot safely because
+  // `value` may not have originated from `agstrdup`/`agstrdup_html`.
+  if (value != NULL) {
+    const char *const alias = agstrbind_html(graph, value);
+    if (alias == value && aghtmlstr(alias)) {
+      return agxset_html(obj, a, value);
+    }
+  }
+
+  return agxset(obj, a, value);
+}
+
 static void agraphattr_init_wrapper(Agraph_t *g, Agobj_t *ignored1,
                                     void *ignored2) {
   (void)ignored1;
@@ -537,7 +631,7 @@ static void init_all_attrs(Agraph_t * g)
     Agedge_t *e;
 
     root = agroot(g);
-    agapply(root, (Agobj_t*)root, agraphattr_init_wrapper, NULL, true);
+    agapply(root, &root->base, agraphattr_init_wrapper, NULL, true);
     for (n = agfstnode(root); n; n = agnxtnode(root, n)) {
 	agnodeattr_init(g, n);
 	for (e = agfstout(root, n); e; e = agnxtout(root, e)) {
@@ -556,7 +650,6 @@ int agcopyattr(void *oldobj, void *newobj)
     Agsym_t *sym;
     Agsym_t *newsym;
     char* val;
-    char* nval;
     int r = 1;
 
     g = agraphof(oldobj);
@@ -568,14 +661,10 @@ int agcopyattr(void *oldobj, void *newobj)
 	if (!newsym)
 	    return 1;
 	val = agxget(oldobj, sym);
-	r = agxset(newobj, newsym, val);
-	/* FIX(?): Each graph has its own string cache, so a whole new refstr is possibly
-	 * allocated. If the original was an html string, make sure the new one is as well.
-	 * If cgraph goes to single string table, this can be removed.
-	 */
-	if (aghtmlstr (val)) {
-	    nval = agxget (newobj, newsym);
-	    agmarkhtmlstr (nval);
+	if (aghtmlstr(val)) {
+	    r = agxset_html(newobj, newsym, val);
+	} else {
+	    r = agxset(newobj, newsym, val);
 	}
     }
     return r;
