@@ -1,9 +1,17 @@
+/**
+ * @file
+ * @brief implements @ref agwrite, @ref agcanon, @ref agstrcanon,
+ * and @ref agcanonStr
+ *
+ * @ingroup cgraph_graph
+ * @ingroup cgraph_core
+ */
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -15,9 +23,9 @@
 #include <stdio.h>		/* need sprintf() */
 #include <ctype.h>
 #include <cgraph/cghdr.h>
-#include <cgraph/gv_ctype.h>
-#include <cgraph/strcasecmp.h>
 #include <inttypes.h>
+#include <util/gv_ctype.h>
+#include <util/strcasecmp.h>
 
 #define EMPTY(s)		(((s) == 0) || (s)[0] == '\0')
 #define MAX(a,b)     ((a)>(b)?(a):(b))
@@ -33,11 +41,21 @@ static int ioput(Agraph_t * g, iochan_t * ofile, char *str)
 
 #define MAX_OUTPUTLINE		128
 #define MIN_OUTPUTLINE		 60
-static int write_body(Agraph_t * g, iochan_t * ofile);
 static int Level;
 static int Max_outputline = MAX_OUTPUTLINE;
 static Agsym_t *Tailport, *Headport;
 
+typedef struct {
+	uint64_t *preorder_number;	// of a graph or subgraph
+	uint64_t *node_last_written;	// postorder number of subg when node was last written
+	uint64_t *edge_last_written;	// postorder number of subg when edge was last written
+} write_info_t;
+
+static int write_body(Agraph_t *g, iochan_t *ofile, write_info_t *wr_info);
+
+static write_info_t before_write(Agraph_t *);
+static void after_write(write_info_t);
+
 static int indent(Agraph_t * g, iochan_t * ofile)
 {
     int i;
@@ -88,8 +106,7 @@ static bool is_escape(const char *str) {
   return false;
 }
 
-/* _agstrcanon:
- * Canonicalize ordinary strings. 
+/* Canonicalize ordinary strings. 
  * Assumes buf is large enough to hold output.
  */
 static char *_agstrcanon(char *arg, char *buf)
@@ -143,6 +160,8 @@ static char *_agstrcanon(char *arg, char *buf)
 	else if (!(gv_isalnum(uc) || uc == '_' || !isascii(uc))) {
 	    needs_quotes = true;
 	    part_of_escape = false;
+	} else {
+	    part_of_escape = false;
 	}
 	*p++ = uc;
 	uc = *s++;
@@ -183,7 +202,7 @@ static char *_agstrcanon(char *arg, char *buf)
     return arg;
 }
 
-/* agcanonhtmlstr:
+/**
  * Canonicalize html strings. 
  */
 static char *agcanonhtmlstr(const char *arg, char *buf)
@@ -192,7 +211,7 @@ static char *agcanonhtmlstr(const char *arg, char *buf)
     return buf;
 }
 
-/*
+/**
  * canonicalize a string for printing.
  * must agree with strings in scan.l
  * Unsafe if buffer is not large enough.
@@ -222,7 +241,7 @@ static char *getoutputbuffer(const char *str)
     return rv;
 }
 
-/*
+/**
  * canonicalize a string for printing.
  * must agree with strings in scan.l
  * Shared static buffer - unsafe.
@@ -235,22 +254,6 @@ char *agcanonStr(char *str)
     return agstrcanon(str, buffer);
 }
 
-/*
- * canonicalize a string for printing.
- * If html is true, use HTML canonicalization.
- * Shared static buffer - unsafe.
- */
-char *agcanon(char *str, int html)
-{
-    char* buf = getoutputbuffer(str);
-    if (buf == NULL)
-	return NULL;
-    if (html)
-	return agcanonhtmlstr(str, buf);
-    else
-	return _agstrcanon(str, buf);
-}
-
 static int _write_canonstr(Agraph_t *g, iochan_t *ofile, char *str, bool chk) {
     if (chk) {
 	str = agcanonStr(str);
@@ -263,18 +266,20 @@ static int _write_canonstr(Agraph_t *g, iochan_t *ofile, char *str, bool chk) {
     return ioput(g, ofile, str);
 }
 
-static int write_canonstr(Agraph_t * g, iochan_t * ofile, char *str)
-{
+/// @param known Is `str` already known to be a reference-counted string?
+static int write_canonstr(Agraph_t *g, iochan_t *ofile, char *str, bool known) {
     char *s;
 
     /* str may not have been allocated by agstrdup, so we first need to turn it
      * into a valid refstr
      */
-    s = agstrdup(g, str);
+    s = known ? str : agstrdup(g, str);
 
     int r = _write_canonstr(g, ofile, s, true);
 
-    agstrfree(g, s);
+    if (!known) {
+	agstrfree(g, s, false);
+    }
     return r;
 }
 
@@ -306,9 +311,9 @@ static int write_dict(Agraph_t * g, iochan_t * ofile, char *name,
 	    CHKRV(ioput(g, ofile, ",\n"));
 	    CHKRV(indent(g, ofile));
 	}
-	CHKRV(write_canonstr(g, ofile, sym->name));
+	CHKRV(write_canonstr(g, ofile, sym->name, true));
 	CHKRV(ioput(g, ofile, "="));
-	CHKRV(write_canonstr(g, ofile, sym->defval));
+	CHKRV(write_canonstr(g, ofile, sym->defval, true));
     }
     if (cnt > 0) {
 	Level--;
@@ -349,8 +354,8 @@ static int write_hdr(Agraph_t *g, iochan_t *ofile, bool top) {
 	    kind = "";
 	if (agisstrict(g))
 	    strict = "strict ";
-	Tailport = agattr(g, AGEDGE, TAILPORT_ID, NULL);
-	Headport = agattr(g, AGEDGE, HEADPORT_ID, NULL);
+	Tailport = agattr_text(g, AGEDGE, TAILPORT_ID, NULL);
+	Headport = agattr_text(g, AGEDGE, HEADPORT_ID, NULL);
     }
     name = agnameof(g);
     sep = " ";
@@ -367,7 +372,7 @@ static int write_hdr(Agraph_t *g, iochan_t *ofile, bool top) {
 	CHKRV(ioput(g, ofile, "graph "));
     }
     if (hasName)
-	CHKRV(write_canonstr(g, ofile, name));
+	CHKRV(write_canonstr(g, ofile, name, false));
     CHKRV(ioput(g, ofile, sep));
     CHKRV(ioput(g, ofile, "{\n"));
     Level++;
@@ -385,16 +390,34 @@ static int write_trl(Agraph_t * g, iochan_t * ofile)
     return 0;
 }
 
+/// is this graph unnamed?
+///
+/// @param g Graph to inspect
+/// @return True if this graph was given no explicit name
+static bool is_anonymous(Agraph_t *g) {
+  assert(g != NULL);
+
+  // handle the common case inline for performance
+  if (AGDISC(g, id) == &AgIdDisc) {
+    // replicate `idprint`
+    const IDTYPE id = AGID(g);
+    if (id % 2 != 0) {
+      return true;
+    }
+    return *(char *)(uintptr_t)id == LOCALNAMEPREFIX;
+  }
+
+  const char *const name = agnameof(g);
+  return name == NULL || name[0] == LOCALNAMEPREFIX;
+}
+
 static bool irrelevant_subgraph(Agraph_t * g)
 {
     int i, n;
     Agattr_t *sdata, *pdata, *rdata;
     Agdatadict_t *dd;
 
-    char *name;
-
-    name = agnameof(g);
-    if (name && name[0] != LOCALNAMEPREFIX)
+    if (!is_anonymous(g))
 	return false;
     if ((sdata = agattrrec(g)) && (pdata = agattrrec(agparent(g)))) {
 	rdata = agattrrec(agroot(g));
@@ -412,37 +435,11 @@ static bool irrelevant_subgraph(Agraph_t * g)
     return true;
 }
 
-static bool node_in_subg(Agraph_t * g, Agnode_t * n)
-{
-    Agraph_t *subg;
-
-    for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
-	if (irrelevant_subgraph(subg))
-	    continue;
-	if (agsubnode(subg, n, 0))
-	    return true;
-    }
-    return false;
-}
-
 static bool has_no_edges(Agraph_t * g, Agnode_t * n)
 {
     return agfstin(g, n) == NULL && agfstout(g, n) == NULL;
 }
 
-static bool has_no_predecessor_below(Agraph_t * g, Agnode_t * n,
-				    uint64_t val)
-{
-    Agedge_t *e;
-
-    if (AGSEQ(n) < val)
-	return false;
-    for (e = agfstin(g, n); e; e = agnxtin(g, e))
-	if (AGSEQ(e->node) < val)
-	    return false;
-    return true;
-}
-
 static bool not_default_attrs(Agraph_t * g, Agnode_t * n)
 {
     Agattr_t *data;
@@ -458,17 +455,16 @@ static bool not_default_attrs(Agraph_t * g, Agnode_t * n)
     return false;
 }
 
-static int write_subgs(Agraph_t * g, iochan_t * ofile)
-{
+static int write_subgs(Agraph_t *g, iochan_t *ofile, write_info_t *wr_info) {
     Agraph_t *subg;
 
     for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
 	if (irrelevant_subgraph(subg)) {
-	    write_subgs(subg, ofile);
+	    write_subgs(subg, ofile, wr_info);
 	}
 	else {
 	    CHKRV(write_hdr(subg, ofile, false));
-	    CHKRV(write_body(subg, ofile));
+	    CHKRV(write_body(subg, ofile, wr_info));
 	    CHKRV(write_trl(subg, ofile));
 	}
     }
@@ -486,7 +482,7 @@ static int write_edge_name(Agedge_t *e, iochan_t *ofile, bool terminate) {
 	    Level++;
 	}
 	CHKRV(ioput(g, ofile, "\t[key="));
-	CHKRV(write_canonstr(g, ofile, p));
+	CHKRV(write_canonstr(g, ofile, p, false));
 	if (terminate)
 	    CHKRV(ioput(g, ofile, "]"));
 	return 1;
@@ -527,9 +523,9 @@ static int write_nondefault_attrs(void *obj, iochan_t * ofile,
 		    CHKRV(ioput(g, ofile, ",\n"));
 		    CHKRV(indent(g, ofile));
 		}
-		CHKRV(write_canonstr(g, ofile, sym->name));
+		CHKRV(write_canonstr(g, ofile, sym->name, true));
 		CHKRV(ioput(g, ofile, "="));
-		CHKRV(write_canonstr(g, ofile, data->str[sym->id]));
+		CHKRV(write_canonstr(g, ofile, data->str[sym->id], true));
 	    }
 	}
     if (cnt > 0) {
@@ -548,7 +544,7 @@ static int write_nodename(Agnode_t * n, iochan_t * ofile)
     name = agnameof(n);
     g = agraphof(n);
     if (name) {
-	CHKRV(write_canonstr(g, ofile, name));
+	CHKRV(write_canonstr(g, ofile, name, false));
     } else {
 	char buf[sizeof("__SUSPECT") + 20];
 	snprintf(buf, sizeof(buf), "_%" PRIu64 "_SUSPECT", AGID(n));	/* could be deadly wrong */
@@ -562,8 +558,8 @@ static int attrs_written(void *obj)
     return AGATTRWF(obj);
 }
 
-static int write_node(Agnode_t * n, iochan_t * ofile, Dict_t * d)
-{
+static int write_node(Agraph_t *subg, Agnode_t *n, iochan_t *ofile, Dict_t *d,
+                      write_info_t *wr_info) {
     Agraph_t *g;
 
     g = agraphof(n);
@@ -571,6 +567,8 @@ static int write_node(Agnode_t * n, iochan_t * ofile, Dict_t * d)
     CHKRV(write_nodename(n, ofile));
     if (!attrs_written(n))
 	CHKRV(write_nondefault_attrs(n, ofile, d));
+    wr_info->node_last_written[AGSEQ(n)] =
+	wr_info->preorder_number[AGSEQ(subg)];
     return ioput(g, ofile, ";\n");
 }
 
@@ -578,13 +576,13 @@ static int write_node(Agnode_t * n, iochan_t * ofile, Dict_t * d)
  * a subgraph or one of its predecessors, and if it is a singleton
  * or has non-default attributes.
  */
-static bool write_node_test(Agraph_t * g, Agnode_t * n,
-			   uint64_t pred_id)
-{
-    if (!node_in_subg(g, n) && has_no_predecessor_below(g, n, pred_id)) {
-	if (has_no_edges(g, n) || not_default_attrs(g, n))
-	    return true;
-    }
+static bool write_node_test(Agraph_t *g, Agnode_t *n, write_info_t *wr_info) {
+    /* test if node was already written in g or a subgraph of g */
+    if (wr_info->node_last_written[AGSEQ(n)] >=
+	wr_info->preorder_number[AGSEQ(g)]) return false;
+
+    if (has_no_edges(g, n) || not_default_attrs(g, n))
+	return true;
     return false;
 }
 
@@ -602,7 +600,7 @@ static int write_port(Agedge_t * e, iochan_t * ofile, Agsym_t * port)
 
     CHKRV(ioput(g, ofile, ":"));
     if (aghtmlstr(val)) {
-	CHKRV(write_canonstr(g, ofile, val));
+	CHKRV(write_canonstr(g, ofile, val, true));
     } else {
 	char *s = strchr(val, ':');
 	if (s) {
@@ -618,21 +616,14 @@ static int write_port(Agedge_t * e, iochan_t * ofile, Agsym_t * port)
     return 0;
 }
 
-static bool write_edge_test(Agraph_t *g, Agedge_t *e) {
-    Agraph_t *subg;
-
-    /* can use agedge() because we subverted the dict compar_f */
-    for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
-	if (irrelevant_subgraph(subg))
-	    continue;
-	if (agsubedge(subg, e, 0))
-	    return false;
-    }
+static bool write_edge_test(Agraph_t *g, Agedge_t *e, write_info_t *wr_info) {
+    if (wr_info->edge_last_written[AGSEQ(e)] >=
+        wr_info->preorder_number[AGSEQ(g)]) return false;
     return true;
 }
 
-static int write_edge(Agedge_t * e, iochan_t * ofile, Dict_t * d)
-{
+static int write_edge(Agraph_t *subg, Agedge_t *e, iochan_t *ofile, Dict_t *d,
+                      write_info_t *wr_info) {
     Agnode_t *t, *h;
     Agraph_t *g;
 
@@ -650,28 +641,29 @@ static int write_edge(Agedge_t * e, iochan_t * ofile, Dict_t * d)
     } else {
 	CHKRV(write_edge_name(e, ofile, true));
     }
+    wr_info->edge_last_written[AGSEQ(e)] =
+	wr_info->preorder_number[AGSEQ(subg)];
     return ioput(g, ofile, ";\n");
 }
 
-static int write_body(Agraph_t * g, iochan_t * ofile)
-{
+static int write_body(Agraph_t *g, iochan_t *ofile, write_info_t *wr_info) {
     Agnode_t *n, *prev;
     Agedge_t *e;
     Agdatadict_t *dd;
 
-    CHKRV(write_subgs(g, ofile));
+    CHKRV(write_subgs(g, ofile, wr_info));
     dd = agdatadict(g, false);
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	if (write_node_test(g, n, AGSEQ(n)))
-	    CHKRV(write_node(n, ofile, dd ? dd->dict.n : 0));
+	if (write_node_test(g, n, wr_info))
+	    CHKRV(write_node(g, n, ofile, dd ? dd->dict.n : 0, wr_info));
 	prev = n;
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-	    if (prev != aghead(e) && write_node_test(g, aghead(e), AGSEQ(n))) {
-		CHKRV(write_node(aghead(e), ofile, dd ? dd->dict.n : 0));
+	    if (prev != aghead(e) && write_node_test(g, aghead(e), wr_info)) {
+		CHKRV(write_node(g, aghead(e), ofile, dd ? dd->dict.n : 0, wr_info));
 		prev = aghead(e);
 	    }
-	    if (write_edge_test(g, e))
-		CHKRV(write_edge(e, ofile, dd ? dd->dict.e : 0));
+	    if (write_edge_test(g, e, wr_info))
+		CHKRV(write_edge(g, e, ofile, dd ? dd->dict.e : 0, wr_info));
 	}
 
 	}
@@ -697,9 +689,7 @@ static void set_attrwf(Agraph_t * g, bool toplevel, bool value)
     }
 }
 
-/* agwrite:
- * Return 0 on success, EOF on failure
- */
+/// Return 0 on success, EOF on failure
 int agwrite(Agraph_t * g, void *ofile)
 {
     char* s;
@@ -707,13 +697,42 @@ int agwrite(Agraph_t * g, void *ofile)
     s = agget(g, "linelength");
     if (s != NULL && gv_isdigit(*s)) {
 	unsigned long len = strtoul(s, NULL, 10);
-	if ((len == 0 || len >= MIN_OUTPUTLINE) && len <= (unsigned long)INT_MAX)
+	if ((len == 0 || len >= MIN_OUTPUTLINE) && len <= INT_MAX)
 	    Max_outputline = (int)len;
     }
-    set_attrwf(g, true, false);
+    write_info_t wr_info = before_write(g);
     CHKRV(write_hdr(g, ofile, true));
-    CHKRV(write_body(g, ofile));
+    CHKRV(write_body(g, ofile, &wr_info));
     CHKRV(write_trl(g, ofile));
+    after_write(wr_info);
     Max_outputline = MAX_OUTPUTLINE;
     return AGDISC(g, io)->flush(ofile);
 }
+
+static uint64_t subgdfs(Agraph_t *g, uint64_t ix, write_info_t *wr_info) {
+    uint64_t ix0 = ix;
+    Agraph_t *subg;
+
+    wr_info->preorder_number[AGSEQ(g)] = ix0;
+    for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
+	ix0 = subgdfs(subg, ix0, wr_info);
+    }
+    return ix0 + 1;
+}
+
+static write_info_t before_write(Agraph_t *g) {
+    write_info_t wr_info = {0};
+    set_attrwf(g, true, false);
+    
+    wr_info.preorder_number = gv_calloc(g->clos->seq[AGRAPH] + 1, sizeof(uint64_t));
+    wr_info.node_last_written = gv_calloc(g->clos->seq[AGNODE] + 1, sizeof(uint64_t));
+    wr_info.edge_last_written = gv_calloc(g->clos->seq[AGEDGE] + 1, sizeof(uint64_t));
+    subgdfs(g, 1, &wr_info);
+    return wr_info;
+}
+
+static void after_write(write_info_t wr_info) {
+  free(wr_info.preorder_number);
+  free(wr_info.node_last_written);
+  free(wr_info.edge_last_written);
+}
