@@ -1,9 +1,12 @@
+/// @file
+/// @brief  [edge splines](https://graphviz.org/docs/attrs/splines/)
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -15,8 +18,13 @@
 
 #include <math.h>
 #include <common/render.h>
-#include <cgraph/unreachable.h>
 #include <stdbool.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/gv_math.h>
+#include <util/unreachable.h>
 
 #ifdef DEBUG
 static int debugleveln(edge_t* e, int i)
@@ -30,22 +38,16 @@ static int debugleveln(edge_t* e, int i)
 
 static void showPoints(pointf ps[], int pn)
 {
-    char buf[BUFSIZ];
-    int newcnt = Show_cnt + pn + 3;
-    int bi, li;
+    int bi;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = strdup ("%% self list");
-    Show_boxes[li++] = strdup ("dbgstart");
+    show_boxes_append(&Show_boxes, gv_strdup("%% self list"));
+    show_boxes_append(&Show_boxes, gv_strdup("dbgstart"));
     for (bi = 0; bi < pn; bi++) {
-	snprintf(buf, sizeof(buf), "%.5g %.5g point", ps[bi].x, ps[bi].y);
-	Show_boxes[li++] = strdup (buf);
+	agxbuf buf = {0};
+	agxbprint(&buf, "%.5g %.5g point", ps[bi].x, ps[bi].y);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_boxes[li++] = strdup ("grestore");
-
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("grestore"));
 }
 #endif
 
@@ -58,11 +60,10 @@ static void showPoints(pointf ps[], int pn)
  */
 static void
 arrow_clip(edge_t * fe, node_t * hn,
-	   pointf * ps, int *startp, int *endp,
+	   pointf * ps, size_t *startp, size_t *endp,
 	   bezier * spl, splineInfo * info)
 {
     edge_t *e;
-    int i, sflag, eflag;
     bool j;
 
     for (e = fe; ED_to_orig(e); e = ED_to_orig(e));
@@ -71,6 +72,7 @@ arrow_clip(edge_t * fe, node_t * hn,
 	j = false;
     else
 	j = info->swapEnds(e);
+    uint32_t sflag, eflag;
     arrow_flags(e, &sflag, &eflag);
     if (info->splineMerge(hn))
 	eflag = ARR_NONE;
@@ -78,7 +80,7 @@ arrow_clip(edge_t * fe, node_t * hn,
 	sflag = ARR_NONE;
     /* swap the two ends */
     if (j) {
-	i = sflag;
+	uint32_t i = sflag;
 	sflag = eflag;
 	eflag = i;
     }
@@ -130,13 +132,13 @@ void bezier_clip(inside_t * inside_context,
     do {
 	opt = pt;
 	t = (high + low) / 2.0;
-	pt = Bezier(sp, 3, t, left, right);
+	pt = Bezier(sp, t, left, right);
 	if (inside(inside_context, pt)) {
 	    *idir = t;
-	} else {
 	    for (i = 0; i < 4; i++)
 		best[i] = seg[i];
 	    found = true;
+	} else {
 	    *odir = t;
 	}
     } while (fabs(opt.x - pt.x) > .5 || fabs(opt.y - pt.y) > .5);
@@ -195,13 +197,11 @@ void shape_clip(node_t * n, pointf curve[4])
     double save_real_size;
     bool left_inside;
     pointf c;
-    inside_t inside_context;
 
     if (ND_shape(n) == NULL || ND_shape(n)->fns->insidefn == NULL)
 	return;
 
-    inside_context.s.n = n;
-    inside_context.s.bp = NULL;
+    inside_t inside_context = {.s = {.n = n}};
     save_real_size = ND_rw(n);
     c.x = curve[0].x - ND_coord(n).x;
     c.y = curve[0].y - ND_coord(n).y;
@@ -213,18 +213,18 @@ void shape_clip(node_t * n, pointf curve[4])
 /* new_spline:
  * Create and attach a new bezier of size sz to the edge d
  */
-bezier *new_spline(edge_t * e, int sz)
-{
+bezier *new_spline(edge_t *e, size_t sz) {
     bezier *rv;
-    while (ED_edge_type(e) != NORMAL)
+    while (ED_to_orig(e) != NULL && ED_edge_type(e) != NORMAL)
 	e = ED_to_orig(e);
     if (ED_spl(e) == NULL)
-	ED_spl(e) = NEW(splines);
-    ED_spl(e)->list = ALLOC(ED_spl(e)->size + 1, ED_spl(e)->list, bezier);
+	ED_spl(e) = gv_alloc(sizeof(splines));
+    ED_spl(e)->list = gv_recalloc(ED_spl(e)->list, ED_spl(e)->size,
+                                  ED_spl(e)->size + 1, sizeof(bezier));
     rv = &(ED_spl(e)->list[ED_spl(e)->size++]);
-    rv->list = N_NEW(sz, pointf);
+    rv->list = gv_calloc(sz, sizeof(pointf));
     rv->size = sz;
-    rv->sflag = rv->eflag = FALSE;
+    rv->sflag = rv->eflag = 0;
     rv->sp.x = rv->sp.y = rv->ep.x = rv->ep.y = 0;
     return rv;
 }
@@ -236,30 +236,28 @@ bezier *new_spline(edge_t * e, int sz)
  * edge.
  */
 void
-clip_and_install(edge_t * fe, node_t * hn, pointf * ps, int pn,
+clip_and_install(edge_t *fe, node_t *hn, pointf *ps, size_t pn,
 		 splineInfo * info)
 {
     pointf p2;
     bezier *newspl;
     node_t *tn;
-    int start, end, i, clipTail, clipHead;
+    int clipTail, clipHead;
+    size_t start, end;
     graph_t *g;
     edge_t *orig;
     boxf *tbox, *hbox;
-    inside_t inside_context;
 
     tn = agtail(fe);
     g = agraphof(tn);
     newspl = new_spline(fe, pn);
 
-    for (orig = fe; ED_edge_type(orig) != NORMAL; orig = ED_to_orig(orig));
+    for (orig = fe; ED_to_orig(orig) != NULL && ED_edge_type(orig) != NORMAL;
+         orig = ED_to_orig(orig));
 
     /* may be a reversed flat edge */
     if (!info->ignoreSwap && ND_rank(tn) == ND_rank(hn) && ND_order(tn) > ND_order(hn)) {
-	node_t *tmp;
-	tmp = hn;
-	hn = tn;
-	tn = tmp;
+	SWAP(&hn, &tn);
     }
     if (tn == agtail(orig)) {
 	clipTail = ED_tail_port(orig).clip;
@@ -276,8 +274,7 @@ clip_and_install(edge_t * fe, node_t * hn, pointf * ps, int pn,
 
     /* spline may be interior to node */
     if(clipTail && ND_shape(tn) && ND_shape(tn)->fns->insidefn) {
-	inside_context.s.n = tn;
-	inside_context.s.bp = tbox;
+	inside_t inside_context = {.s = {.n = tn, .bp = tbox}};
 	for (start = 0; start < pn - 4; start += 3) {
 	    p2.x = ps[start + 3].x - ND_coord(tn).x;
 	    p2.y = ps[start + 3].y - ND_coord(tn).y;
@@ -288,8 +285,7 @@ clip_and_install(edge_t * fe, node_t * hn, pointf * ps, int pn,
     } else
 	start = 0;
     if(clipHead && ND_shape(hn) && ND_shape(hn)->fns->insidefn) {
-	inside_context.s.n = hn;
-	inside_context.s.bp = hbox;
+	inside_t inside_context = {.s = {.n = hn, .bp = hbox}};
 	for (end = pn - 4; end > 0; end -= 3) {
 	    p2.x = ps[end].x - ND_coord(hn).x;
 	    p2.y = ps[end].y - ND_coord(hn).y;
@@ -306,7 +302,7 @@ clip_and_install(edge_t * fe, node_t * hn, pointf * ps, int pn,
 	if (! APPROXEQPT(ps[end], ps[end + 3], MILLIPOINT))
 	    break;
     arrow_clip(fe, hn, ps, &start, &end, newspl, info);
-    for (i = start; i < end + 4; ) {
+    for (size_t i = start; i < end + 4; ) {
 	pointf cp[4];
 	newspl->list[i - start] = ps[i];
 	cp[0] = ps[i];
@@ -819,22 +815,19 @@ else
   return pair_a[tail_i][head_i];
 }
 
-
-static void selfBottom (edge_t* edges[], int ind, int cnt,
-	double sizex, double stepy, splineInfo* sinfo)
-{
+static void selfBottom(edge_t *edges[], size_t ind, size_t cnt, double sizex,
+                       double stepy, splineInfo *sinfo) {
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
-    int i, sgn, point_pair;
+    int sgn, point_pair;
     double hy, ty, stepx, dx, dy, height;
     pointf points[1000];
-    int pointn;
 
     e = edges[ind];
     n = agtail(e);
 
-    stepx = fmax(sizex / 2.0 / cnt, 2.0);
+    stepx = fmax(sizex / 2.0 / (double)cnt, 2.0);
     np = ND_coord(n);
     tp = ED_tail_port(e).p;
     tp.x += np.x;
@@ -857,19 +850,19 @@ static void selfBottom (edge_t* edges[], int ind, int cnt,
     }
     ty = fmin(dy, 3 * (tp.y + dy - np.y));
     hy = fmin(dy, 3 * (hp.y + dy - np.y));
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
         e = edges[ind++];
         dy += stepy;
         ty += stepy;
         hy += stepy;
         dx += sgn * stepx;
-        pointn = 0;
+        size_t pointn = 0;
         points[pointn++] = tp;
-        points[pointn++] = pointfof(tp.x + dx, tp.y - ty / 3);
-        points[pointn++] = pointfof(tp.x + dx, np.y - dy);
-        points[pointn++] = pointfof((tp.x+hp.x)/2, np.y - dy);
-        points[pointn++] = pointfof(hp.x - dx, np.y - dy);
-        points[pointn++] = pointfof(hp.x - dx, hp.y - hy / 3);
+        points[pointn++] = (pointf){tp.x + dx, tp.y - ty / 3};
+        points[pointn++] = (pointf){tp.x + dx, np.y - dy};
+        points[pointn++] = (pointf){(tp.x + hp.x) / 2, np.y - dy};
+        points[pointn++] = (pointf){hp.x - dx, np.y - dy};
+        points[pointn++] = (pointf){hp.x - dx, hp.y - hy / 3};
         points[pointn++] = hp;
         if (ED_label(e)) {
 	if (GD_flip(agraphof(agtail(e)))) {
@@ -879,7 +872,7 @@ static void selfBottom (edge_t* edges[], int ind, int cnt,
     	}
     	ED_label(e)->pos.y = ND_coord(n).y - dy - height / 2.0;
     	ED_label(e)->pos.x = ND_coord(n).x;
-    	ED_label(e)->set = TRUE;
+    	ED_label(e)->set = true;
     	if (height > stepy)
     	    dy += height - stepy;
         }
@@ -891,23 +884,19 @@ static void selfBottom (edge_t* edges[], int ind, int cnt,
     }
 }
 
-
-static void
-selfTop (edge_t* edges[], int ind, int cnt, double sizex, double stepy,
-           splineInfo* sinfo)
-{
-    int i, sgn, point_pair;
+static void selfTop(edge_t *edges[], size_t ind, size_t cnt, double sizex,
+                    double stepy, splineInfo *sinfo) {
+    int sgn, point_pair;
     double hy, ty,  stepx, dx, dy, height;
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
     pointf points[1000];
-    int pointn;
 
     e = edges[ind];
     n = agtail(e);
 
-    stepx = fmax(sizex / 2.0 / cnt, 2.0);
+    stepx = fmax(sizex / 2.0 / (double)cnt, 2.0);
     np = ND_coord(n);
     tp = ED_tail_port(e).p;
     tp.x += np.x;
@@ -967,19 +956,19 @@ selfTop (edge_t* edges[], int ind, int cnt, double sizex, double stepy,
     }
     ty = fmin(dy, 3 * (np.y + dy - tp.y));
     hy = fmin(dy, 3 * (np.y + dy - hp.y));
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
         e = edges[ind++];
         dy += stepy;
         ty += stepy;
         hy += stepy;
         dx += sgn * stepx;
-        pointn = 0;
+        size_t pointn = 0;
         points[pointn++] = tp;
-        points[pointn++] = pointfof(tp.x + dx, tp.y + ty / 3);
-        points[pointn++] = pointfof(tp.x + dx, np.y + dy);
-        points[pointn++] = pointfof((tp.x+hp.x)/2, np.y + dy);
-        points[pointn++] = pointfof(hp.x - dx, np.y + dy);
-        points[pointn++] = pointfof(hp.x - dx, hp.y + hy / 3);
+        points[pointn++] = (pointf){tp.x + dx, tp.y + ty / 3};
+        points[pointn++] = (pointf){tp.x + dx, np.y + dy};
+        points[pointn++] = (pointf){(tp.x + hp.x) / 2, np.y + dy};
+        points[pointn++] = (pointf){hp.x - dx, np.y + dy};
+        points[pointn++] = (pointf){hp.x - dx, hp.y + hy / 3};
         points[pointn++] = hp;
         if (ED_label(e)) {
 	    if (GD_flip(agraphof(agtail(e)))) {
@@ -989,7 +978,7 @@ selfTop (edge_t* edges[], int ind, int cnt, double sizex, double stepy,
 	    }
 	    ED_label(e)->pos.y = ND_coord(n).y + dy + height / 2.0;
 	    ED_label(e)->pos.x = ND_coord(n).x;
-	    ED_label(e)->set = TRUE;
+	    ED_label(e)->set = true;
 	    if (height > stepy)
 		dy += height - stepy;
         }
@@ -999,25 +988,21 @@ selfTop (edge_t* edges[], int ind, int cnt, double sizex, double stepy,
 	    showPoints (points, pointn);
 #endif
     }
-    return;
 }
 
-static void
-selfRight (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
-           splineInfo* sinfo)
-{
-    int i, sgn, point_pair;
+static void selfRight(edge_t *edges[], size_t ind, size_t cnt, double stepx,
+                      double sizey, splineInfo *sinfo) {
+    int sgn, point_pair;
     double hx, tx, stepy, dx, dy, width;
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
     pointf points[1000];
-    int pointn;
 
     e = edges[ind];
     n = agtail(e);
 
-    stepy = fmax(sizey / 2.0 / cnt, 2.0);
+    stepy = fmax(sizey / 2.0 / (double)cnt, 2.0);
     np = ND_coord(n);
     tp = ED_tail_port(e).p;
     tp.x += np.x;
@@ -1041,19 +1026,19 @@ selfRight (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
     }
     tx = fmin(dx, 3 * (np.x + dx - tp.x));
     hx = fmin(dx, 3 * (np.x + dx - hp.x));
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
         e = edges[ind++];
         dx += stepx;
         tx += stepx;
         hx += stepx;
         dy += sgn * stepy;
-        pointn = 0;
+        size_t pointn = 0;
         points[pointn++] = tp;
-        points[pointn++] = pointfof(tp.x + tx / 3, tp.y + dy);
-        points[pointn++] = pointfof(np.x + dx, tp.y + dy);
-        points[pointn++] = pointfof(np.x + dx, (tp.y+hp.y)/2);
-        points[pointn++] = pointfof(np.x + dx, hp.y - dy);
-        points[pointn++] = pointfof(hp.x + hx / 3, hp.y - dy);
+        points[pointn++] = (pointf){tp.x + tx / 3, tp.y + dy};
+        points[pointn++] = (pointf){np.x + dx, tp.y + dy};
+        points[pointn++] = (pointf){np.x + dx, (tp.y + hp.y) / 2};
+        points[pointn++] = (pointf){np.x + dx, hp.y - dy};
+        points[pointn++] = (pointf){hp.x + hx / 3, hp.y - dy};
         points[pointn++] = hp;
         if (ED_label(e)) {
 	    if (GD_flip(agraphof(agtail(e)))) {
@@ -1063,7 +1048,7 @@ selfRight (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
 	    }
 	    ED_label(e)->pos.x = ND_coord(n).x + dx + width / 2.0;
 	    ED_label(e)->pos.y = ND_coord(n).y;
-	    ED_label(e)->set = TRUE;
+	    ED_label(e)->set = true;
 	    if (width > stepx)
 		dx += width - stepx;
         }
@@ -1073,25 +1058,21 @@ selfRight (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
 	    showPoints (points, pointn);
 #endif
     }
-    return;
 }
 
-static void
-selfLeft (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
-          splineInfo* sinfo)
-{
-    int i, sgn,point_pair;
+static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
+                     double sizey, splineInfo *sinfo) {
+    int sgn,point_pair;
     double hx, tx, stepy, dx, dy, width;
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
     pointf points[1000];
-    int pointn;
 
     e = edges[ind];
     n = agtail(e);
 
-    stepy = fmax(sizey / 2.0 / cnt, 2.0);
+    stepy = fmax(sizey / 2.0 / (double)cnt, 2.0);
     np = ND_coord(n);
     tp = ED_tail_port(e).p;
     tp.x += np.x;
@@ -1118,19 +1099,19 @@ selfLeft (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
     }
     tx = fmin(dx, 3 * (tp.x + dx - np.x));
     hx = fmin(dx, 3 * (hp.x + dx - np.x));
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
         e = edges[ind++];
         dx += stepx;
         tx += stepx;
         hx += stepx;
         dy += sgn * stepy;
-        pointn = 0;
+        size_t pointn = 0;
         points[pointn++] = tp;
-        points[pointn++] = pointfof(tp.x - tx / 3, tp.y + dy);
-        points[pointn++] = pointfof(np.x - dx, tp.y + dy);
-        points[pointn++] = pointfof(np.x - dx, (tp.y+hp.y)/2);
-        points[pointn++] = pointfof(np.x - dx, hp.y - dy);
-        points[pointn++] = pointfof(hp.x - hx / 3, hp.y - dy);
+        points[pointn++] = (pointf){tp.x - tx / 3, tp.y + dy};
+        points[pointn++] = (pointf){np.x - dx, tp.y + dy};
+        points[pointn++] = (pointf){np.x - dx, (tp.y + hp.y) / 2};
+        points[pointn++] = (pointf){np.x - dx, hp.y - dy};
+        points[pointn++] = (pointf){hp.x - hx / 3, hp.y - dy};
 
         points[pointn++] = hp;
         if (ED_label(e)) {
@@ -1141,7 +1122,7 @@ selfLeft (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
     	}
     	ED_label(e)->pos.x = ND_coord(n).x - dx - width / 2.0;
     	ED_label(e)->pos.y = ND_coord(n).y;
-    	ED_label(e)->set = TRUE;
+    	ED_label(e)->set = true;
     	if (width > stepx)
     	    dx += width - stepx;
         }
@@ -1188,13 +1169,9 @@ double selfRightSpace(edge_t *e) {
  * FIX: With this bias, labels tend to be placed on top of each other.
  * Perhaps for self-edges, the label should be centered.
  */
-void
-makeSelfEdge(edge_t * edges[], int ind, int cnt, double sizex,
-	     double sizey, splineInfo * sinfo)
-{
-    edge_t *e;
-
-    e = edges[ind];
+void makeSelfEdge(edge_t *edges[], size_t ind, size_t cnt, double sizex,
+                  double sizey, splineInfo * sinfo) {
+    edge_t *e = edges[ind];
 
     /* self edge without ports or
      * self edge with all ports inside, on the right, or at most 1 on top
@@ -1284,22 +1261,21 @@ static pointf
 polylineMidpoint (splines* spl, pointf* pp, pointf* pq)
 {
     bezier bz;
-    int i, j, k;
     double d, dist = 0;
     pointf pf, qf, mf;
 
-    for (i = 0; i < spl->size; i++) {
+    for (size_t i = 0; i < spl->size; i++) {
 	bz = spl->list[i];
-	for (j = 0, k=3; k < bz.size; j+=3,k+=3) {
+	for (size_t j = 0, k = 3; k < bz.size; j += 3, k += 3) {
 	    pf = bz.list[j];
 	    qf = bz.list[k];
 	    dist += DIST(pf, qf);
 	}
     }
     dist /= 2;
-    for (i = 0; i < spl->size; i++) {
+    for (size_t i = 0; i < spl->size; i++) {
 	bz = spl->list[i];
-	for (j = 0, k=3; k < bz.size; j+=3,k+=3) {
+	for (size_t j = 0, k = 3; k < bz.size; j += 3, k += 3) {
 	    pf = bz.list[j];
 	    qf = bz.list[k];
 	    d = DIST(pf,qf);
@@ -1353,7 +1329,6 @@ void addEdgeLabels(edge_t *e) {
 
 #define AGXGET(o,a) agxget(o,a)
 
-/* vladimir */
 /* place_portlabel:
  * place the {head,tail}label (depending on HEAD_P) of edge E
  * N.B. Assume edges are normalized, so tail is at spl->list[0].list[0]
@@ -1367,7 +1342,6 @@ int place_portlabel(edge_t * e, bool head_p)
     bezier *bez;
     double dist, angle;
     pointf c[4], pe, pf;
-    int i;
     char* la;
     char* ld;
 
@@ -1388,9 +1362,9 @@ int place_portlabel(edge_t * e, bool head_p)
 	    pf = bez->list[0];
 	} else {
 	    pe = bez->list[0];
-	    for (i = 0; i < 4; i++)
+	    for (size_t i = 0; i < 4; i++)
 		c[i] = bez->list[i];
-	    pf = Bezier(c, 3, 0.1, NULL, NULL);
+	    pf = Bezier(c, 0.1, NULL, NULL);
 	}
     } else {
 	bez = &spl->list[spl->size - 1];
@@ -1399,9 +1373,9 @@ int place_portlabel(edge_t * e, bool head_p)
 	    pf = bez->list[bez->size - 1];
 	} else {
 	    pe = bez->list[bez->size - 1];
-	    for (i = 0; i < 4; i++)
+	    for (size_t i = 0; i < 4; i++)
 		c[i] = bez->list[bez->size - 4 + i];
-	    pf = Bezier(c, 3, 0.9, NULL, NULL);
+	    pf = Bezier(c, 0.9, NULL, NULL);
 	}
     }
     angle = atan2(pf.y - pe.y, pf.x - pe.x) +
@@ -1421,7 +1395,7 @@ splines *getsplinepoints(edge_t * e)
     for (le = e; !(sp = ED_spl(le)) && ED_edge_type(le) != NORMAL;
 	 le = ED_to_orig(le));
     if (sp == NULL)
-	agerr (AGERR, "getsplinepoints: no spline points available for edge (%s,%s)\n",
+	agerrorf("getsplinepoints: no spline points available for edge (%s,%s)\n",
 	    agnameof(agtail(e)), agnameof(aghead(e)));
     return sp;
 }
