@@ -1,14 +1,11 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 
@@ -20,32 +17,37 @@
  * created and correctly separated.
  */
 
-#include "dot.h"
-#include "aspect.h"
+#include <common/geomprocs.h>
+#include <dotgen/dot.h>
+#include <dotgen/aspect.h>
+#include <math.h>
+#include <stdbool.h>
+#include <stdlib.h>
+#include <util/alloc.h>
+#include <util/gv_math.h>
 
 static int nsiter2(graph_t * g);
 static void create_aux_edges(graph_t * g);
 static void remove_aux_edges(graph_t * g);
 static void set_xcoords(graph_t * g);
 static void set_ycoords(graph_t * g);
-static void set_aspect(graph_t * g, aspect_t* );
+static void set_aspect(graph_t *g);
 static void expand_leaves(graph_t * g);
 static void make_lrvn(graph_t * g);
 static void contain_nodes(graph_t * g);
-static boolean idealsize(graph_t * g, double);
+static bool idealsize(graph_t * g, double);
 
-#if DEBUG > 1
+#if defined(DEBUG) && DEBUG > 1
 static void
 dumpNS (graph_t * g)
 {
     node_t* n = GD_nlist(g);
     elist el;
     edge_t* e;
-    int i;
 
     while (n) {
 	el = ND_out(n);
-	for (i = 0; i < el.size; i++) {
+	for (size_t i = 0; i < el.size; i++) {
 	    e = el.list[i];
 	    fprintf (stderr, "%s(%x) -> ", agnameof(agtail(e)),agtail(e));
 	    fprintf (stderr, "%s(%x) : %d\n", agnameof(aghead(e)), aghead(e),
@@ -59,12 +61,14 @@ dumpNS (graph_t * g)
 static double
 largeMinlen (double l)
 {
-    agerr (AGERR, "Edge length %f larger than maximum %u allowed.\nCheck for overwide node(s).\n", l, USHRT_MAX); 
-    return (double)USHRT_MAX;
+  agerrorf(
+        "Edge length %f larger than maximum %d allowed.\nCheck for overwide "
+        "node(s).\n",
+        l, INT_MAX);
+  return INT_MAX;
 }
 
-/* connectGraph:
- * When source and/or sink nodes are defined, it is possible that
+/* When source and/or sink nodes are defined, it is possible that
  * after the auxiliary edges are added, the graph may still have 2 or
  * 3 components. To fix this, we put trivial constraints connecting the
  * first items of each rank.
@@ -72,7 +76,8 @@ largeMinlen (double l)
 static void
 connectGraph (graph_t* g)
 {
-    int i, j, r, found;
+    int i, j, r;
+    bool found;
     node_t* tp;
     node_t* hp;
     node_t* sn;
@@ -81,14 +86,14 @@ connectGraph (graph_t* g)
 
     for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 	rp = GD_rank(g)+r;
-	found =FALSE;
+	found = false;
         tp = NULL;
 	for (i = 0; i < rp->n; i++) {
 	    tp = rp->v[i];
 	    if (ND_save_out(tp).list) {
         	for (j = 0; (e = ND_save_out(tp).list[j]); j++) {
-		    if ((ND_rank(aghead(e)) > r) || (ND_rank(agtail(e)) > r)) {
-			found = TRUE;
+		    if (ND_rank(aghead(e)) > r || ND_rank(agtail(e)) > r) {
+			found = true;
 			break;
 		    }
         	}
@@ -96,8 +101,8 @@ connectGraph (graph_t* g)
 	    }
 	    if (ND_save_in(tp).list) {
         	for (j = 0; (e = ND_save_in(tp).list[j]); j++) {
-		    if ((ND_rank(agtail(e)) > r) || (ND_rank(aghead(e)) > r)) {
-			found = TRUE;
+		    if (ND_rank(agtail(e)) > r || ND_rank(aghead(e)) > r) {
+			found = true;
 			break;
 		    }
         	}
@@ -117,8 +122,7 @@ connectGraph (graph_t* g)
     }
 }
 
-void dot_position(graph_t * g, aspect_t* asp)
-{
+void dot_position(graph_t *g) {
     if (GD_nlist(g) == NULL)
 	return;			/* ignore empty graph */
     mark_lowclusters(g);	/* we could remove from splines.c now */
@@ -133,9 +137,10 @@ void dot_position(graph_t * g, aspect_t* asp)
 	connectGraph (g);
 	const int rank_result = rank(g, 2, nsiter2(g));
 	assert(rank_result == 0);
+	(void)rank_result;
     }
     set_xcoords(g);
-    set_aspect(g, asp);
+    set_aspect(g);
     remove_aux_edges(g);	/* must come after set_aspect since we now
 				 * use GD_ln and GD_rn for bbox width.
 				 */
@@ -147,26 +152,24 @@ static int nsiter2(graph_t * g)
     char *s;
 
     if ((s = agget(g, "nslimit")))
-	maxiter = atof(s) * agnnodes(g);
+	maxiter = scale_clamp(agnnodes(g), atof(s));
     return maxiter;
 }
 
-static int go(node_t * u, node_t * v)
-{
+static bool go(node_t *u, node_t *v) {
     int i;
     edge_t *e;
 
     if (u == v)
-	return TRUE;
+	return true;
     for (i = 0; (e = ND_out(u).list[i]); i++) {
 	if (go(aghead(e), v))
-	    return TRUE;
+	    return true;
     }
-    return FALSE;
+    return false;
 }
 
-static int canreach(node_t * u, node_t * v)
-{
+static bool canreach(node_t *u, node_t *v) {
     return go(u, v);
 }
 
@@ -174,15 +177,15 @@ edge_t *make_aux_edge(node_t * u, node_t * v, double len, int wt)
 {
     edge_t *e;
 
-    Agedgepair_t* e2 = NEW(Agedgepair_t);
-    AGTYPE(&(e2->in)) = AGINEDGE;
-    AGTYPE(&(e2->out)) = AGOUTEDGE;
-    e2->out.base.data = (Agrec_t*)NEW(Agedgeinfo_t);
-    e = &(e2->out);
+    Agedgepair_t* e2 = gv_alloc(sizeof(Agedgepair_t));
+    AGTYPE(&e2->in) = AGINEDGE;
+    AGTYPE(&e2->out) = AGOUTEDGE;
+    e2->out.base.data = gv_alloc(sizeof(Agedgeinfo_t));
+    e = &e2->out;
 
     agtail(e) = u;
     aghead(e) = v;
-    if (len > USHRT_MAX)
+    if (len > INT_MAX)
 	len = largeMinlen (len);
     ED_minlen(e) = ROUND(len);
     ED_weight(e) = wt;
@@ -207,14 +210,11 @@ static void allocate_aux_edges(graph_t * g)
     }
 }
 
-/* make_LR_constraints:
- */
 static void 
 make_LR_constraints(graph_t * g)
 {
-    int i, j, k;
-    int sw;			/* self width */
-    int m0, m1;
+    int i, j;
+    int m0;
     double width;
     int sep[2];
     int nodesep;      /* separation between nodes on same rank */
@@ -247,8 +247,8 @@ make_LR_constraints(graph_t * g)
                  * Note that this would not only affect left and right
                  * positioning but may also affect interrank spacing.
                  */
-		sw = 0;
-		for (k = 0; (e = ND_other(u).list[k]); k++) {
+		double sw = 0; // self width
+		for (size_t k = 0; (e = ND_other(u).list[k]); k++) {
 		    if (agtail(e) == aghead(e)) {
 			sw += selfRightSpace (e);
 		    }
@@ -263,7 +263,7 @@ make_LR_constraints(graph_t * g)
 	    }
 
 	    /* constraints from labels of flat edges on previous rank */
-	    if ((e = (edge_t*)ND_alg(u))) {
+	    if ((e = ND_alg(u))) {
 		e0 = ND_save_out(u).list[0];
 		e1 = ND_save_out(u).list[1];
 		if (ND_order(aghead(e0)) > ND_order(aghead(e1))) {
@@ -271,21 +271,21 @@ make_LR_constraints(graph_t * g)
 		    e0 = e1;
 		    e1 = ff;
 		}
-		m0 = (ED_minlen(e) * GD_nodesep(g)) / 2;
-		m1 = m0 + ND_rw(aghead(e0)) + ND_lw(agtail(e0));
+		m0 = ED_minlen(e) * GD_nodesep(g) / 2;
+		double m1 = m0 + ND_rw(aghead(e0)) + ND_lw(agtail(e0));
 		/* these guards are needed because the flat edges
 		 * work very poorly with cluster layout */
-		if (canreach(agtail(e0), aghead(e0)) == FALSE)
+		if (!canreach(agtail(e0), aghead(e0)))
 		    make_aux_edge(aghead(e0), agtail(e0), m1,
 			ED_weight(e));
 		m1 = m0 + ND_rw(agtail(e1)) + ND_lw(aghead(e1));
-		if (canreach(aghead(e1), agtail(e1)) == FALSE)
+		if (!canreach(aghead(e1), agtail(e1)))
 		    make_aux_edge(agtail(e1), aghead(e1), m1,
 			ED_weight(e));
 	    }
 
 	    /* position flat edge endpoints */
-	    for (k = 0; k < ND_flat_out(u).size; k++) {
+	    for (size_t k = 0; k < ND_flat_out(u).size; k++) {
 		e = ND_flat_out(u).list[k];
 		if (ND_order(agtail(e)) < ND_order(aghead(e))) {
 		    t0 = agtail(e);
@@ -303,8 +303,6 @@ make_LR_constraints(graph_t * g)
                      * ED_dist contains the largest label width.
                      */
 		    m0 = MAX(m0, width + GD_nodesep(g) + ROUND(ED_dist(e)));
-		    if (m0 > USHRT_MAX)
-			m0 = largeMinlen (m0);
 		    ED_minlen(e0) = MAX(ED_minlen(e0), m0);
 		    ED_weight(e0) = MAX(ED_weight(e0), ED_weight(e));
 		}
@@ -323,7 +321,7 @@ make_LR_constraints(graph_t * g)
     }
 }
 
-/* make_edge_pairs: make virtual edge pairs corresponding to input edges */
+/// make virtual edge pairs corresponding to input edges
 static void make_edge_pairs(graph_t * g)
 {
     int i, m0, m1;
@@ -342,16 +340,6 @@ static void make_edge_pairs(graph_t * g)
 		    m1 = -m0;
 		    m0 = 0;
 		}
-#ifdef NOTDEF
-/* was trying to improve LR balance */
-		if ((ND_save_out(n).size % 2 == 0)
-		    && (i == ND_save_out(n).size / 2 - 1)) {
-		    node_t *u = ND_save_out(n).list[i]->head;
-		    node_t *v = ND_save_out(n).list[i + 1]->head;
-		    double width = ND_rw(u) + ND_lw(v) + GD_nodesep(g);
-		    m0 = width / 2 - 1;
-		}
-#endif
 		make_aux_edge(sn, agtail(e), m0 + 1, ED_weight(e));
 		make_aux_edge(sn, aghead(e), m1 + 1, ED_weight(e));
 		ND_rank(sn) =
@@ -377,22 +365,20 @@ static void contain_clustnodes(graph_t * g)
 	contain_clustnodes(GD_clust(g)[c]);
 }
 
-static int vnode_not_related_to(graph_t * g, node_t * v)
-{
+static bool vnode_not_related_to(graph_t *g, node_t *v) {
     edge_t *e;
 
     if (ND_node_type(v) != VIRTUAL)
-	return FALSE;
+	return false;
     for (e = ND_save_out(v).list[0]; ED_to_orig(e); e = ED_to_orig(e));
     if (agcontains(g, agtail(e)))
-	return FALSE;
+	return false;
     if (agcontains(g, aghead(e)))
-	return FALSE;
-    return TRUE;
+	return false;
+    return true;
 }
 
-/* keepout_othernodes:
- * Guarantee nodes outside the cluster g are placed outside of it.
+/* Guarantee nodes outside the cluster g are placed outside of it.
  * This is done by adding constraints to make sure such nodes have
  * a gap of margin from the left or right bounding box node ln or rn.
  * 
@@ -416,7 +402,7 @@ static void keepout_othernodes(graph_t * g)
 	for (i = ND_order(v) - 1; i >= 0; i--) {
 	    u = GD_rank(dot_root(g))[r].v[i];
 	    /* can't use "is_a_vnode_of" because elists are swapped */
-	    if ((ND_node_type(u) == NORMAL) || vnode_not_related_to(g, u)) {
+	    if (ND_node_type(u) == NORMAL || vnode_not_related_to(g, u)) {
 		make_aux_edge(u, GD_ln(g), margin + ND_rw(u), 0);
 		break;
 	    }
@@ -424,7 +410,7 @@ static void keepout_othernodes(graph_t * g)
 	for (i = ND_order(v) + GD_rank(g)[r].n; i < GD_rank(dot_root(g))[r].n;
 	     i++) {
 	    u = GD_rank(dot_root(g))[r].v[i];
-	    if ((ND_node_type(u) == NORMAL) || vnode_not_related_to(g, u)) {
+	    if (ND_node_type(u) == NORMAL || vnode_not_related_to(g, u)) {
 		make_aux_edge(GD_rn(g), u, margin + ND_lw(u), 0);
 		break;
 	    }
@@ -435,8 +421,7 @@ static void keepout_othernodes(graph_t * g)
 	keepout_othernodes(GD_clust(g)[c]);
 }
 
-/* contain_subclust:
- * Make sure boxes of subclusters of g are offset from the
+/* Make sure boxes of subclusters of g are offset from the
  * box of g. This is done by a constraint between the left and
  * right bounding box nodes ln and rn of g and a subcluster.
  * The gap needs to include any left or right labels.
@@ -459,8 +444,7 @@ static void contain_subclust(graph_t * g)
     }
 }
 
-/* separate_subclust:
- * Guarantee space between subcluster of g.
+/* Guarantee space between subcluster of g.
  * This is done by adding a constraint between the right bbox node rn
  * of the left cluster and the left bbox node ln of the right cluster.
  * This is only done if the two clusters overlap in some rank.
@@ -499,7 +483,7 @@ static void separate_subclust(graph_t * g)
     }
 }
 
-/* pos_clusters: create constraints for:
+/*	create constraints for:
  *	node containment in clusters,
  *	cluster containment in clusters,
  *	separation of sibling clusters.
@@ -525,7 +509,7 @@ static void compress_graph(graph_t * g)
     if (p.x * p.y <= 1)
 	return;
     contain_nodes(g);
-    if (GD_flip(g) == FALSE)
+    if (!GD_flip(g))
 	x = p.x;
     else
 	x = p.y;
@@ -580,9 +564,7 @@ static void remove_aux_edges(graph_t * g)
     ND_prev(GD_nlist(g)) = NULL;
 }
 
-/* set_xcoords:
- * Set x coords of nodes.
- */
+/// Set x coords of nodes.
 static void 
 set_xcoords(graph_t * g)
 {
@@ -599,8 +581,7 @@ set_xcoords(graph_t * g)
     }
 }
 
-/* adjustSimple:
- * Expand cluster height by delta, adding half to top
+/* Expand cluster height by delta, adding half to top
  * and half to bottom. If the bottom expansion exceeds the
  * ht1 of the rank, shift the ranks in the cluster up.
  * If the top expansion, including any shift from the bottom
@@ -610,16 +591,16 @@ set_xcoords(graph_t * g)
  * FIX: There can be excess space between ranks. Not sure where this is
  * coming from but it could be cleaned up.
  */
-static void adjustSimple(graph_t * g, int delta, int margin_total)
-{
-    int r, bottom, deltop, delbottom;
+static void adjustSimple(graph_t *g, double delta, int margin_total) {
+    int r;
+    double deltop;
     graph_t *root = dot_root(g);
     rank_t *rank = GD_rank(root);
     int maxr = GD_maxrank(g);
     int minr = GD_minrank(g);
 
-    bottom = (delta+1) / 2;
-    delbottom = GD_ht1(g) + bottom - (rank[maxr].ht1 - margin_total);
+    const double bottom = (delta + 1) / 2;
+    const double delbottom = GD_ht1(g) + bottom - (rank[maxr].ht1 - margin_total);
     if (delbottom > 0) {
 	for (r = maxr; r >= minr; r--) {
 	    if (rank[r].n > 0)
@@ -635,12 +616,11 @@ static void adjustSimple(graph_t * g, int delta, int margin_total)
 		ND_coord(rank[r].v[0]).y += deltop;
 	}
     }
-    GD_ht2(g) += (delta - bottom);
+    GD_ht2(g) += delta - bottom;
     GD_ht1(g) += bottom;
 }
 
-/* adjustRanks:
- * Recursively adjust ranks to take into account
+/* Recursively adjust ranks to take into account
  * wide cluster labels when rankdir=LR.
  * We divide the extra space between the top and bottom.
  * Adjust the ht1 and ht2 values in the process.
@@ -666,15 +646,15 @@ static void adjustRanks(graph_t * g, int margin_total)
 	graph_t *subg = GD_clust(g)[c];
 	adjustRanks(subg, margin+margin_total);
 	if (GD_maxrank(subg) == GD_maxrank(g))
-	    ht1 = MAX(ht1, GD_ht1(subg) + margin);
+	    ht1 = fmax(ht1, GD_ht1(subg) + margin);
 	if (GD_minrank(subg) == GD_minrank(g))
-	    ht2 = MAX(ht2, GD_ht2(subg) + margin);
+	    ht2 = fmax(ht2, GD_ht2(subg) + margin);
     }
 
     GD_ht1(g) = ht1;
     GD_ht2(g) = ht2;
 
-    if ((g != dot_root(g)) && GD_label(g)) {
+    if (g != dot_root(g) && GD_label(g)) {
 	lht = MAX(GD_border(g)[LEFT_IX].y, GD_border(g)[RIGHT_IX].y);
 	maxr = GD_maxrank(g);
 	minr = GD_minrank(g);
@@ -687,13 +667,12 @@ static void adjustRanks(graph_t * g, int margin_total)
 
     /* update the global ranks */
     if (g != dot_root(g)) {
-	rank[GD_minrank(g)].ht2 = MAX(rank[GD_minrank(g)].ht2, GD_ht2(g));
-	rank[GD_maxrank(g)].ht1 = MAX(rank[GD_maxrank(g)].ht1, GD_ht1(g));
+	rank[GD_minrank(g)].ht2 = fmax(rank[GD_minrank(g)].ht2, GD_ht2(g));
+	rank[GD_maxrank(g)].ht1 = fmax(rank[GD_maxrank(g)].ht1, GD_ht1(g));
     }
 }
 
-/* clust_ht:
- * recursively compute cluster ht requirements.  assumes GD_ht1(subg) and ht2
+/* recursively compute cluster ht requirements.  assumes GD_ht1(subg) and ht2
  * are computed from primitive nodes only.  updates ht1 and ht2 to reflect
  * cluster nesting and labels.  also maintains global rank ht1 and ht2.
  * Return true if some cluster has a label.
@@ -772,7 +751,7 @@ static void set_ycoords(graph_t * g)
 		for (j = 0; (e = ND_other(n).list[j]); j++) {
 		    if (agtail(e) == aghead(e)) {
 			if (ED_label(e))
-			    ht2 = MAX(ht2, ED_label(e)->dimen.y / 2);
+			    ht2 = fmax(ht2, ED_label(e)->dimen.y / 2);
 		    }
 		}
 
@@ -786,9 +765,9 @@ static void set_ycoords(graph_t * g)
 	    if ((clust = ND_clust(n))) {
 		int yoff = (clust == g ? 0 : late_int (clust, G_margin, CL_OFFSET, 0));
 		if (ND_rank(n) == GD_minrank(clust))
-		    GD_ht2(clust) = MAX(GD_ht2(clust), ht2 + yoff);
+		    GD_ht2(clust) = fmax(GD_ht2(clust), ht2 + yoff);
 		if (ND_rank(n) == GD_maxrank(clust))
-		    GD_ht1(clust) = MAX(GD_ht1(clust), ht2 + yoff);
+		    GD_ht1(clust) = fmax(GD_ht1(clust), ht2 + yoff);
 	    }
 	}
     }
@@ -799,19 +778,19 @@ static void set_ycoords(graph_t * g)
     /* make the initial assignment of ycoords to leftmost nodes by ranks */
     maxht = 0;
     r = GD_maxrank(g);
-    (ND_coord(rank[r].v[0])).y = rank[r].ht1;
+    ND_coord(rank[r].v[0]).y = rank[r].ht1;
     while (--r >= GD_minrank(g)) {
 	d0 = rank[r + 1].pht2 + rank[r].pht1 + GD_ranksep(g);	/* prim node sep */
 	d1 = rank[r + 1].ht2 + rank[r].ht1 + CL_OFFSET;	/* cluster sep */
-	delta = MAX(d0, d1);
+	delta = fmax(d0, d1);
 	if (rank[r].n > 0)	/* this may reflect some problem */
-		(ND_coord(rank[r].v[0])).y = (ND_coord(rank[r + 1].v[0])).y + delta;
+		ND_coord(rank[r].v[0]).y = ND_coord(rank[r + 1].v[0]).y + delta;
 #ifdef DEBUG
 	else
 	    fprintf(stderr, "dot set_ycoords: rank %d is empty\n",
 		    rank[r].n);
 #endif
-	maxht = MAX(maxht, delta);
+	maxht = fmax(maxht, delta);
     }
 
     /* If there are cluster labels and the drawing is rotated, we need special processing to
@@ -824,11 +803,11 @@ static void set_ycoords(graph_t * g)
 	if (GD_exact_ranksep(g)) {  /* recompute maxht */
 	    maxht = 0;
 	    r = GD_maxrank(g);
-	    d0 = (ND_coord(rank[r].v[0])).y;
+	    d0 = ND_coord(rank[r].v[0]).y;
 	    while (--r >= GD_minrank(g)) {
-		d1 = (ND_coord(rank[r].v[0])).y;
+		d1 = ND_coord(rank[r].v[0]).y;
 		delta = d1 - d0;
-		maxht = MAX(maxht, delta);
+		maxht = fmax(maxht, delta);
 		d0 = d1;
 	    }
 	}
@@ -838,17 +817,15 @@ static void set_ycoords(graph_t * g)
     if (GD_exact_ranksep(g)) {
 	for (r = GD_maxrank(g) - 1; r >= GD_minrank(g); r--)
 	    if (rank[r].n > 0)	/* this may reflect the same problem :-() */
-			(ND_coord(rank[r].v[0])).y=
-		    (ND_coord(rank[r + 1].v[0])).y + maxht;
+			ND_coord(rank[r].v[0]).y = ND_coord(rank[r + 1].v[0]).y + maxht;
     }
 
     /* copy ycoord assignment from leftmost nodes to others */
     for (n = GD_nlist(g); n; n = ND_next(n))
-	ND_coord(n).y = (ND_coord(rank[ND_rank(n)].v[0])).y;
+	ND_coord(n).y = ND_coord(rank[ND_rank(n)].v[0]).y;
 }
 
-/* dot_compute_bb:
- * Compute bounding box of g.
+/* Compute bounding box of g.
  * The x limits of clusters are given by the x positions of ln and rn.
  * This information is stored in the rank field, since it was calculated
  * using network simplex.
@@ -863,15 +840,15 @@ static void dot_compute_bb(graph_t * g, graph_t * root)
     pointf LL, UR;
 
     if (g == dot_root(g)) {
-	LL.x = (double)(INT_MAX);
-	UR.x = (double)(-INT_MAX);
+	LL.x = INT_MAX;
+	UR.x = -INT_MAX;
 	for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 	    int rnkn = GD_rank(g)[r].n;
 	    if (rnkn == 0)
 		continue;
 	    if ((v = GD_rank(g)[r].v[0]) == NULL)
 		continue;
-	    for (c = 1; (ND_node_type(v) != NORMAL) && c < rnkn; c++)
+	    for (c = 1; ND_node_type(v) != NORMAL && c < rnkn; c++)
 		v = GD_rank(g)[r].v[c];
 	    if (ND_node_type(v) == NORMAL) {
 		x = ND_coord(v).x - ND_lw(v);
@@ -893,8 +870,8 @@ static void dot_compute_bb(graph_t * g, graph_t * root)
 	    UR.x = MAX(UR.x, x);
 	}
     } else {
-	LL.x = (double)(ND_rank(GD_ln(g)));
-	UR.x = (double)(ND_rank(GD_rn(g)));
+	LL.x = ND_rank(GD_ln(g));
+	UR.x = ND_rank(GD_rn(g));
     }
     LL.y = ND_coord(GD_rank(root)[GD_maxrank(g)].v[0]).y - GD_ht1(g);
     UR.y = ND_coord(GD_rank(root)[GD_minrank(g)].v[0]).y + GD_ht2(g);
@@ -910,106 +887,72 @@ static void rec_bb(graph_t * g, graph_t * root)
     dot_compute_bb(g, root);
 }
 
-/* scale_bb:
- * Recursively rescale all bounding boxes using scale factors
+/* Recursively rescale all bounding boxes using scale factors
  * xf and yf. We assume all the bboxes have been computed.
  */
-static void scale_bb(graph_t * g, graph_t * root, double xf, double yf)
-{
+static void scale_bb(graph_t *g, double xf, double yf) {
     int c;
 
     for (c = 1; c <= GD_n_cluster(g); c++)
-	scale_bb(GD_clust(g)[c], root, xf, yf);
+	scale_bb(GD_clust(g)[c], xf, yf);
     GD_bb(g).LL.x *= xf;
     GD_bb(g).LL.y *= yf;
     GD_bb(g).UR.x *= xf;
     GD_bb(g).UR.y *= yf;
 }
 
-/* adjustAspectRatio:
- */
-static void adjustAspectRatio (graph_t* g, aspect_t* asp)
-{
-    double AR = (GD_bb(g).UR.x - GD_bb(g).LL.x)/(GD_bb(g).UR.y - GD_bb(g).LL.y);
-    if (Verbose) {
-        fprintf(stderr, "AR=%0.4lf\t Area= %0.4lf\t", AR, (double)(GD_bb(g).UR.x - GD_bb(g).LL.x)*(GD_bb(g).UR.y - GD_bb(g).LL.y)/10000.0);
-        fprintf(stderr, "Dummy=%d\n", countDummyNodes(g));
-    }
-    if (AR > 1.1*asp->targetAR) {
-      asp->nextIter = (int)(asp->targetAR * (double)(asp->curIterations - asp->prevIterations)/(AR));
-    }
-    else if (AR <= 0.8 * asp->targetAR) {
-      asp->nextIter = -1;
-      if (Verbose)
-        fprintf(stderr, "Going to apply another expansion.\n");
-    }
-    else {
-	asp->nextIter = 0;
-    }
-    if (Verbose)
-        fprintf(stderr, "next#iter=%d\n", asp->nextIter);
-}
-
-/* set_aspect:
- * Set bounding boxes and, if ratio is set, rescale graph.
+/* Set bounding boxes and, if ratio is set, rescale graph.
  * Note that if some dimension shrinks, there may be problems
  * with labels.
  */
-static void set_aspect(graph_t * g, aspect_t* asp)
-{
+static void set_aspect(graph_t *g) {
     double xf = 0.0, yf = 0.0, actual, desired;
     node_t *n;
-    boolean scale_it, filled;
-    point sz;
+    bool filled;
 
     rec_bb(g, g);
-    if ((GD_maxrank(g) > 0) && (GD_drawing(g)->ratio_kind)) {
-	sz.x = GD_bb(g).UR.x - GD_bb(g).LL.x;
-	sz.y = GD_bb(g).UR.y - GD_bb(g).LL.y;	/* normalize */
+    if (GD_maxrank(g) > 0 && GD_drawing(g)->ratio_kind) {
+	pointf sz = sub_pointf(GD_bb(g).UR, GD_bb(g).LL); // normalize
 	if (GD_flip(g)) {
-	    int t = sz.x;
-	    sz.x = sz.y;
-	    sz.y = t;
+	    sz = exch_xyf(sz);
 	}
-	scale_it = TRUE;
+	bool scale_it = true;
 	if (GD_drawing(g)->ratio_kind == R_AUTO)
 	    filled = idealsize(g, .5);
 	else
-	    filled = (GD_drawing(g)->ratio_kind == R_FILL);
+	    filled = GD_drawing(g)->ratio_kind == R_FILL;
 	if (filled) {
 	    /* fill is weird because both X and Y can stretch */
 	    if (GD_drawing(g)->size.x <= 0)
-		scale_it = FALSE;
+		scale_it = false;
 	    else {
-		xf = (double) GD_drawing(g)->size.x / (double) sz.x;
-		yf = (double) GD_drawing(g)->size.y / (double) sz.y;
-		if ((xf < 1.0) || (yf < 1.0)) {
+		xf = GD_drawing(g)->size.x / sz.x;
+		yf = GD_drawing(g)->size.y / sz.y;
+		if (xf < 1.0 || yf < 1.0) {
 		    if (xf < yf) {
-			yf = yf / xf;
+			yf /= xf;
 			xf = 1.0;
 		    } else {
-			xf = xf / yf;
+			xf /= yf;
 			yf = 1.0;
 		    }
 		}
 	    }
 	} else if (GD_drawing(g)->ratio_kind == R_EXPAND) {
 	    if (GD_drawing(g)->size.x <= 0)
-		scale_it = FALSE;
+		scale_it = false;
 	    else {
-		xf = (double) GD_drawing(g)->size.x /
-		    (double) GD_bb(g).UR.x;
-		yf = (double) GD_drawing(g)->size.y /
-		    (double) GD_bb(g).UR.y;
-		if ((xf > 1.0) && (yf > 1.0)) {
-		    double scale = MIN(xf, yf);
+		xf = GD_drawing(g)->size.x / GD_bb(g).UR.x;
+		yf = GD_drawing(g)->size.y / GD_bb(g).UR.y;
+		if (xf > 1.0 && yf > 1.0) {
+		    const double scale = fmin(xf, yf);
 		    xf = yf = scale;
 		} else
-		    scale_it = FALSE;
+		    scale_it = false;
 	    }
 	} else if (GD_drawing(g)->ratio_kind == R_VALUE) {
 	    desired = GD_drawing(g)->ratio;
-	    actual = ((double) sz.y) / ((double) sz.x);
+	    actual = sz.y / sz.x;
 	    if (actual < desired) {
 		yf = desired / actual;
 		xf = 1.0;
@@ -1018,45 +961,18 @@ static void set_aspect(graph_t * g, aspect_t* asp)
 		yf = 1.0;
 	    }
 	} else
-	    scale_it = FALSE;
+	    scale_it = false;
 	if (scale_it) {
 	    if (GD_flip(g)) {
-		double t = xf;
-		xf = yf;
-		yf = t;
+		SWAP(&xf, &yf);
 	    }
 	    for (n = GD_nlist(g); n; n = ND_next(n)) {
-		ND_coord(n).x = ROUND(ND_coord(n).x * xf);
-		ND_coord(n).y = ROUND(ND_coord(n).y * yf);
+		ND_coord(n).x = round(ND_coord(n).x * xf);
+		ND_coord(n).y = round(ND_coord(n).y * yf);
 	    }
-	    scale_bb(g, g, xf, yf);
+	    scale_bb(g, xf, yf);
 	}
     }
-
-    if (asp) adjustAspectRatio (g, asp);
-}
-
-static point resize_leaf(node_t * leaf, point lbound)
-{
-    gv_nodesize(leaf, GD_flip(agraphof(leaf)));
-    ND_coord(leaf).y = lbound.y;
-    ND_coord(leaf).x = lbound.x + ND_lw(leaf);
-    lbound.x = lbound.x + ND_lw(leaf) + ND_rw(leaf) + GD_nodesep(agraphof(leaf));
-    return lbound;
-}
-
-static point place_leaf(graph_t* ing, node_t * leaf, point lbound, int order)
-{
-    node_t *leader;
-    graph_t *g = dot_root(ing);
-
-    leader = UF_find(leaf);
-    if (leaf != leader)
-	fast_nodeapp(leader, leaf);
-    ND_order(leaf) = order;
-    ND_rank(leaf) = ND_rank(leader);
-    GD_rank(g)[ND_rank(leaf)].v[ND_order(leaf)] = leaf;
-    return resize_leaf(leaf, lbound);
 }
 
 /* make space for the leaf nodes of each rank */
@@ -1077,62 +993,27 @@ static void make_leafslots(graph_t * g)
 	}
 	if (j <= GD_rank(g)[r].n)
 	    continue;
-	GD_rank(g)[r].v = ALLOC(j + 1, GD_rank(g)[r].v, node_t *);
+	node_t **new_v = gv_calloc(j + 1, sizeof(node_t*));
 	for (i = GD_rank(g)[r].n - 1; i >= 0; i--) {
 	    v = GD_rank(g)[r].v[i];
-	    GD_rank(g)[r].v[ND_order(v)] = v;
+	    new_v[ND_order(v)] = v;
 	}
 	GD_rank(g)[r].n = j;
-	GD_rank(g)[r].v[j] = NULL;
-    }
-}
-
-static void do_leaves(graph_t * g, node_t * leader)
-{
-    int j;
-    point lbound;
-    node_t *n;
-    edge_t *e;
-
-    if (ND_UF_size(leader) <= 1)
-	return;
-    lbound.x = ND_coord(leader).x - ND_lw(leader);
-    lbound.y = ND_coord(leader).y;
-    lbound = resize_leaf(leader, lbound);
-    if (ND_out(leader).size > 0) {	/* in-edge leaves */
-	n = aghead(ND_out(leader).list[0]);
-	j = ND_order(leader) + 1;
-	for (e = agfstin(g, n); e; e = agnxtin(g, e)) {
-	    edge_t *e1 = AGMKOUT(e);
-	    if ((agtail(e1) != leader) && (UF_find(agtail(e1)) == leader)) {
-		lbound = place_leaf(g, agtail(e1), lbound, j++);
-		unmerge_oneway(e1);
-		elist_append(e1, ND_in(aghead(e1)));
-	    }
-	}
-    } else {			/* out edge leaves */
-	n = agtail(ND_in(leader).list[0]);
-	j = ND_order(leader) + 1;
-	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-	    if ((aghead(e) != leader) && (UF_find(aghead(e)) == leader)) {
-		lbound = place_leaf(g, aghead(e), lbound, j++);
-		unmerge_oneway(e);
-		elist_append(e, ND_out(agtail(e)));
-	    }
-	}
+	new_v[j] = NULL;
+	free(GD_rank(g)[r].v);
+	GD_rank(g)[r].v = new_v;
     }
 }
 
 int ports_eq(edge_t * e, edge_t * f)
 {
-    return ((ED_head_port(e).defined == ED_head_port(f).defined)
-	    && (((ED_head_port(e).p.x == ED_head_port(f).p.x) &&
-		 (ED_head_port(e).p.y == ED_head_port(f).p.y))
-		|| (ED_head_port(e).defined == FALSE))
-	    && (((ED_tail_port(e).p.x == ED_tail_port(f).p.x) &&
-		 (ED_tail_port(e).p.y == ED_tail_port(f).p.y))
-		|| (ED_tail_port(e).defined == FALSE))
-	);
+    return ED_head_port(e).defined == ED_head_port(f).defined
+	    && ((ED_head_port(e).p.x == ED_head_port(f).p.x &&
+		 ED_head_port(e).p.y == ED_head_port(f).p.y)
+		|| !ED_head_port(e).defined)
+	    && ((ED_tail_port(e).p.x == ED_tail_port(f).p.x &&
+		 ED_tail_port(e).p.y == ED_tail_port(f).p.y)
+		|| !ED_tail_port(e).defined);
 }
 
 static void expand_leaves(graph_t * g)
@@ -1143,16 +1024,12 @@ static void expand_leaves(graph_t * g)
 
     make_leafslots(g);
     for (n = GD_nlist(g); n; n = ND_next(n)) {
-	if (ND_inleaf(n))
-	    do_leaves(g, ND_inleaf(n));
-	if (ND_outleaf(n))
-	    do_leaves(g, ND_outleaf(n));
 	if (ND_other(n).list)
 	    for (i = 0; (e = ND_other(n).list[i]); i++) {
 		if ((d = ND_rank(aghead(e)) - ND_rank(aghead(e))) == 0)
 		    continue;
 		f = ED_to_orig(e);
-		if (ports_eq(e, f) == FALSE) {
+		if (!ports_eq(e, f)) {
 		    zapinlist(&(ND_other(n)), e);
 		    if (d == 1)
 			fast_edge(e);
@@ -1163,8 +1040,7 @@ static void expand_leaves(graph_t * g)
     }
 }
 
-/* make_lrvn:
- * Add left and right slacknodes to a cluster which
+/* Add left and right slacknodes to a cluster which
  * are used in the LP to constrain nodes not in g but
  * sharing its ranks to be to the left or right of g
  * by a specified amount.
@@ -1188,7 +1064,7 @@ static void make_lrvn(graph_t * g)
     rn = virtual_node(dot_root(g));
     ND_node_type(rn) = SLACKNODE;
 
-    if (GD_label(g) && (g != dot_root(g)) && !GD_flip(agroot(g))) {
+    if (GD_label(g) && g != dot_root(g) && !GD_flip(agroot(g))) {
 	int w = MAX(GD_border(g)[BOTTOM_IX].x, GD_border(g)[TOP_IX].x);
 	make_aux_edge(ln, rn, w, 0);
     }
@@ -1197,8 +1073,7 @@ static void make_lrvn(graph_t * g)
     GD_rn(g) = rn;
 }
 
-/* contain_nodes: 
- * make left and right bounding box virtual nodes ln and rn
+/* make left and right bounding box virtual nodes ln and rn
  * constrain interior nodes
  */
 static void contain_nodes(graph_t * g)
@@ -1215,7 +1090,7 @@ static void contain_nodes(graph_t * g)
 	    continue;
 	v = GD_rank(g)[r].v[0];
 	if (v == NULL) {
-	    agerr(AGERR, "contain_nodes clust %s rank %d missing node\n",
+	    agerrorf("contain_nodes clust %s rank %d missing node\n",
 		  agnameof(g), r);
 	    continue;
 	}
@@ -1227,11 +1102,10 @@ static void contain_nodes(graph_t * g)
     }
 }
 
-/* idealsize:
- * set g->drawing->size to a reasonable default.
+/* set g->drawing->size to a reasonable default.
  * returns a boolean to indicate if drawing is to
  * be scaled and filled */
-static boolean idealsize(graph_t * g, double minallowed)
+static bool idealsize(graph_t * g, double minallowed)
 {
     double xf, yf, f, R;
     pointf b, relpage, margin;
@@ -1239,7 +1113,7 @@ static boolean idealsize(graph_t * g, double minallowed)
     /* try for one page */
     relpage = GD_drawing(g)->page;
     if (relpage.x < 0.001 || relpage.y < 0.001)
-	return FALSE;		/* no page was specified */
+	return false;		/* no page was specified */
     margin = GD_drawing(g)->margin;
     relpage = sub_pointf(relpage, margin);
     relpage = sub_pointf(relpage, margin);
@@ -1247,17 +1121,17 @@ static boolean idealsize(graph_t * g, double minallowed)
     b.y = GD_bb(g).UR.y;
     xf = relpage.x / b.x;
     yf = relpage.y / b.y;
-    if ((xf >= 1.0) && (yf >= 1.0))
-	return FALSE;		/* fits on one page */
+    if (xf >= 1.0 && yf >= 1.0)
+	return false;		/* fits on one page */
 
     f = MIN(xf, yf);
     xf = yf = MAX(f, minallowed);
 
-    R = ceil((xf * b.x) / relpage.x);
-    xf = ((R * relpage.x) / b.x);
-    R = ceil((yf * b.y) / relpage.y);
-    yf = ((R * relpage.y) / b.y);
+    R = ceil(xf * b.x / relpage.x);
+    xf = R * relpage.x / b.x;
+    R = ceil(yf * b.y / relpage.y);
+    yf = R * relpage.y / b.y;
     GD_drawing(g)->size.x = b.x * xf;
     GD_drawing(g)->size.y = b.y * yf;
-    return TRUE;
+    return true;
 }
