@@ -23,6 +23,7 @@
 #include <tchar.h>
 #endif
 #include <glcomp/opengl.h>
+#include <util/list.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -33,10 +34,6 @@ extern "C" {
 #define	GLCOMPSET_PANEL_COLOR_G		0.44f
 #define	GLCOMPSET_PANEL_COLOR_B		0.87f
 #define	GLCOMPSET_PANEL_COLOR_ALPHA	0.5f
-#define	GLCOMPSET_PANEL_SHADOW_COLOR_R		0.0f
-#define	GLCOMPSET_PANEL_SHADOW_COLOR_G		0.0f
-#define	GLCOMPSET_PANEL_SHADOW_COLOR_B		0.0f
-#define	GLCOMPSET_PANEL_SHADOW_COLOR_A		0.3f
 #define GLCOMPSET_PANEL_SHADOW_WIDTH		4.0f
 
 #define	GLCOMPSET_BUTTON_COLOR_R		0.0f
@@ -71,8 +68,6 @@ extern "C" {
 	    glMouseMiddleButton } glMouseButtonType;
 
     typedef enum { glTexImage, glTexLabel } glCompTexType;
-    typedef enum { glPanelObj, glButtonObj, glLabelObj,
-	    glImageObj } glObjType;
 
     typedef struct glCompObj_ glCompObj;
 
@@ -80,22 +75,12 @@ extern "C" {
     typedef void (*glcompdrawfunc_t) (void *obj);
     typedef void (*glcompclickfunc_t)(glCompObj *obj, float x,
 				       float y, glMouseButtonType t);
-    typedef void (*glcompdoubleclickfunc_t) (glCompObj * obj, float x,
-					     float y,
-					     glMouseButtonType t);
     typedef void (*glcompmouseoverfunc_t) (glCompObj * obj, float x,
 					   float y);
-    typedef void (*glcompmouseinfunc_t) (glCompObj * obj, float x,
-					 float y);
-    typedef void (*glcompmouseoutfunc_t) (glCompObj * obj, float x,
-					  float y);
     typedef void (*glcompmousedownfunc_t) (glCompObj * obj, float x,
 					   float y, glMouseButtonType t);
     typedef void (*glcompmouseupfunc_t) (glCompObj * obj, float x,
 					 float y, glMouseButtonType t);
-    typedef void (*glcompmousedragfunct_t) (glCompObj * obj, float dx,
-					    float dy,
-					    glMouseButtonType t);
 
     typedef struct {
 	int topAnchor;		/*anchor booleans */
@@ -120,10 +105,7 @@ extern "C" {
 	float x, y, z;
     } glCompPoint;
 
-    typedef struct {
-	int cnt;
-	glCompPoint* pts;
-    }glCompPoly;
+DEFINE_LIST(glCompPoly, glCompPoint)
 
     typedef struct {
 	float R;
@@ -142,8 +124,8 @@ extern "C" {
 	uint32_t id;
 	char *def;
 	char *text;
-	float width;
-	float height;
+	int width;
+	int height;
 	glCompTexType type;
 	int userCount;
 	int fontSize;
@@ -160,7 +142,6 @@ extern "C" {
 	int transparent;
 	glCompTex *tex;		/* texture, if type is pangotext */
 	int size;
-	int reference;		/*if font has references to parent */
 	glCompJustify justify;
 	bool is2D;
     } glCompFont;
@@ -168,14 +149,9 @@ extern "C" {
     typedef struct {
 	glcompdrawfunc_t draw;
 	glcompclickfunc_t click;
-	glcompdoubleclickfunc_t doubleclick;
 	glcompmouseoverfunc_t mouseover;
-	glcompmouseinfunc_t mousein;
-	glcompmouseoutfunc_t mouseout;
 	glcompmousedownfunc_t mousedown;
 	glcompmouseupfunc_t mouseup;
-	glcompmousedragfunct_t mousedrag;
-
     } glCompCallBacks;
 
 typedef struct glCompSet_ glCompSet;
@@ -195,7 +171,7 @@ typedef struct glCompSet_ glCompSet;
 	glCompSet *compset; ///< compset
 	void *parent;		/*parent widget */
 	int data;
-	glCompFont *font;	//pointer to font to use
+	glCompFont font; ///< font to use
 	glCompAlignment align;
 	glCompAnchor anchor;
 	int layer;		/*keep track of object order, what to draw on top */
@@ -206,7 +182,6 @@ typedef struct glCompSet_ glCompSet;
 
 /// object prototype
 struct glCompObj_ {
-  glObjType objType;
   glCompCommon common;
 };
 
@@ -215,24 +190,19 @@ struct glCompObj_ {
 	glCompObj base;
 	glCompTex *texture;
 	float width, height;  /* width and height in world coords */
-	/* char *pngFile; */
-	int stretch;
     } glCompImage;
 
 /*generic panel*/
     typedef struct {
 	glCompObj base;
 	float shadowwidth;
-	glCompColor shadowcolor;
 	char *text;
-	glCompImage *image;
     } glCompPanel;
 
 /*label*/
     typedef struct {
 	glCompObj base;
 	char *text;
-	int transparent;
     } glCompLabel;
 
 /*buttons*/
@@ -251,7 +221,7 @@ struct glCompObj_ {
 
     typedef struct {
 	glMouseButtonType t;
-	glCompPoint pos; /*current mouse pos,*/
+	float x, y; ///< current mouse pos
 	glCompPoint GLpos;/*3d converted opengl position*/
 	glCompPoint GLinitPos;/*mouse button down pos*/
 	glCompPoint GLfinalPos;/*mouse button up pos*/
@@ -260,9 +230,7 @@ struct glCompObj_ {
 	glCompObj *clickedObj;
 	glCompCallBacks callbacks;
 	glCompCallBacks functions;
-	int down;
-
-
+	bool down;
     } glCompMouse;
 
 
