@@ -14,7 +14,6 @@
 
 #include <ast/ast.h>
 #include <ast/error.h>
-#include <cgraph/gv_ctype.h>
 #include <gvpr/actions.h>
 #include <gvpr/compile.h>
 #include <limits.h>
@@ -26,6 +25,7 @@
 #include <unistd.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
+#include <util/gv_ctype.h>
 #include <util/strcasecmp.h>
 #include <util/unreachable.h>
 #include <util/unused.h>
@@ -33,18 +33,6 @@
 #define KINDS(p)                                                               \
   ((AGTYPE(p) == AGRAPH) ? "graph" : (AGTYPE(p) == AGNODE) ? "node" : "edge")
 
-static int iofread(void *chan, char *buf, int bufsize) {
-  FILE *fp = chan;
-
-  return (int)read(fileno(fp), buf, bufsize);
-}
-
-static int ioputstr(void *chan, const char *str) { return fputs(str, chan); }
-
-static int ioflush(void *chan) { return fflush(chan); }
-
-static Agiodisc_t gprIoDisc = {iofread, ioputstr, ioflush};
-
 /* sameG:
  * Return common root if objects belong to same root graph.
  * NULL otherwise
@@ -133,12 +121,12 @@ int copyAttr(Agobj_t *src, Agobj_t *tgt) {
   while ((sym = agnxtattr(srcg, skind, sym))) {
     tsym = agattrsym(tgt, sym->name);
     if (!tsym)
-      tsym = agattr(tgtg, tkind, sym->name, sym->defval);
+      tsym = agattr_text(tgtg, tkind, sym->name, sym->defval);
     val = agxget(src, sym);
     if (aghtmlstr(val)) {
       val = agstrdup_html(tgtg, val);
       agxset(tgt, tsym, val);
-      agstrfree(tgtg, val);
+      agstrfree(tgtg, val, true);
     } else
       agxset(tgt, tsym, val);
   }
@@ -541,14 +529,14 @@ int deleteObj(Agraph_t *g, Agobj_t *obj) {
 
 /* sfioWrite:
  * If the graph is passed in from a library, its output discipline
- * might not use sfio. In this case, we push an sfio discipline on
+ * might not use stdio. In this case, we push a stdio discipline on
  * the graph, write it, and then pop it off.
  */
 int sfioWrite(Agraph_t *g, FILE *fp) {
   int rv;
 
   Agiodisc_t *saveio = g->clos->disc.io;
-  g->clos->disc.io = &gprIoDisc;
+  g->clos->disc.io = &AgIoDisc;
   rv = agwrite(g, fp);
   g->clos->disc.io = saveio;
   return rv;
