@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,13 +17,11 @@
 
 double pxmin, pxmax, pymin, pymax;	/* clipping window */
 
-static int nedges;
 static Freelist efl;
 
 void edgeinit(void)
 {
     freeinit(&efl, sizeof(Edge));
-    nedges = 0;
 }
 
 Edge *gvbisect(Site * s1, Site * s2)
@@ -56,8 +54,6 @@ Edge *gvbisect(Site * s1, Site * s2)
 	newedge->c /= dy;
     };
 
-    newedge->edgenbr = nedges;
-    nedges += 1;
     return (newedge);
 }
 
