@@ -33,16 +33,18 @@
 #include <common/render.h>
 #include <common/utils.h>
 #include <neatogen/sgd.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/bitarray.h>
 #include <cgraph/cgraph.h>
-#include <cgraph/gv_ctype.h>
-#include <cgraph/startswith.h>
 #include <float.h>
+#include <stdatomic.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <util/alloc.h>
+#include <util/bitarray.h>
+#include <util/gv_ctype.h>
+#include <util/gv_math.h>
+#include <util/itos.h>
 #include <util/prisize_t.h>
+#include <util/startswith.h>
 #include <util/strcasecmp.h>
 #include <util/streq.h>
 
@@ -272,7 +274,7 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
     pointf sp = { 0, 0 }, ep = { 0, 0};
     bezier *newspl;
     int more = 1;
-    static bool warned;
+    static atomic_flag warned;
 
     pos = agxget(e, E_pos);
     if (*pos == '\0')
@@ -303,8 +305,7 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 	n = npts;
 	if (n < 4 || n % 3 != 1) {
 	    gv_free_splines(e);
-	    if (!warned) {
-		warned = true;
+	    if (!atomic_flag_test_and_set(&warned)) {
 		agwarningf("pos attribute for edge (%s,%s) doesn't have 3n+1 points\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	    }
 	    return 0;
@@ -314,8 +315,7 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 	while (n) {
 	    i = sscanf(pos, "%lf,%lf%n", &x, &y, &nc);
 	    if (i < 2) {
-		if (!warned) {
-		    warned = true;
+		if (!atomic_flag_test_and_set(&warned)) {
 		    agwarningf("syntax error in pos attribute for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 		}
 		free(ps);
@@ -383,12 +383,11 @@ static pos_edge nop_init_edges(Agraph_t * g)
     node_t *n;
     edge_t *e;
     int nedges = 0;
-    attrsym_t *E_pos;
 
     if (agnedges(g) == 0)
 	return AllEdges;
 
-    E_pos = agfindedgeattr(g, "pos");
+    attrsym_t *const E_pos = agfindedgeattr(g, "pos");
     if (!E_pos || Nop < 2)
 	return NoEdges;
 
@@ -444,9 +443,7 @@ static int chkBB(Agraph_t * g, attrsym_t * G_bb, boxf* bbp)
          * we assume the input was produced using -y, so we normalize
 	 * the bb.
 	 */
-	    double tmp = bb.LL.y;
-	    bb.LL.y = bb.UR.y;
-	    bb.UR.y = tmp;
+	    SWAP(&bb.LL.y, &bb.UR.y);
 	}
 	*bbp = bb;
 	return 1;
@@ -547,7 +544,7 @@ int init_nop(Agraph_t * g, int adjust)
 
     /* If G_bb not defined, define it */
     if (!G_bb)
-	G_bb = agattr(g, AGRAPH, "bb", "");
+	G_bb = agattr_text(g, AGRAPH, "bb", "");
 
     scan_graph(g);		/* mainly to set up GD_neato_nlist */
     for (i = 0; (np = GD_neato_nlist(g)[i]); i++) {
@@ -640,7 +637,7 @@ static int neatoModel(graph_t * g)
     if (streq(p, "shortpath"))
 	return MODEL_SHORTPATH;
     if (streq(p, "mds")) {
-	if (agattr(g, AGEDGE, "len", 0))
+	if (agattr_text(g, AGEDGE, "len", 0))
 	    return MODEL_MDS;
 	else {
 	    agwarningf(
@@ -694,12 +691,9 @@ static int checkEdge(PointMap * pm, edge_t * ep, int idx)
 {
     int i = ND_id(agtail(ep));
     int j = ND_id(aghead(ep));
-    int tmp;
 
     if (i > j) {
-	tmp = i;
-	i = j;
-	j = tmp;
+	SWAP(&i, &j);
     }
     return insertPM(pm, i, j, idx);
 }
@@ -795,7 +789,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
     bool haveLen = false;
     bool haveWt = false;
     if (model != MODEL_SUBSET) {
-	haveLen = agattr(g, AGEDGE, "len", 0) != NULL;
+	haveLen = agattr_text(g, AGEDGE, "len", 0) != NULL;
 	haveWt = E_weight != 0;
     }
     bool haveDir = mode == MODE_HIER || mode == MODE_IPSEP;
@@ -983,10 +977,7 @@ setSeed (graph_t * G, int dflt, long* seedp)
 #else
 	    seed = (unsigned) getpid() ^ (unsigned) time(NULL);
 #endif
-	    agxbuf buf = {0};
-	    agxbprint(&buf, "%ld", seed);
-	    agset(G, "start", agxbuse(&buf));
-	    agxbfree(&buf);
+	    agset(G, "start", ITOS(seed));
 	}
 	*seedp = seed;
     }
