@@ -3,14 +3,15 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
-#include <cgraph/alloc.h>
 #include <neatogen/overlap.h>
+#include <stdatomic.h>
+#include <util/alloc.h>
 
 #if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
 
@@ -55,13 +56,13 @@ static void ideal_distance_avoid_overlap(int dim, SparseMatrix A, double *x, dou
 	} else if (dy < MACHINEACC*wy){
 	  t = wx/dx;
 	} else {
-	  t = MIN(wx/dx, wy/dy);
+	  t = fmin(wx / dx, wy / dy);
 	}
-	if (t > 1) t = MAX(t, 1.001);/* no point in things like t = 1.00000001 as this slow down convergence */
-	*tmax = MAX(*tmax, t);
-	*tmin = MIN(*tmin, t);
-	t = MIN(expandmax, t);
-	t = MAX(expandmin, t);
+	if (t > 1) t = fmax(t, 1.001);/* no point in things like t = 1.00000001 as this slow down convergence */
+	*tmax = fmax(*tmax, t);
+	*tmin = fmin(*tmin, t);
+	t = fmin(expandmax, t);
+	t = fmax(expandmin, t);
 	if (t > 1) {
 	  ideal_distance[j] = t*dist;
 	} else {
@@ -75,14 +76,11 @@ static void ideal_distance_avoid_overlap(int dim, SparseMatrix A, double *x, dou
 
 enum {INTV_OPEN, INTV_CLOSE};
 
-struct scan_point_struct{
+typedef struct {
   int node;
   double x;
   int status;
-};
-
-typedef struct scan_point_struct scan_point;
-
+} scan_point;
 
 static int comp_scan_points(const void *p, const void *q){
   const scan_point *pp = p;
@@ -103,30 +101,15 @@ static int comp_scan_points(const void *p, const void *q){
 }
 
 static void NodeDest(void* a) {
+  (void)a;
   /*  free((int*)a);*/
 }
 
-static int NodeComp(const void* a,const void* b) {
-  return comp_scan_points(a,b);
-
-}
-
-static void NodePrint(const void* a) {
-  const scan_point *aa = a;
-  fprintf(stderr, "node {%d, %f, %d}\n", aa->node, aa->x, aa->status);
-}
-
-static void InfoPrint(void* a) {
-}
-
-static void InfoDest(void *a){
-}
-
 static SparseMatrix get_overlap_graph(int dim, int n, double *x, double *width, int check_overlap_only){
   /* if check_overlap_only = TRUE, we only check whether there is one overlap */
   int i, k, neighbor;
   SparseMatrix A = NULL, B = NULL;
-  rb_red_blk_node *newNode, *newNode0, *newNode2 = NULL;
+  rb_red_blk_node *newNode, *newNode0;
   rb_red_blk_tree* treey;
   double one = 1;
 
@@ -153,8 +136,7 @@ static SparseMatrix get_overlap_graph(int dim, int n, double *x, double *width,
     scanpointsy[i+n].status = INTV_CLOSE;
   }
 
-
-  treey = RBTreeCreate(NodeComp,NodeDest,InfoDest,NodePrint,InfoPrint);
+  treey = RBTreeCreate(comp_scan_points, NodeDest);
 
   for (i = 0; i < 2*n; i++){
 #ifdef DEBUG_RBTREE
@@ -170,14 +152,14 @@ static SparseMatrix get_overlap_graph(int dim, int n, double *x, double *width,
       treey->PrintKey(&(scanpointsy[k]));
 #endif
 
-      RBTreeInsert(treey, &(scanpointsy[k]), NULL); /* add both open and close int for y */
+      RBTreeInsert(treey, &scanpointsy[k]); // add both open and close int for y
 
 #ifdef DEBUG_RBTREE
       fprintf(stderr, "inserting2...");
       treey->PrintKey(&(scanpointsy[k+n]));
 #endif
 
-      RBTreeInsert(treey, &(scanpointsy[k+n]), NULL);
+      RBTreeInsert(treey, &scanpointsy[k + n]);
     } else {
       double bsta, bbsta, bsto, bbsto; int ii; 
 
@@ -189,7 +171,7 @@ static SparseMatrix get_overlap_graph(int dim, int n, double *x, double *width,
       bsta = scanpointsy[ii].x; bsto = scanpointsy[ii+n].x;
 
 #ifdef DEBUG_RBTREE
-      fprintf(stderr, "poping..%d....yinterval={%f,%f}\n", scanpointsy[k + n].node, bsta, bsto);
+      fprintf(stderr, "popping..%d....yinterval={%f,%f}\n", scanpointsy[k + n].node, bsta, bsto);
       treey->PrintKey(newNode->key);
 #endif
 
@@ -208,30 +190,15 @@ static SparseMatrix get_overlap_graph(int dim, int n, double *x, double *width,
 #endif
 	    if (check_overlap_only) goto check_overlap_RETURN;
 	  }
-	} else {
-	  newNode2 = newNode;
 	}
-
       }
 
 #ifdef DEBUG_RBTREE
-      fprintf(stderr, "deleteing...");
+      fprintf(stderr, "deleting...");
       treey->PrintKey(newNode0->key);
 #endif
 
       if (newNode0) RBDelete(treey,newNode0);
-
-
-     if (newNode2 && newNode2 != treey->nil && newNode2 != newNode0) {
-
-#ifdef DEBUG_RBTREE
-	fprintf(stderr, "deleteing2...");
-	treey->PrintKey(newNode2->key);
-#endif
-
-	if (newNode0) RBDelete(treey,newNode2);
-      }
-
     }
   }
 
@@ -254,13 +221,13 @@ check_overlap_RETURN:
 
 
 static void relative_position_constraints_delete(void *d){
-  relative_position_constraints data;
   if (!d) return;
-  data = (relative_position_constraints) d;
+  relative_position_constraints data = d;
   free(data->irn);
   free(data->jcn);
   free(data->val);
-  /* other stuff inside relative_position_constraints is assed back to the user hence no need to deallocator*/
+  // other stuff inside `relative_position_constraints` is passed back to the
+  // user hence no need to deallocate
   free(d);
 }
 
@@ -285,7 +252,9 @@ static void scale_coord(int dim, int m, double *x, double scale){
   }
 }
 
-double overlap_scaling(int dim, int m, double *x, double *width, double scale_sta, double scale_sto, double epsilon, int maxiter){
+static double overlap_scaling(int dim, int m, double *x, double *width,
+                              double scale_sta, double scale_sto,
+                              double epsilon, int maxiter) {
   /* do a bisection between scale_sta and scale_sto, up to maxiter iterations or till interval <= epsilon, to find the best scaling to avoid overlap
      m: number of points
      x: the coordinates
@@ -360,7 +329,7 @@ double overlap_scaling(int dim, int m, double *x, double *width, double scale_st
 }
  
 OverlapSmoother OverlapSmoother_new(SparseMatrix A, int m, 
-				    int dim, double lambda0, double *x, double *width, int include_original_graph, int neighborhood_only, 
+				    int dim, double *x, double *width, bool neighborhood_only,
 				    double *max_overlap, double *min_overlap,
 				    int edge_labeling_scheme, int n_constr_nodes, int *constr_nodes, SparseMatrix A_constr, int shrink
 				    ){
@@ -381,10 +350,9 @@ OverlapSmoother OverlapSmoother_new(SparseMatrix A, int m,
   }
 
   sm->tol_cg = 0.01;
-  sm->maxit_cg = sqrt((double) A->m);
+  sm->maxit_cg = floor(sqrt(A->m));
 
-  double *lambda = sm->lambda = gv_calloc(m, sizeof(double));
-  for (i = 0; i < m; i++) sm->lambda[i] = lambda0;
+  sm->lambda = gv_calloc(m, sizeof(double));
   
   B= call_tri(m, x);
 
@@ -396,23 +364,9 @@ OverlapSmoother OverlapSmoother_new(SparseMatrix A, int m,
     SparseMatrix_delete(C);
     B = D;
   }
-  if (include_original_graph){
-    sm->Lw = SparseMatrix_add(A, B);
-    SparseMatrix_delete(B);
-  } else {
-    sm->Lw = B;
-  }
+  sm->Lw = B;
   sm->Lwd = SparseMatrix_copy(sm->Lw);
 
-#ifdef DEBUG
-  {
-    FILE *fp;
-    fp = fopen("/tmp/111","w");
-    export_embedding(fp, dim, sm->Lwd, x, NULL);
-    fclose(fp);
-  }
-#endif
-
   if (!(sm->Lw) || !(sm->Lwd)) {
     OverlapSmoother_delete(sm);
     return NULL;
@@ -424,7 +378,8 @@ OverlapSmoother OverlapSmoother_new(SparseMatrix A, int m,
 
   /* no overlap at all! */
   if (*max_overlap < 1 && shrink){
-    double scale_sta = MIN(1, *max_overlap*1.0001), scale_sto = 1;
+    double scale_sta = fmin(1, *max_overlap * 1.0001);
+    const double scale_sto = 1;
 
     if (Verbose) fprintf(stderr," no overlap (overlap = %f), rescale to shrink\n", *max_overlap - 1);
 
@@ -460,10 +415,8 @@ OverlapSmoother OverlapSmoother_new(SparseMatrix A, int m,
 
     }
 
-    lambda[i] *= (-diag_w);/* alternatively don't do that then we have a constant penalty term scaled by lambda0 */
-
     assert(jdiag >= 0);
-    w[jdiag] = -diag_w + lambda[i];
+    w[jdiag] = -diag_w;
     d[jdiag] = -diag_d;
   }
  RETURN:
@@ -479,14 +432,7 @@ void OverlapSmoother_delete(OverlapSmoother sm){
 double OverlapSmoother_smooth(OverlapSmoother sm, int dim, double *x){
   int maxit_sm = 1;/* only using 1 iteration of stress majorization 
 		      is found to give better results and save time! */
-  double res = StressMajorizationSmoother_smooth(sm, dim, x, maxit_sm, 0.001);
-#ifdef DEBUG
-  {FILE *fp;
-  fp = fopen("/tmp/222","w");
-  export_embedding(fp, dim, sm->Lwd, x, NULL);
-  fclose(fp);}
-#endif
-  return res;
+  return StressMajorizationSmoother_smooth(sm, dim, x, maxit_sm);
 }
 
 /*================================= end OverlapSmoother =============*/
@@ -500,7 +446,7 @@ static void scale_to_edge_length(int dim, SparseMatrix A, double *x, double avg_
   if (Verbose) fprintf(stderr,"avg edge len=%f avg_label-size= %f\n", dist, avg_label_size);
 
 
-  dist = avg_label_size/MAX(dist, MACHINEACC);
+  dist = avg_label_size / fmax(dist, MACHINEACC);
 
   for (i = 0; i < dim*A->m; i++) x[i] *= dist;
 }
@@ -515,8 +461,8 @@ static void print_bounding_box(int n, int dim, double *x){
 
   for (i = 0; i < n; i++){
     for (k = 0; k < dim; k++){
-      xmin[k] = MIN(xmin[k],x[i*dim+k]);
-      xmax[k] = MAX(xmax[k],x[i*dim+k]);
+      xmin[k] = fmin(xmin[k], x[i * dim + k]);
+      xmax[k] = fmax(xmax[k], x[i * dim + k]);
     }
   }
   fprintf(stderr,"bounding box = \n");
@@ -527,13 +473,15 @@ static void print_bounding_box(int n, int dim, double *x){
   free(xmax);
 }
 
-static int check_convergence(double max_overlap, double res, int has_penalty_terms, double epsilon){
-  if (!has_penalty_terms) return (max_overlap <= 1);
+static int check_convergence(double max_overlap, double res,
+                             bool has_penalty_terms, double epsilon) {
+  if (!has_penalty_terms)
+    return (max_overlap <= 1);
   return res < epsilon;
 }
 
 void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes, int ntry, double initial_scaling, 
-		    int edge_labeling_scheme, int n_constr_nodes, int *constr_nodes, SparseMatrix A_constr, int do_shrinking){
+		    int edge_labeling_scheme, int n_constr_nodes, int *constr_nodes, SparseMatrix A_constr, bool do_shrinking) {
   /* 
      edge_labeling_scheme: if ELSCHEME_NONE, n_constr_nodes/constr_nodes/A_constr are not used
 
@@ -546,14 +494,12 @@ void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes, int
 
   */
 
-  double lambda = 0.00;
   OverlapSmoother sm;
-  int include_original_graph = 0, i;
+  int i;
   double LARGE = 100000;
   double avg_label_size, res = LARGE;
   double max_overlap = 0, min_overlap = 999;
-  int neighborhood_only = TRUE;
-  int has_penalty_terms = FALSE;
+  bool neighborhood_only = true;
   double epsilon = 0.005;
   int shrink = 0;
 
@@ -580,54 +526,43 @@ void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes, int
 
 #ifdef DEBUG
   _statistics[0] = _statistics[1] = 0.;
-  {FILE*fp;
-  fp = fopen("x1","w");
-  for (i = 0; i < A->m; i++){
-    fprintf(fp, "%f %f\n",x[i*2],x[i*2+1]);
-  }
-  fclose(fp);
-  }
 #endif
 
-#ifdef ANIMATE
-  {FILE*fp;
-    fp = fopen("/tmp/m","wa");
-    fprintf(fp,"{");
-#endif
-
-  has_penalty_terms = (edge_labeling_scheme != ELSCHEME_NONE && n_constr_nodes > 0);
+  bool has_penalty_terms =
+      edge_labeling_scheme != ELSCHEME_NONE && n_constr_nodes > 0;
   for (i = 0; i < ntry; i++){
     if (Verbose) print_bounding_box(A->m, dim, x);
-    sm = OverlapSmoother_new(A, A->m, dim, lambda, x, label_sizes, include_original_graph, neighborhood_only,
+    sm = OverlapSmoother_new(A, A->m, dim, x, label_sizes, neighborhood_only,
 			     &max_overlap, &min_overlap, edge_labeling_scheme, n_constr_nodes, constr_nodes, A_constr, shrink); 
-    if (Verbose) fprintf(stderr, "overlap removal neighbors only?= %d iter -- %d, overlap factor = %g underlap factor = %g\n", neighborhood_only, i, max_overlap - 1, min_overlap);
-    if (check_convergence(max_overlap, res, has_penalty_terms, epsilon)){
+    if (Verbose) {
+      fprintf(stderr,
+              "overlap removal neighbors only?= %d iter -- %d, overlap factor = %g underlap factor = %g\n",
+              (int)neighborhood_only, i, max_overlap - 1, min_overlap);
+    }
+    if (check_convergence(max_overlap, res, has_penalty_terms, epsilon)) {
     
       OverlapSmoother_delete(sm);
       if (!neighborhood_only){
 	break;
       } else {
 	res = LARGE;
-	neighborhood_only = FALSE; if (do_shrinking) shrink = 1;
+	neighborhood_only = false;
+        if (do_shrinking) {
+          shrink = 1;
+        }
 	continue;
       }
     }
     
     res = OverlapSmoother_smooth(sm, dim, x);
     if (Verbose) fprintf(stderr,"res = %f\n",res);
-#ifdef ANIMATE
-    if (i != 0) fprintf(fp,",");
-    export_embedding(fp, dim, A, x, label_sizes);
-#endif
     OverlapSmoother_delete(sm);
   }
-  if (Verbose) fprintf(stderr, "overlap removal neighbors only?= %d iter -- %d, overlap factor = %g underlap factor = %g\n", neighborhood_only, i, max_overlap - 1, min_overlap);
-
-#ifdef ANIMATE
-  fprintf(fp,"}");
-    fclose(fp);
+  if (Verbose) {
+    fprintf(stderr,
+            "overlap removal neighbors only?= %d iter -- %d, overlap factor = %g underlap factor = %g\n",
+            (int)neighborhood_only, i, max_overlap - 1, min_overlap);
   }
-#endif
 
   if (has_penalty_terms){
     /* now do without penalty */
@@ -638,23 +573,8 @@ void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes, int
 #ifdef DEBUG
   fprintf(stderr," number of cg iter = %f, number of stress majorization iter = %f number of overlap removal try = %d\n",
 	  _statistics[0], _statistics[1], i - 1);
-
-  {FILE*fp;
-  fp = fopen("x2","w");
-  for (i = 0; i < A->m; i++){
-    fprintf(fp, "%f %f\n",x[i*2],x[i*2+1]);
-  }
-  fclose(fp);
-  }
 #endif
 
-#ifdef DEBUG
-  {FILE*fp;
-  fp = fopen("/tmp/m","w");
-  if (A) export_embedding(fp, dim, A, x, label_sizes);
-  fclose(fp);
-  }
-#endif
 #ifdef TIME
   fprintf(stderr, "post processing %f\n",((double) (clock() - cpu)) / CLOCKS_PER_SEC);
 #endif
@@ -664,9 +584,9 @@ void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes, int
 #include <common/types.h>
 #include <sparse/SparseMatrix.h>
 void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes, int ntry, double initial_scaling,
-		    int edge_labeling_scheme, int n_constr_nodes, int *constr_nodes, SparseMatrix A_constr, int do_shrinking)
+		    int edge_labeling_scheme, int n_constr_nodes, int *constr_nodes, SparseMatrix A_constr, bool do_shrinking)
 {
-    static int once;
+    static atomic_flag once;
 
     (void)dim;
     (void)A;
@@ -680,9 +600,8 @@ void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes, int
     (void)A_constr;
     (void)do_shrinking;
 
-    if (once == 0) {
-	once = 1;
-	agerr(AGERR, "remove_overlap: Graphviz not built with triangulation library\n");
+    if (!atomic_flag_test_and_set(&once)) {
+	agerrorf("remove_overlap: Graphviz not built with triangulation library\n");
     }
 }
 #endif
