@@ -3,17 +3,15 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
 
-#ifdef _WIN32
-#include <io.h>
-#endif
-
+#include <assert.h>
+#include <math.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdlib.h>
@@ -24,15 +22,16 @@
 
 #include <gvc/gvplugin_render.h>
 #include <gvc/gvplugin_device.h>
-#include <cgraph/alloc.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/gv_ctype.h>
-#include <cgraph/prisize_t.h>
-#include <cgraph/streq.h>
-#include <cgraph/unreachable.h>
 #include <common/utils.h>
 #include <gvc/gvc.h>
 #include <gvc/gvio.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/gv_ctype.h>
+#include <util/prisize_t.h>
+#include <util/streq.h>
+#include <util/unreachable.h>
+#include "core_loadimage_xdot.h"
 
 typedef enum {
 	FORMAT_DOT,
@@ -77,6 +76,7 @@ typedef struct {
     attrsym_t *tl_draw;
     unsigned short version;
     char* version_s;
+    double yOff; ///< ymin + ymax
 } xdot_state_t;
 static xdot_state_t* xd;
 
@@ -91,6 +91,34 @@ static void xdot_str (GVJ_t *job, char* pfx, const char* s)
     xdot_str_xbuf (xbufs[emit_state], pfx, s);
 }
 
+/// output a color
+///
+/// @param xb Buffer to write to
+/// @param String prefix to prepend
+/// @param rgba Color in RGBA format
+static void xdot_str_color_xbuf(agxbuf *xb, const char *prefix,
+                                const unsigned char rgba[4]) {
+  if (rgba[3] == 0xFF) {
+    agxbprint(xb, "%s%" PRISIZE_T " -#%02x%02x%02x ", prefix, strlen("#FFFFFF"),
+              rgba[0], rgba[1], rgba[2]);
+  } else {
+    agxbprint(xb, "%s%" PRISIZE_T " -#%02x%02x%02x%02x ", prefix,
+              strlen("#FFFFFFFF"), rgba[0], rgba[1], rgba[2], rgba[3]);
+  }
+}
+
+/// output a color
+///
+/// @param job Job indicating which state buffer to write to
+/// @param prefix String prefix to prepend
+/// @param rgba Color in RGBA format
+static void xdot_str_color(GVJ_t *job, const char *prefix,
+                           const unsigned char rgba[4]) {
+  emit_state_t emit_state = job->obj->emit_state;
+  agxbuf *xb = xbufs[emit_state];
+  xdot_str_color_xbuf(xb, prefix, rgba);
+}
+
 /* xdot_fmt_num:
  * Convert double to string with space at end.
  * Trailing zeros are removed and decimal point, if possible.
@@ -104,7 +132,8 @@ static void xdot_fmt_num(agxbuf *buf, double v) {
 static void xdot_point(agxbuf *xb, pointf p)
 {
   xdot_fmt_num(xb, p.x);
-  xdot_fmt_num(xb, yDir(p.y));
+  assert(xd != NULL);
+  xdot_fmt_num(xb, yDir(p.y, xd->yOff));
 }
 
 static void xdot_num(agxbuf *xb, double v)
@@ -119,27 +148,14 @@ static void xdot_points(GVJ_t *job, char c, pointf *A, size_t n) {
         xdot_point(xbufs[emit_state], A[i]);
 }
 
-static char*
-color2str (unsigned char rgba[4])
-{
-    static char buf [10];
-
-    if (rgba[3] == 0xFF)
-	snprintf(buf, sizeof(buf), "#%02x%02x%02x", rgba[0], rgba[1],  rgba[2]);
-    else
-	snprintf(buf, sizeof(buf), "#%02x%02x%02x%02x", rgba[0], rgba[1],  rgba[2],
-	         rgba[3]);
-    return buf;
-}
-
 static void xdot_pencolor (GVJ_t *job)
 {
-    xdot_str (job, "c ", color2str (job->obj->pencolor.u.rgba));
+  xdot_str_color(job, "c ", job->obj->pencolor.u.rgba);
 }
 
 static void xdot_fillcolor (GVJ_t *job)
 {
-    xdot_str (job, "C ", color2str (job->obj->fillcolor.u.rgba));
+  xdot_str_color(job, "C ", job->obj->fillcolor.u.rgba);
 }
 
 static void xdot_style (GVJ_t *job)
@@ -148,7 +164,7 @@ static void xdot_style (GVJ_t *job)
     char* p, **s;
 
     /* First, check if penwidth state is correct */
-    if (job->obj->penwidth != penwidth[job->obj->emit_state]) {
+    if (fabs(job->obj->penwidth - penwidth[job->obj->emit_state]) >= 0.0005) {
 	penwidth[job->obj->emit_state] = job->obj->penwidth;
 	agxbput (&xb, "setlinewidth(");
 	agxbprint(&xb, "%.3f", job->obj->penwidth);
@@ -282,7 +298,7 @@ static unsigned short versionStr2Version(const char *str) {
     }
     unsigned short digit = (unsigned short)(str[i] - '0');
     if (us > (USHRT_MAX - digit) / 10) {
-      agerr(AGWARN, "xdot version \"%s\" too long", str);
+      agwarningf("xdot version \"%s\" too long", str);
       break;
     }
     us = (unsigned short)(us * 10 + digit);
@@ -311,10 +327,11 @@ static unsigned short versionStr2Version(const char *str) {
  * and "t" (for tooltip) and "x" (for position). We use  those letters in 
  * our drawing spec (and also "<" and ">"), so if you  start generating 
  * output with them, it could break what we have. 
+ *
+ * @param yOff ymax - ymin
  */
-static void
-xdot_begin_graph (graph_t *g, int s_arrows, int e_arrows, format_type id)
-{
+static void xdot_begin_graph(graph_t *g, bool s_arrows, bool e_arrows,
+                             format_type id, double yOff) {
     int i;
     unsigned short us;
     char* s;
@@ -374,12 +391,12 @@ xdot_begin_graph (graph_t *g, int s_arrows, int e_arrows, format_type id)
 
     for (i = 0; i < NUMXBUFS; i++)
 	xbuf[i] = (agxbuf){0};
+
+    xd->yOff = yOff;
 }
 
 static void dot_begin_graph(GVJ_t *job)
 {
-    int e_arrows;            /* graph has edges with end arrows */
-    int s_arrows;            /* graph has edges with start arrows */
     graph_t *g = job->obj->u.g;
 
     switch (job->render.id) {
@@ -395,10 +412,13 @@ static void dot_begin_graph(GVJ_t *job)
 	    break;
 	case FORMAT_XDOT:
 	case FORMAT_XDOT12:
-	case FORMAT_XDOT14:
-	    attach_attrs_and_arrows(g, &s_arrows, &e_arrows);
-	    xdot_begin_graph(g, s_arrows, e_arrows, job->render.id);
+	case FORMAT_XDOT14: {
+	    bool e_arrows; // graph has edges with end arrows
+	    bool s_arrows; // graph has edges with start arrows
+	    const double yOff = attach_attrs_and_arrows(g, &s_arrows, &e_arrows);
+	    xdot_begin_graph(g, s_arrows, e_arrows, job->render.id, yOff);
 	    break;
+	}
 	default:
 	    UNREACHABLE();
     }
@@ -514,12 +534,11 @@ static void xdot_textspan(GVJ_t * job, pointf p, textspan_t * span)
     xdot_str (job, "", span->str);
 }
 
-static void xdot_color_stop (agxbuf* xb, float v, gvcolor_t* clr)
-{
+static void xdot_color_stop(agxbuf *xb, double v, gvcolor_t *clr) {
   agxbprint(xb, "%.03f", v);
   agxbuf_trim_zeros(xb);
   agxbputc(xb, ' ');
-  xdot_str_xbuf(xb, "", color2str (clr->u.rgba));
+  xdot_str_color_xbuf(xb, "", clr->u.rgba);
 }
 
 static void xdot_gradient_fillcolor(GVJ_t *job, int filled, pointf *A, size_t n)
@@ -549,8 +568,8 @@ static void xdot_gradient_fillcolor(GVJ_t *job, int filled, pointf *A, size_t n)
 	    c1.y = G[0].y;
 	}
 	else {
-	    c1.x = G[0].x +  (r2/4) * cos(angle);
-	    c1.y = G[0].y +  (r2/4) * sin(angle);
+	    c1.x = G[0].x + r2 / 4 * cos(angle);
+	    c1.y = G[0].y + r2 / 4 * sin(angle);
 	}
 	c2.x = G[0].x;
 	c2.y = G[0].y;
@@ -587,7 +606,7 @@ static void xdot_ellipse(GVJ_t * job, pointf * A, int filled)
     xdot_style (job);
     xdot_pencolor (job);
     if (filled) {
-	if ((filled == GRADIENT) || (filled == RGRADIENT)) {
+	if (filled == GRADIENT || filled == RGRADIENT) {
 	   xdot_gradient_fillcolor (job, filled, A, 2);
 	}
         else 
@@ -605,7 +624,7 @@ static void xdot_bezier(GVJ_t *job, pointf *A, size_t n, int filled) {
     xdot_style (job);
     xdot_pencolor (job);
     if (filled) {
-	if ((filled == GRADIENT) || (filled == RGRADIENT)) {
+	if (filled == GRADIENT || filled == RGRADIENT) {
 	   xdot_gradient_fillcolor(job, filled, A, n);
 	}
         else
@@ -620,7 +639,7 @@ static void xdot_polygon(GVJ_t *job, pointf *A, size_t n, int filled) {
     xdot_style (job);
     xdot_pencolor (job);
     if (filled) {
-	if ((filled == GRADIENT) || (filled == RGRADIENT)) {
+	if (filled == GRADIENT || filled == RGRADIENT) {
 	   xdot_gradient_fillcolor(job, filled, A, n);
 	}
         else
