@@ -16,13 +16,13 @@
 #define _GNU_SOURCE
 #include "config.h"
 #include <assert.h>
-#include <cgraph/agxbuf.h>
 #include <math.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
+#include <util/agxbuf.h>
 #include <util/prisize_t.h>
 
 #include <common/macros.h>
@@ -295,9 +295,7 @@
 
 #define DPI 72.0
 #define RENDERER_COLOR_TYPE RGBA_BYTE
-typedef enum { FORMAT_POV, } format_type;
-
-//#define DEBUG
+enum { FORMAT_POV, };
 
 //TODO: check why this dot file does not work (90 rotated)
 //   /usr/share/graphviz/graphs/directed/NaN.gv
@@ -743,7 +741,7 @@ gvrender_features_t render_features_pov = {
 	    | GVRENDER_DOES_MAP_BSPLINE
 	    | GVRENDER_NO_WHITE_BG
 	    | GVRENDER_DOES_TRANSFORM
-	    | GVRENDER_DOES_Z | GVRENDER_DOES_MAP_BSPLINE,
+	    | GVRENDER_DOES_Z,
 	4.0,			/* default pad - graph units */
 	pov_knowncolors,	/* knowncolors */
 	sizeof(pov_knowncolors) / sizeof(char *),	/* strings in knowncolors */
