@@ -3,19 +3,19 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifndef GVPLUGIN_QUARTZ_H
-#define GVPLUGIN_QUARTZ_H
+#pragma once
 
 #include <Availability.h>
+#include <TargetConditionals.h>
 
-#if defined(__IPHONE_OS_VERSION_MIN_REQUIRED)
+#if TARGET_OS_IPHONE
 #include <CoreGraphics/CoreGraphics.h>
-#elif defined(__MAC_OS_X_VERSION_MIN_REQUIRED)
+#else
 #include <ApplicationServices/ApplicationServices.h>
 #endif
 
@@ -44,16 +44,12 @@ typedef enum {
 
 static const int BYTE_ALIGN = 15;			/* align to 16 bytes */
 static const int BITS_PER_COMPONENT = 8;	/* bits per color component */
-static const int BYTES_PER_PIXEL = 4;		/* bytes per pixel */
 
-#if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 1040 || __ENVIRONMENT_IPHONE_OS_VERSION_MIN_REQUIRED__ >= 40000
 CFStringRef format_to_uti(format_type format);
-#endif
 
 extern CGDataConsumerCallbacks device_data_consumer_callbacks;
 
 /* gvtextlayout_quartz.c in Mac OS X: layout is a CoreText CTLineRef */
-/* GVTextLayout.m in iPhoneOS: layout is a custom Objective-C GVTextLayout */
 
 void *quartz_new_layout(char* fontname, double fontsize, char* text);
 void quartz_size_layout(void *layout, double* width, double* height, double* yoffset_layout);
@@ -63,5 +59,3 @@ void quartz_free_layout(void *layout);
 #ifdef __cplusplus
 }
 #endif
-
-#endif
