@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -13,32 +13,19 @@
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
+#include <TargetConditionals.h>
 
 #include <gvc/gvplugin_textlayout.h>
 #include "gvplugin_quartz.h"
 
-#ifdef __ENVIRONMENT_IPHONE_OS_VERSION_MIN_REQUIRED__
-#if __ENVIRONMENT_IPHONE_OS_VERSION_MIN_REQUIRED__ >= 30200
+#if TARGET_OS_IPHONE
 #include <CoreText/CoreText.h>
 #endif
-#endif
-
-#if (defined(__ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__) &&                 \
-     __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 1050) ||                 \
-    (defined(__ENVIRONMENT_IPHONE_OS_VERSION_MIN_REQUIRED__) &&                \
-     __ENVIRONMENT_IPHONE_OS_VERSION_MIN_REQUIRED__ >= 30200)
-
-#ifdef __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__
-#if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ < 1060
-/* symbol defined in 10.5.x dylib but not in headers */
-extern const CFStringRef kCTForegroundColorFromContextAttributeName;
-#endif
-#endif
 
 void *quartz_new_layout(char* fontname, double fontsize, char* text)
 {
-	CFStringRef fontnameref = CFStringCreateWithBytes(kCFAllocatorDefault, (const UInt8 *)fontname, strlen(fontname), kCFStringEncodingUTF8, FALSE);
-	CFStringRef textref = CFStringCreateWithBytes(kCFAllocatorDefault, (const UInt8 *)text, strlen(text), kCFStringEncodingUTF8, FALSE);
+	CFStringRef fontnameref = CFStringCreateWithBytes(NULL, (const UInt8 *)fontname, strlen(fontname), kCFStringEncodingUTF8, false);
+	CFStringRef textref = CFStringCreateWithBytes(NULL, (const UInt8 *)text, strlen(text), kCFStringEncodingUTF8, false);
 	CTLineRef line = NULL;
 	
 	if (fontnameref && textref) {
@@ -47,13 +34,13 @@ void *quartz_new_layout(char* fontname, double fontsize, char* text)
 		CFTypeRef attributeNames[] = { kCTFontAttributeName, kCTForegroundColorFromContextAttributeName };
 		CFTypeRef attributeValues[] = { font, kCFBooleanTrue };
 		CFDictionaryRef attributes = CFDictionaryCreate(
-			kCFAllocatorDefault,
+			NULL,
 			(const void**)attributeNames,
 			(const void**)attributeValues,
 			2,
 			&kCFTypeDictionaryKeyCallBacks,
 			&kCFTypeDictionaryValueCallBacks);
-		CFAttributedStringRef attributed = CFAttributedStringCreate(kCFAllocatorDefault, textref, attributes);
+		CFAttributedStringRef attributed = CFAttributedStringCreate(NULL, textref, attributes);
 		line = CTLineCreateWithAttributedString(attributed);
 		
 		CFRelease(attributed);
@@ -90,9 +77,7 @@ void quartz_free_layout(void *layout)
 {
 	if (layout)
 		CFRelease(layout);
-};
-
-#endif
+}
 
 static bool quartz_textlayout(textspan_t *para, char **fontpath)
 {
