@@ -17,6 +17,7 @@
 
 #include "config.h"
 
+#include "../tcl-compat.h"
 #include                <sys/types.h>
 #include                <stdbool.h>
 #include                <stdint.h>
@@ -24,7 +25,6 @@
 #include                <string.h>
 
 #include <assert.h>
-#include <cgraph/list.h>
 #include <limits.h>
 #include "makecw.h"
 #include <math.h>
@@ -35,9 +35,11 @@
 #include <tcl.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
+#include <util/itos.h>
+#include <util/list.h>
 #include <util/prisize_t.h>
 
-#if ((TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 6)) || ( TCL_MAJOR_VERSION > 8)
+#if (TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION >= 6) || TCL_MAJOR_VERSION > 8
 #else
 #ifndef Tcl_GetStringResult
 #define Tcl_GetStringResult(interp) interp->result
@@ -71,8 +73,7 @@ static vgpanes_t vgpaneTable;
 
 static int polyid = 0;		/* unique and unchanging id for each poly */
 
-static poly *allocpoly(vgpane_t * vgp, int id, int npts)
-{
+static poly *allocpoly(vgpane_t *vgp, int id, Tcl_Size npts) {
     polys_append(&vgp->poly, (poly){.id = id});
     poly *rv = polys_back(&vgp->poly);
     rv->boundary.pn = 0;
@@ -225,11 +226,11 @@ static char *buildBindings(char *s1, const char *s2)
 /* convert x and y string args to point */
 static int scanpoint(Tcl_Interp * interp, const char *argv[], point * p)
 {
-    if (sscanf(argv[0], "%lg", &(p->x)) != 1) {
+    if (sscanf(argv[0], "%lg", &p->x) != 1) {
 	Tcl_AppendResult(interp, "invalid x coordinate: \"", argv[0], "\"", NULL);
 	return TCL_ERROR;
     }
-    if (sscanf(argv[1], "%lg", &(p->y)) != 1) {
+    if (sscanf(argv[1], "%lg", &p->y) != 1) {
 	Tcl_AppendResult(interp, "invalid y coordinate: \"", argv[1], "\"", NULL);
 	return TCL_ERROR;
     }
@@ -296,23 +297,19 @@ static bool remove_poly(vgpane_t *vgp, int id) {
     return false;
 }
 
-static int
-insert_poly(Tcl_Interp * interp, vgpane_t * vgp, int id, const char *vargv[],
-	    int vargc)
-{
+static int insert_poly(Tcl_Interp *interp, vgpane_t *vgp, int id,
+                       const char *vargv[], Tcl_Size vargc) {
     poly *np;
-    int i, result;
+    int result;
 
     np = allocpoly(vgp, id, vargc);
-    for (i = 0; i < vargc; i += 2) {
-	result =
-	    scanpoint(interp, &vargv[i],
-		      &(np->boundary.ps[np->boundary.pn]));
+    for (Tcl_Size i = 0; i < vargc; i += 2) {
+	result = scanpoint(interp, &vargv[i], &np->boundary.ps[np->boundary.pn]);
 	if (result != TCL_OK)
 	    return result;
 	np->boundary.pn++;
     }
-    make_CW(&(np->boundary));
+    make_CW(&np->boundary);
     vc_stale(vgp);
     return TCL_OK;
 }
@@ -419,9 +416,7 @@ vgpanecmd(ClientData clientData, Tcl_Interp * interp, int argc,
 {
     (void)clientData;
 
-    int vargc, result;
-    char vbuf[30];
-    const char **vargv;
+    int result;
     vgpane_t *vgp;
     point p, q, *ps;
     double alpha, gain;
@@ -464,20 +459,25 @@ vgpanecmd(ClientData clientData, Tcl_Interp * interp, int argc,
 	    return TCL_ERROR;
 	}
 	/* accept either inline or delimited list */
+	Tcl_Size vargc;
+	const char **vargv;
+	bool vargv_needs_free = false;
 	if (argc == 4) {
-	    result =
-		Tcl_SplitList(interp, argv[3], &vargc,
-			      (const char ***) &vargv);
+	    result = Tcl_SplitList(interp, argv[3], &vargc, &vargv);
 	    if (result != TCL_OK) {
 		return result;
 	    }
+	    vargv_needs_free = true;
 	} else {
-	    vargc = argc - 3;
+	    vargc = (Tcl_Size)argc - 3;
 	    vargv = &argv[3];
 	}
 	if (!vargc || vargc % 2) {
 	    Tcl_AppendResult(interp,
 			     "There must be a multiple of two terms in the list.", NULL);
+	    if (vargv_needs_free) {
+		Tcl_Free((char *)vargv);
+	    }
 	    return TCL_ERROR;
 	}
 
@@ -486,10 +486,17 @@ vgpanecmd(ClientData clientData, Tcl_Interp * interp, int argc,
 
 	if (!remove_poly(vgp, polyid)) {
 	    Tcl_AppendResult(interp, " no such polygon: ", argv[2], NULL);
+	    if (vargv_needs_free) {
+		Tcl_Free((char *)vargv);
+	    }
 	    return TCL_ERROR;
 	}
 
-	return insert_poly(interp, vgp, polyid, vargv, vargc);
+	result = insert_poly(interp, vgp, polyid, vargv, vargc);
+	if (vargv_needs_free) {
+	    Tcl_Free((char *)vargv);
+	}
+	return result;
 
     } else if (strcmp(argv[1], "debug") == 0) {
 	/* debug only */
@@ -515,26 +522,28 @@ vgpanecmd(ClientData clientData, Tcl_Interp * interp, int argc,
 			     " ", argv[1], " x y\"", NULL);
 	    return TCL_ERROR;
 	}
+	const char **vargv;
+	bool vargv_needs_free = false;
 	if (argc == 3) {
-	    result =
-		Tcl_SplitList(interp, argv[2], &vargc,
-			      (const char ***) &vargv);
+	    result = Tcl_SplitList(interp, argv[2], &(Tcl_Size){0}, &vargv);
 	    if (result != TCL_OK) {
 		return result;
 	    }
+	    vargv_needs_free = true;
 	} else {
-	    vargc = argc - 2;
 	    vargv = &argv[2];
 	}
 	result = scanpoint(interp, &vargv[0], &p);
+	if (vargv_needs_free) {
+	    Tcl_Free((char *)vargv);
+	}
 	if (result != TCL_OK)
 	    return result;
 
 	/* determine the polygons (if any) that contain the point */
 	for (size_t i = 0; i < polys_size(&vgp->poly); i++) {
 	    if (in_poly(polys_get(&vgp->poly, i).boundary, p)) {
-		snprintf(vbuf, sizeof(vbuf), "%d", polys_get(&vgp->poly, i).id);
-		Tcl_AppendElement(interp, vbuf);
+		Tcl_AppendElement(interp, ITOS(polys_get(&vgp->poly, i).id));
 	    }
 	}
 	return TCL_OK;
@@ -542,45 +551,51 @@ vgpanecmd(ClientData clientData, Tcl_Interp * interp, int argc,
     } else if (strcmp(argv[1], "insert") == 0) {
 	/* add poly to end poly list, and it coordinates to the end of 
 	   the point list */
-	if ((argc < 3)) {
+	if (argc < 3) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
 			     " ", argv[1], " x1 y1 x2 y2 ...\"", NULL);
 	    return TCL_ERROR;
 	}
 	/* accept either inline or delimited list */
+	Tcl_Size vargc;
+	const char **vargv;
+	bool vargv_needs_free = false;
 	if (argc == 3) {
-	    result =
-		Tcl_SplitList(interp, argv[2], &vargc,
-			      (const char ***) &vargv);
+	    result = Tcl_SplitList(interp, argv[2], &vargc, &vargv);
 	    if (result != TCL_OK) {
 		return result;
 	    }
+	    vargv_needs_free = true;
 	} else {
-	    vargc = argc - 2;
+	    vargc = (Tcl_Size)argc - 2;
 	    vargv = &argv[2];
 	}
 
 	if (!vargc || vargc % 2) {
 	    Tcl_AppendResult(interp,
 			     "There must be a multiple of two terms in the list.", NULL);
+	    if (vargv_needs_free) {
+		Tcl_Free((char *)vargv);
+	    }
 	    return TCL_ERROR;
 	}
 
 	polyid++;
 
 	result = insert_poly(interp, vgp, polyid, vargv, vargc);
+	if (vargv_needs_free) {
+	    Tcl_Free((char *)vargv);
+	}
 	if (result != TCL_OK)
 	    return result;
 
-	snprintf(vbuf, sizeof(vbuf), "%d", polyid);
-	Tcl_AppendResult(interp, vbuf, NULL);
+	Tcl_AppendResult(interp, ITOS(polyid), NULL);
 	return TCL_OK;
 
     } else if (strcmp(argv[1], "list") == 0) {
 	/* return list of polygon ids */
 	for (size_t i = 0; i < polys_size(&vgp->poly); i++) {
-	    snprintf(vbuf, sizeof(vbuf), "%d", polys_get(&vgp->poly, i).id);
-	    Tcl_AppendElement(interp, vbuf);
+	    Tcl_AppendElement(interp, ITOS(polys_get(&vgp->poly, i).id));
 	}
 	return TCL_OK;
 
@@ -592,26 +607,38 @@ vgpanecmd(ClientData clientData, Tcl_Interp * interp, int argc,
 			     " ", argv[1], " x1 y1 x2 y2\"", NULL);
 	    return TCL_ERROR;
 	}
+	Tcl_Size vargc;
+	const char **vargv;
+	bool vargv_needs_free = false;
 	if (argc == 3) {
-	    result =
-		Tcl_SplitList(interp, argv[2], &vargc,
-			      (const char ***) &vargv);
+	    result = Tcl_SplitList(interp, argv[2], &vargc, &vargv);
 	    if (result != TCL_OK) {
 		return result;
 	    }
+	    vargv_needs_free = true;
 	} else {
-	    vargc = argc - 2;
+	    vargc = (Tcl_Size)argc - 2;
 	    vargv = &argv[2];
 	}
 	if (vargc < 4) {
 	    Tcl_AppendResult(interp,
 			     "invalid points: should be: \"x1 y1 x2 y2\"", NULL);
+	    if (vargv_needs_free) {
+		Tcl_Free((char *)vargv);
+	    }
 	    return TCL_ERROR;
 	}
 	result = scanpoint(interp, &vargv[0], &p);
-	if (result != TCL_OK)
+	if (result != TCL_OK) {
+	    if (vargv_needs_free) {
+		Tcl_Free((char *)vargv);
+	    }
 	    return result;
+	}
 	result = scanpoint(interp, &vargv[2], &q);
+	if (vargv_needs_free) {
+	    Tcl_Free((char *)vargv);
+	}
 	if (result != TCL_OK)
 	    return result;
 
@@ -658,27 +685,36 @@ vgpanecmd(ClientData clientData, Tcl_Interp * interp, int argc,
 			     " ", argv[1], " x1 y1 x2 y2\"", NULL);
 	    return TCL_ERROR;
 	}
+	Tcl_Size vargc;
+	const char **vargv;
+	bool vargv_needs_free = false;
 	if (argc == 3) {
-	    result =
-		Tcl_SplitList(interp, argv[2], &vargc,
-			      (const char ***) &vargv);
+	    result = Tcl_SplitList(interp, argv[2], &vargc, &vargv);
 	    if (result != TCL_OK) {
 		return result;
 	    }
+	    vargv_needs_free = true;
 	} else {
-	    vargc = argc - 2;
+	    vargc = (Tcl_Size)argc - 2;
 	    vargv = &argv[2];
 	}
-	if ((vargc < 4)) {
+	if (vargc < 4) {
 	    Tcl_AppendResult(interp,
 			     "invalid points: should be: \"x1 y1 x2 y2\"", NULL);
 	    return TCL_ERROR;
 	}
 
 	result = scanpoint(interp, &vargv[0], &p);
-	if (result != TCL_OK)
+	if (result != TCL_OK) {
+	    if (vargv_needs_free) {
+		Tcl_Free((char *)vargv);
+	    }
 	    return result;
+	}
 	result = scanpoint(interp, &vargv[2], &q);
+	if (vargv_needs_free) {
+	    Tcl_Free((char *)vargv);
+	}
 	if (result != TCL_OK)
 	    return result;
 
